/*
 * GXMame
 *
 * 2002, 2003, Stphane Pontier <shadow_walker@users.sourceforge.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "gxmame.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "about.h"
#include "gui.h"
#include "../pixmaps-builtin/gxmame.xpm"


GtkWidget*
create_AboutWindow (void)
{
	GtkWidget *AboutWindow;
	GtkWidget *dialog_vbox2;
	GtkWidget *hbox1;
	GtkWidget *pixmap1;
	GtkWidget *vbox2;
	GtkWidget *label1;
	GtkWidget *hseparator2;
	GtkWidget *hseparator3;
	GtkWidget *label2;
	GtkWidget *label3;
	GtkWidget *dialog_action_area2;
	GtkWidget *buttonOK;
	char abouttitle[45];
	GdkPixmap *testpix = NULL;
	GdkPixmap *testmask = NULL;
	GdkColor transparent;

	AboutWindow = gtk_dialog_new ();
	gtk_object_set_data (GTK_OBJECT (AboutWindow), "AboutWindow", AboutWindow);
	g_snprintf(abouttitle, sizeof(abouttitle), _("About GXMame v%s"), VERSION);
	gtk_window_set_title (GTK_WINDOW (AboutWindow), abouttitle);
	GTK_WINDOW (AboutWindow)->type = GTK_WINDOW_DIALOG;
	gtk_window_set_position (GTK_WINDOW (AboutWindow), GTK_WIN_POS_CENTER);
	gtk_window_set_policy (GTK_WINDOW (AboutWindow), FALSE, FALSE, FALSE);

	dialog_vbox2 = GTK_DIALOG (AboutWindow)->vbox;
	gtk_object_set_data (GTK_OBJECT (AboutWindow), "dialog_vbox2", dialog_vbox2);
	gtk_widget_show (dialog_vbox2);

	hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_widget_ref (hbox1);
	gtk_object_set_data_full (GTK_OBJECT (AboutWindow), "hbox1", hbox1,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (dialog_vbox2), hbox1, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox1), 5);

	/* pixmap1 = create_pixmap (AboutWindow, "gxmame/mame.xpm"); */
	testpix = gdk_pixmap_create_from_xpm_d(MainWindow->window,
					&testmask,
					&transparent,
					gxmame_xpm);
	pixmap1 = gtk_pixmap_new (testpix,testmask);
	gtk_widget_ref (pixmap1);
	gtk_object_set_data_full (GTK_OBJECT (AboutWindow), "pixmap1", pixmap1,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (pixmap1);
	gtk_box_pack_start (GTK_BOX (hbox1), pixmap1, TRUE, TRUE, 0);
	gtk_misc_set_padding (GTK_MISC (pixmap1), 5, 0);

	vbox2 = gtk_vbox_new (FALSE, 3);
	gtk_widget_ref (vbox2);
	gtk_object_set_data_full (GTK_OBJECT (AboutWindow), "vbox2", vbox2,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox2);
	gtk_box_pack_start (GTK_BOX (hbox1), vbox2, FALSE, FALSE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 10);

	label1 = gtk_label_new (_("M.A.M.E.\nMultiple Arcade Machine Emulator\nCopyright (C) 1997-2002\n by Nicola Salmoria and The MAME Team"));
	gtk_widget_ref (label1);
	gtk_object_set_data_full (GTK_OBJECT (AboutWindow), "label1", label1,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label1);
	gtk_box_pack_start (GTK_BOX (vbox2), label1, TRUE, TRUE, 0);

	hseparator2 = gtk_hseparator_new ();
	gtk_widget_ref (hseparator2);
	gtk_object_set_data_full (GTK_OBJECT (AboutWindow), "hseparator2", hseparator2,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hseparator2);
	gtk_box_pack_start (GTK_BOX (vbox2), hseparator2, FALSE, FALSE, 0);

	label2 = gtk_label_new (_("XMame originally ported by\nAllard Van Der Bas,\nDick the Ridder,\nJuan Antonio Martinez,\nand Hans de Goede.\n\ncurrently maintained by\n Lawrence Gold"));
	gtk_widget_ref (label2);
	gtk_object_set_data_full (GTK_OBJECT (AboutWindow), "label2", label2,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label2);
	gtk_box_pack_start (GTK_BOX (vbox2), label2, TRUE, TRUE, 0);
	hseparator3 = gtk_hseparator_new ();
	gtk_widget_ref (hseparator3);
	gtk_object_set_data_full (GTK_OBJECT (AboutWindow), "hseparator3", hseparator3,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hseparator3);
	gtk_box_pack_start (GTK_BOX (vbox2), hseparator3, FALSE, FALSE, 0);

	label3 = gtk_label_new (_("GXMame (GTK Front-end for X-Mame)\nby Shadow Walker\n<shadow_walker@users.sourceforge.net>\nhttp://gxmame.sourceforge.net"));
	gtk_widget_ref (label3);
	gtk_object_set_data_full (GTK_OBJECT (AboutWindow), "label3", label3,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label3);
	gtk_box_pack_start (GTK_BOX (vbox2), label3, TRUE, TRUE, 0);


	dialog_action_area2 = GTK_DIALOG (AboutWindow)->action_area;
	gtk_object_set_data (GTK_OBJECT (AboutWindow), "dialog_action_area2", dialog_action_area2);
	gtk_widget_show (dialog_action_area2);
	gtk_container_set_border_width (GTK_CONTAINER (dialog_action_area2), 10);

	buttonOK = gtk_button_new_with_label (_("OK"));
	gtk_widget_ref (buttonOK);
	gtk_object_set_data_full (GTK_OBJECT (AboutWindow), "buttonOK", buttonOK,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (buttonOK);
	gtk_box_pack_start (GTK_BOX (dialog_action_area2), buttonOK, TRUE, FALSE, 0);
	gtk_widget_set_usize (buttonOK, 50, -2);
	GTK_WIDGET_SET_FLAGS (buttonOK, GTK_CAN_DEFAULT);

	gtk_signal_connect (GTK_OBJECT (AboutWindow), "delete_event",
			    GTK_SIGNAL_FUNC (on_AboutWindow_delete_event),
			    NULL);
	gtk_signal_connect (GTK_OBJECT (buttonOK), "clicked",
			    GTK_SIGNAL_FUNC (on_aboutButtonOK_clicked_event),
			    NULL);

	gtk_widget_grab_focus (buttonOK);
	gtk_widget_grab_default (buttonOK);
	return AboutWindow;
}

