/*
 * GXMame
 *
 * 2002, 2003, Stphane Pontier <shadow_walker@users.sourceforge.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "callbacks.h"
#include "interface.h"
#include "about.h"
#include "directories.h"
#include "mameio.h"
#include "support.h"
#include "gxmame.h"
#include "audit.h"
#include "gui.h"
#include "properties.h"
#include "progression_window.h"
#include "general_properties.h"
#include "io.h"
#include "gui_prefs.h"

static GtkWidget *AboutWindow;
guint timeoutid;
guint timeoutfoldid;
guint timeoutsearch;

gboolean delayed_select_game(gpointer game_data);
gboolean delayed_folder_selected(gpointer CTreeNode);
void select_game(RomEntry *rom);

/*Close the main window*/
gboolean
on_MainWindow_delete_event             (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	exit_gxmame();
	return TRUE;
}


/* Main window menu */
/*	File*/
void
on_play_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	play_game(gui_prefs.DefaultGame_p);
}


void
on_play_and_record_input_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *inp_selection;
	/* joystick focus turned off, will be turned on again in:
	   gui.c (select_inp) when cancelling the file selection
	   gxmame.c (record_game) */
	joy_focus_off();
	inp_selection = select_inp(gui_prefs.DefaultGame_p,FALSE);
	gtk_widget_show(inp_selection);
}


void
on_playback_input_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *inp_selection;
	/* joystick focus turned off, will be turned on again in:
	   gui.c (select_inp)
	   gxmame.c (playback_game)*/
	joy_focus_off();
	inp_selection = select_inp(gui_prefs.DefaultGame_p,TRUE);
	gtk_widget_show(inp_selection);
}


void
on_select_random_game_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{	
	GtkWidget *displayed_list;
	gint random_game;
	displayed_list = GTK_BIN(main_gui.scrolled_window_games)->child;
	/* the max should be the number of element in the current gtkCList*/
	random_game = (gint)(visible_games * ((gfloat)rand()/ RAND_MAX));
	gxmame_debug("random game#%i",random_game);
	gtk_clist_select_row(GTK_CLIST(displayed_list),random_game,0);
	gtk_clist_moveto(GTK_CLIST(displayed_list),random_game,0,0.5,0);

}


void
on_add_to_favorites_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gui_prefs.DefaultGame_p->favourite = TRUE;
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.add_to_favorites),FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.remove_from_favorites),TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.popup_add_to_favorites),FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.popup_remove_from_favorites),TRUE);
	/* problems because the row values are completly changed, I redisplay the complete game list
	update_game_in_list((gui_prefs.DefaultGame_p)->position);*/
	if (current_filter->type == FAVORITE)
		create_gamelist_content();
}


void
on_remove_from_favorites_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gui_prefs.DefaultGame_p->favourite = FALSE;
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.add_to_favorites),TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.remove_from_favorites),FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.popup_add_to_favorites),TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.popup_remove_from_favorites),FALSE);
	/* problems because the row values are completly changed, I redisplay the complete game list
	update_game_in_list((gui_prefs.DefaultGame_p)->position);*/
	if (current_filter->type == FAVORITE)
		create_gamelist_content();
}


void
on_properties_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *properties_window;
	/* have to test if a game is selected or not
	   then only after lauche the properties window*/
	/* joystick focus turned off, will be turned on again in properties.c (exit_properties_window)*/
	joy_focus_off();
	properties_window = create_properties_windows();
	gtk_widget_show(properties_window);
	set_icon(properties_window,_("Properties"));
}


void
on_audit_all_games_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gchar *message;
	if (!strcmp(executable_version.name,"Not executable"))
	{
		message = g_strdup_printf(_("%s is not a valid executable"),mame_executable);
		GXMame_error_message(ERROR, message);
		g_free(message);
	}
	else
	{
		create_checking_games_window();
		while (gtk_events_pending()) gtk_main_iteration();
		launch_checking_games_window_1();
	}
	
}


void
on_exit_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	exit_gxmame();
}


/*	View*/
void
on_toolbar_view_menu_activate          (GtkCheckMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (menuitem->active)
	{
		gxmame_debug("Show toolbar");
		show_toolbar();
	}
	else
	{
		gxmame_debug("Hide toolbar");
		hide_toolbar();
	}
}


void
on_status_bar_view_menu_activate       (GtkCheckMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (menuitem->active)
	{
		gxmame_debug("Show status bar");
		show_status_bar();
	}
	else
	{
		gxmame_debug("Hide status bar");
		hide_status_bar();
	}
}


void
on_folder_list_activate                (GtkCheckMenuItem *menuitem,
                                        gpointer         user_data)
{
	if (menuitem->active)
	{
		gxmame_debug("Show filters menu");
		/* shouldn't I block the signal to toggle button 
		   otherwise, infinite loop? 
		   it seems it's working anyway ??
		   maybe the gtk_toggle_button_set_active doesn't provoke a call
		   to on_mode_button_clicked*/
		gtk_toggle_button_set_active (main_gui.filterShowButton,TRUE);
		show_filters();
	}
	else
	{
		gxmame_debug("Hide filters menu");
		gtk_toggle_button_set_active (main_gui.filterShowButton,FALSE);
		hide_filters();
	}

}


void
on_screen_shot_activate                (GtkCheckMenuItem *menuitem,
                                        gpointer         user_data)
{
	if (menuitem->active)
	{
		gxmame_debug("Show snaps menu");
		gtk_toggle_button_set_active (main_gui.snapShowButton,TRUE);
		show_snaps();
	}
	else
	{
		gxmame_debug("Hide snaps menu");
		gtk_toggle_button_set_active (main_gui.snapShowButton,FALSE);
		hide_snaps();
	}
}


void
on_large_icons_menu_activate           (GtkCheckMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (menuitem->active)
	{
		gxmame_debug("Show large icons menu");
		gtk_toggle_button_set_active (main_gui.large_icons_view_button,TRUE);
	}
	else
	{
		gxmame_debug("Hide large icons menu");
		gtk_toggle_button_set_active (main_gui.large_icons_view_button,FALSE);
	}
}


void
on_small_icons_view_menu_activate      (GtkCheckMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (menuitem->active)
	{
		gxmame_debug("Show small icons menu");
		gtk_toggle_button_set_active (main_gui.small_icons_view_button,TRUE);
	}
	else
	{
		gxmame_debug("Hide small icons menu");
		gtk_toggle_button_set_active (main_gui.small_icons_view_button,FALSE);
	}

}


void
on_list_view_menu_activate             (GtkCheckMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (menuitem->active)
	{
		gxmame_debug("Show list menu");
		gtk_toggle_button_set_active (main_gui.list_view_button,TRUE);
	}
	else
	{
		gxmame_debug("Hide list menu");
		gtk_toggle_button_set_active (main_gui.list_view_button,FALSE);
	}

}


void
on_details_view_menu_activate          (GtkCheckMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (menuitem->active)
	{
		gxmame_debug("Show details menu");
		gtk_toggle_button_set_active (main_gui.details_view_button,TRUE);
	}
	else
	{
		gxmame_debug("Hide details menu");
		gtk_toggle_button_set_active (main_gui.details_view_button,FALSE);
	}
}

void
on_indented_view_menu_activate         (GtkCheckMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (menuitem->active)
	{
		gxmame_debug("Show indented menu");
		gtk_toggle_button_set_active (main_gui.indented_view_button,TRUE);
	}
	else
	{
		gxmame_debug("Hide indented menu");
		gtk_toggle_button_set_active (main_gui.indented_view_button,FALSE);
	}
}


void
on_the_prefix_activate                 (GtkCheckMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.modify_the_button),FALSE);
	if (menuitem->active)
	{
		gxmame_debug("Modify the menu");
		gui_prefs.ModifyThe=1;
		/* Change the default game only if one is selected*/
		if(gui_prefs.DefaultGame_p)
		{
			if (gui_prefs.DefaultGame_p ->the_trailer)
				g_snprintf(gui_prefs.DefaultGame,MAX_GAMENAME,"%s, The %s",gui_prefs.DefaultGame_p->gamename,gui_prefs.DefaultGame_p->gamenameext);
			else
				g_snprintf(gui_prefs.DefaultGame,MAX_GAMENAME,"%s %s",gui_prefs.DefaultGame_p->gamename,gui_prefs.DefaultGame_p->gamenameext);
		}
		g_strstrip(gui_prefs.DefaultGame);
		/* Need to sort the glist if sorted by name*/
		roms = g_list_sort(roms,(GCompareFunc )compare_game_name);
		create_gamelist_content();
		gtk_toggle_button_set_active (main_gui.modify_the_button,TRUE);
	}
	else
	{
		gxmame_debug("not Modify the menu");
		gui_prefs.ModifyThe=0;
		/* Change the default game only if one is selected*/
		if(gui_prefs.DefaultGame_p)
		{
			if (gui_prefs.DefaultGame_p->the_trailer)
				g_snprintf(gui_prefs.DefaultGame,MAX_GAMENAME,"The %s %s",gui_prefs.DefaultGame_p->gamename,gui_prefs.DefaultGame_p->gamenameext);
			else
				g_snprintf(gui_prefs.DefaultGame,MAX_GAMENAME,"%s %s",gui_prefs.DefaultGame_p->gamename,gui_prefs.DefaultGame_p->gamenameext);
		}
		g_strstrip(gui_prefs.DefaultGame);
		/* Need to sort the glist if sorted by name*/
		roms = g_list_sort(roms,(GCompareFunc )compare_game_name);
		create_gamelist_content();
		gtk_toggle_button_set_active (main_gui.modify_the_button,FALSE);
	}
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.modify_the_button),TRUE);
}


void
on_refresh_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	static gboolean quick_check_running;
	GList *list_pointer;
	RomEntry *rom;
	
	if(!quick_check_running)
	{	/*prevent user to launch several quick check at the same time */
		quick_check_running=1;
		/* remove all information concerning the presence of roms */
		for (list_pointer = g_list_first(roms);(list_pointer != NULL);)
		{
			rom = (RomEntry *)list_pointer->data;
			rom->has_roms = 2;
			list_pointer = g_list_next(list_pointer);
		}
		/* refresh the display */
		create_gamelist_content();
		not_checked_list = g_list_copy (roms);
	
		quick_check();
		/* final refresh*/
		create_gamelist_content();
		quick_check_running=0;
	}
	else
		gxmame_debug("Quick check already running");
}


/*	Option*/
void
color_selected  (GtkButton       *button,
                 gpointer         user_data)
{
	gdouble color[3];
	gtk_color_selection_get_color(GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(user_data)->colorsel),color);
	gui_prefs.clone_color.red=color[0] * (G_MAXSHORT-G_MINSHORT);
	gui_prefs.clone_color.green=color[1] * (G_MAXSHORT-G_MINSHORT);
	gui_prefs.clone_color.blue=color[2] * (G_MAXSHORT-G_MINSHORT);
	gtk_widget_destroy(user_data);
	create_gamelist_content();
}
void
on_clone_color_menu_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gdouble color[3];
	color[0]=(gdouble)gui_prefs.clone_color.red / (G_MAXSHORT-G_MINSHORT);
	color[1]=(gdouble)gui_prefs.clone_color.green / (G_MAXSHORT-G_MINSHORT);
	color[2]=(gdouble)gui_prefs.clone_color.blue / (G_MAXSHORT-G_MINSHORT);
	main_gui.clone_selector = gtk_color_selection_dialog_new (_("Clone color selection"));
	gtk_widget_hide(GTK_COLOR_SELECTION_DIALOG(main_gui.clone_selector)->help_button);
	gtk_window_set_transient_for(GTK_WINDOW(main_gui.clone_selector),GTK_WINDOW(MainWindow));
	gtk_window_set_modal (GTK_WINDOW (main_gui.clone_selector), TRUE);
	
	gtk_color_selection_set_color(GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(main_gui.clone_selector)->colorsel),color);
	
	gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG(main_gui.clone_selector)->ok_button), "clicked",
			    GTK_SIGNAL_FUNC (color_selected),
			    (gpointer) main_gui.clone_selector);
	gtk_signal_connect_object (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG(main_gui.clone_selector)->cancel_button), "clicked",
			    GTK_SIGNAL_FUNC (gtk_widget_destroy),
			    (gpointer) main_gui.clone_selector);
	gtk_signal_connect (GTK_OBJECT (main_gui.clone_selector), "delete_event",
			    GTK_SIGNAL_FUNC (gtk_widget_destroy),
			    NULL);
	gtk_widget_show(main_gui.clone_selector);
}

void
font_selected  (GtkButton       *button,
                 gpointer         user_data)
{	
	GtkStyle *liststyle;
	if ((gui_prefs.ListFont = gtk_font_selection_dialog_get_font_name (user_data)))
	{/*got a font*/
		if (!gdk_font_load(gui_prefs.ListFont)) {
			gxmame_debug("Could not find the font.");
			gui_prefs.ListFontStruct = NULL;
			if (gui_prefs.ListFontHeight != 16)
				dirty_icon_cache = TRUE;
			gui_prefs.ListFontHeight = 16;
		}
		else
		{
			gint ascent,descent,lbearing,rbearing,temp_height;
			gui_prefs.ListFontStruct = gdk_font_load(gui_prefs.ListFont);
			gdk_string_extents(gui_prefs.ListFontStruct,
 		                                     "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz",
 		                                     &lbearing,
 		                                     &rbearing,
 		                                     NULL,
 		                                     &ascent,
 		                                     &descent);
			temp_height = descent + gdk_string_height(gui_prefs.ListFontStruct,
					               "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz");
			if (temp_height != gui_prefs.ListFontHeight)
				dirty_icon_cache = TRUE;
			gui_prefs.ListFontHeight = temp_height;
			printf("sizing: lbearing:%i rbearing:%i ascent:%i descent:%i height:%i\n",lbearing, rbearing, ascent, descent,gui_prefs.ListFontHeight);
		}
		/* redraw all if the font height change or just change the widget style */
		if (dirty_icon_cache)
		{	
			create_filtersTreeContent();
			create_gamelist_content();
		}
		else
		{
			liststyle = gtk_style_copy(gtk_widget_get_style(GTK_WIDGET(main_gui.displayed_list)));
			if (liststyle == NULL) {
				gtk_style_unref(liststyle);
				liststyle = gtk_style_new();
				gxmame_debug("creating new style");
			}
			liststyle->font = gui_prefs.ListFontStruct;
			gtk_widget_set_style(GTK_WIDGET(main_gui.displayed_list), liststyle);
			liststyle = gtk_style_copy(gtk_widget_get_style(GTK_WIDGET(main_gui.ctree_filters)));
			liststyle->font = gui_prefs.ListFontStruct;
			gtk_widget_set_style(GTK_WIDGET(main_gui.ctree_filters), liststyle);
		}
	}
	else
	{
		gui_prefs.ListFontStruct = NULL;
		if (gui_prefs.ListFontHeight != 16)
			dirty_icon_cache = TRUE;
		gui_prefs.ListFontHeight = 16;		
	}
	gtk_widget_destroy(user_data);
	
}
void
on_game_list_font_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *fontselector;

	fontselector = gtk_font_selection_dialog_new(_("Game list Font"));
	gtk_window_set_transient_for(GTK_WINDOW(fontselector),GTK_WINDOW(MainWindow));
	gtk_window_set_modal (GTK_WINDOW (fontselector), TRUE);
	
	if (gui_prefs.ListFont)
		gtk_font_selection_dialog_set_font_name (GTK_FONT_SELECTION_DIALOG(fontselector),gui_prefs.ListFont);
	
	gtk_signal_connect (GTK_OBJECT (GTK_FONT_SELECTION_DIALOG(fontselector)->ok_button), "clicked",
			    GTK_SIGNAL_FUNC (font_selected),
			    (gpointer) fontselector);
	gtk_signal_connect_object (GTK_OBJECT (GTK_FONT_SELECTION_DIALOG(fontselector)->cancel_button), "clicked",
			    GTK_SIGNAL_FUNC (gtk_widget_destroy),
			    (gpointer) fontselector);
	gtk_signal_connect (GTK_OBJECT (fontselector), "delete_event",
			    GTK_SIGNAL_FUNC (gtk_widget_destroy),
			    NULL);
	gtk_widget_show(fontselector);
}


void
on_rebuild_game_list_menu_activate     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gxmame_debug("freeing glists");
	g_list_foreach(roms, (GFunc)g_free, NULL);
	g_list_free (roms);
	roms = NULL;
	/* free manufacturers and years list */
	g_list_foreach(manufacturers, (GFunc)g_free, NULL);
	g_list_free (manufacturers);
	manufacturers = NULL;
	g_list_free (years);
	years = NULL;
	g_list_foreach(categories, (GFunc)g_free, NULL);
	g_list_free (categories);
	categories = NULL;
	g_list_foreach(versions, (GFunc)g_free, NULL);
	g_list_free (versions);
	versions = NULL;
	
	/* I thought it could free my memory but nope*/
	while (gtk_events_pending()) gtk_main_iteration();
	g_blow_chunks();
	while (gtk_events_pending()) gtk_main_iteration();
	
	/* just to show an empty list */
	create_gamelist_content();
	
	/* reload them */
	/* free category and version list */
	/* reload them */
	gxmame_debug("recreate game list");
	create_game_list_file();
	gxmame_debug("reload everything");
	load_games_list();
	load_games_ini();
	load_catver_ini();
	create_filtersTreeContent();
	create_gamelist_content();
}


void
on_directories_menu_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *directory_window;
	directory_window = create_directories_selection();
	gtk_widget_show(directory_window);
	set_icon(directory_window,_("Directories selection"));
}


void
on_default_option_menu_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *general_properties_window;
	/* have to test if a game is selected or not
	   then only after lauche the properties window*/
	general_properties_window = create_general_properties_windows();
	gtk_widget_show(general_properties_window);
	set_icon(general_properties_window,_("General Properties"));
}


void
on_startup_option_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *gui_prefs_window;
	gui_prefs_window = create_gui_prefs_window();
	gtk_widget_show(gui_prefs_window);
	set_icon(gui_prefs_window,_("Startup Options"));
}


/*	Help*/
void
on_about_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (AboutWindow)
	{
		gdk_window_raise(AboutWindow->window);
	}
	else
	{
		AboutWindow = create_AboutWindow ();
		gtk_widget_show(AboutWindow);
		set_icon(AboutWindow, _("About"));	
	}
  
}


/*About window*/
gboolean
on_AboutWindow_delete_event            (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_widget_hide(AboutWindow);
  gtk_widget_destroy(AboutWindow);
  AboutWindow = NULL;
  return FALSE;
}

void
on_aboutButtonOK_clicked_event         (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(AboutWindow);
	AboutWindow = NULL;
}


/* Toolbox menu*/
void
on_filterShowButton_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton))
	{
		gxmame_debug("Show filters");
		/* the on_folder_list_activate callback will call show_filter()*/
		gtk_check_menu_item_set_active(main_gui.folder_list_menu,TRUE);
	}
	else
	{
		gxmame_debug("Hide filters");
		/* the on_folder_list_activate callback will call hide_filter()*/
		gtk_check_menu_item_set_active(main_gui.folder_list_menu,FALSE);
	}
}


void
on_snapShowButton_toggled              (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton))
	{
		gxmame_debug("Show snaps");
		gtk_check_menu_item_set_active(main_gui.screen_shot_menu,TRUE);
	}
	else
	{
		gxmame_debug("Hide snaps");
		gtk_check_menu_item_set_active(main_gui.screen_shot_menu,FALSE);
	}
}


void
on_mode_button_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)))
	{
		if (gui_prefs.current_mode!=GPOINTER_TO_INT(user_data))
		{
			gui_prefs.previous_mode=gui_prefs.current_mode;
			gui_prefs.current_mode=GPOINTER_TO_INT(user_data);
			gxmame_debug("current mode changed to:%d,previous changed to :%d",gui_prefs.current_mode,gui_prefs.previous_mode);
			switch(gui_prefs.current_mode)
			{
				case(LARGE_ICONS):
					gtk_check_menu_item_set_active(main_gui.large_icons_view_menu,TRUE);
					break;
				case(SMALL_ICONS):
					gtk_check_menu_item_set_active(main_gui.small_icons_view_menu,TRUE);
					break;
				case(LIST):
					gtk_check_menu_item_set_active(main_gui.list_view_menu,TRUE);
					break;
				case(DETAILS):
					gtk_check_menu_item_set_active(main_gui.details_view_menu,TRUE);
					break;
				case(INDENTED):
					gtk_check_menu_item_set_active(main_gui.indented_view_menu,TRUE);
					break;
			}
			switch(gui_prefs.previous_mode)
			{
				case(LARGE_ICONS):
					gtk_toggle_button_set_active(main_gui.large_icons_view_button,FALSE);
					break;
				case(SMALL_ICONS):
					gtk_toggle_button_set_active(main_gui.small_icons_view_button,FALSE);
					break;
				case(LIST):
					gtk_toggle_button_set_active(main_gui.list_view_button,FALSE);
					break;
				case(DETAILS):
					gtk_toggle_button_set_active(main_gui.details_view_button,FALSE);
					break;
				case(INDENTED):
					gtk_toggle_button_set_active(main_gui.indented_view_button,FALSE);
					break;
			}
			create_gamelist_content();
		}		
	}
	else
	{
		if (gui_prefs.current_mode==GPOINTER_TO_INT(user_data))
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),TRUE);
	}
}


void
on_ModifyThe_toggled                   (GtkToggleButton *button,
                                        gpointer         user_data)
{
	gtk_widget_set_sensitive(GTK_WIDGET(button),FALSE);
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)))
	{
		gxmame_debug("Modify the");
		gtk_check_menu_item_set_active(main_gui.modify_the_menu,TRUE);
	}
	else
	{
		gxmame_debug("not Modify the");
		gtk_check_menu_item_set_active(main_gui.modify_the_menu,FALSE);
	}
	gtk_widget_set_sensitive(GTK_WIDGET(button),TRUE);
}


/* Filter box */
void
on_folder_selected                     (GtkCTree        *CTree,
                                        GtkCTreeNode    *CTreeNode,
                                        gint            *column,
                                        gpointer         user_data)
{
		if (timeoutfoldid)
			gtk_timeout_remove(timeoutfoldid);
		timeoutfoldid =
		    gtk_timeout_add(SELECT_TIMEOUT,
				    (GtkFunction) delayed_folder_selected, CTreeNode);
}

gboolean
delayed_folder_selected                (gpointer         CTreeNode)
{
	simple_filter *current_folder_filter;
	current_folder_filter = (simple_filter *)gtk_ctree_node_get_row_data(main_gui.ctree_filters,CTreeNode);
	if (!(current_folder_filter->FolderID == MANUFACTURERS ||
	      current_folder_filter->FolderID == YEARS ||
	      current_folder_filter->FolderID == CUSTOMS ||
	      current_folder_filter->FolderID == CATEGORIES ||
	      current_folder_filter->FolderID == VERSIONS ||
	      current_folder_filter->FolderID == gui_prefs.FolderID))
	{
		gtk_ctree_expand(main_gui.ctree_filters,CTreeNode);
		current_filter = current_folder_filter;
		gui_prefs.previous_mode = gui_prefs.current_mode;
		gxmame_debug("filter:%i selected",current_folder_filter->FolderID);
		create_gamelist_content();
	}
	gui_prefs.FolderID = current_folder_filter->FolderID;
	return FALSE;
}


void
on_folder_unselected                   (GtkCTree        *CTree,
                                        GtkCTreeNode    *CTreeNode,
                                        gint            *column,
                                        gpointer         user_data)
{
	simple_filter *current_folder_filter;
	current_folder_filter = (simple_filter *)gtk_ctree_node_get_row_data(CTree,CTreeNode);
	if (!(current_folder_filter->FolderID == MANUFACTURERS ||
	      current_folder_filter->FolderID == YEARS ||
	      current_folder_filter->FolderID == CUSTOMS ||
	      current_folder_filter->FolderID == CATEGORIES ||
	      current_folder_filter->FolderID == VERSIONS))
	{
		gtk_ctree_collapse(CTree,CTreeNode);
	}
}


/* Main list */
void
on_row_selected                       (GtkCList        *CList,
                                        gint             game,
                                        gint             column,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gchar *message[1];
	RomEntry *game_data;
	game_data = gtk_clist_get_row_data(CList,game);
	gtk_clist_get_pixtext (CList,game,0, message,NULL,NULL,NULL);
	/* dont need anymore to test the 3rd button, since only button 1 select the game */
	
	if (event && event->type == GDK_2BUTTON_PRESS) {
		if (event->button == 1) {
			gxmame_debug("double clic on: %s",message[0]);
			play_game(game_data);
		}
	} else if (event && event->type == GDK_BUTTON_PRESS) {
		/* select the game only if it wasn't the previously selected one  (prevent screenshot flickers)*/
		if((game_data != gui_prefs.DefaultGame_p)) {
			gxmame_debug("game %s selected (row# %d)",message[0],game);
			select_game(game_data);
		}
	}else{
		if(main_gui.displayed_list)
		{
			GTK_CLIST(main_gui.displayed_list)->focus_row = game_data->position;
    		/*	if (GTK_CLIST(main_gui.displayed_list)->freeze_count == 0)
      		/		gtk_widget_draw (GTK_WIDGET (main_gui.displayed_list), NULL);*/
			gtk_widget_queue_draw (GTK_WIDGET (main_gui.displayed_list));
		}
		/* Call rom_selected after a timeout */
		if (timeoutid)
			gtk_timeout_remove(timeoutid);
		timeoutid =
		    gtk_timeout_add(SELECT_TIMEOUT,
				    (GtkFunction) delayed_select_game, game_data);
	}
}

gboolean
on_row_right_clic                      (GtkCList        *CList,
                                        GdkEventButton  *event)
{
	gchar *message[1];
	RomEntry *game_data;
	gint position_x,position_y;
	if (event && event->type == GDK_BUTTON_PRESS && event->button == 3)
	{
		if (gtk_clist_get_selection_info(CList,event->x,event->y,&position_x,&position_y))
		{
			gtk_clist_get_pixtext (CList,position_x,0, message,NULL,NULL,NULL);
			/* prevent the game to be selected twice */
			/* I need to select the game position and block the on_row_selected handler
			   otherwise, the menu will popup and only then change the popup content
			   ie the name of the game */
			gtk_signal_handler_block_by_func(GTK_OBJECT(CList),on_row_selected,NULL);
			game_data = gtk_clist_get_row_data(CList,position_x);
			gtk_clist_select_row(CList,position_x,0);
			/* select the game only if it wasn't the previously selected one  (prevent screenshot flickers)*/
			if(game_data != gui_prefs.DefaultGame_p)
				select_game(game_data);
			gxmame_debug("game %s selected (row# %d)(right button)",message[0],position_x);
			gtk_signal_handler_unblock_by_func(GTK_OBJECT(CList),on_row_selected,NULL);
		}
		gtk_menu_popup(main_gui.popup_menu,NULL,NULL,
				NULL,NULL,event->button,event->time);
	}
	return FALSE;
}


gboolean delayed_select_game(gpointer game_data)
{
	select_game(game_data);
	gtk_timeout_remove(timeoutid);
	return FALSE;
}


void
update_ctree_filter_width              (GtkCTree        *ctree,
                                        GList           *node,
                                        gpointer         user_data)
{
	gtk_clist_columns_autosize(GTK_CLIST(ctree));
	gtk_widget_queue_draw (GTK_WIDGET(ctree));
}


/* Used to detect if key was pressed in the main list 
   that prevent key release detection from another window, dialog box or popup*/
gboolean
on_displayed_list_key_press_event    (GtkWidget	*widget,
					GdkEventKey	*event,
					gpointer	 user_data)
{
	displayedlist_keypressed = TRUE;
	return FALSE;
}


gboolean
on_displayed_list_key_release_event    (GtkWidget       *widget,
				        GdkEventKey     *event,
		                        gpointer         user_data)
{
	gchar *search_char;
	guint pos;
	static gchar search_string[MAX_GAMENAME]="";
	/* if the kay was not pressed inside the main list, do nothing */
	if (!displayedlist_keypressed)
	{
		return FALSE;
	}
	displayedlist_keypressed = FALSE;
	/* Filter out modifiers */
	if ((visible_games > 0) && (!event->state || (event->state & GDK_SHIFT_MASK) ||	/* Shift 	*/
						     (event->state & GDK_LOCK_MASK) ||	/* Caps lock 	*/
						     (event->state & GDK_MOD2_MASK) ||	/* Num lock 	*/
						     (event->state & GDK_MOD5_MASK))) {	/* Scroll lock 	*/

		switch (event->keyval) {
		case GDK_Menu:
		gtk_menu_popup(main_gui.popup_menu,NULL,NULL,
				NULL,NULL,0,0);
			break;
		case GDK_KP_Enter:
			break;
		case GDK_Return:
			break;
		case GDK_Page_Up:
			break;
		case GDK_Page_Down:
			break;
		case GDK_Up:
			break;
		case GDK_Down:
			break;
		case GDK_BackSpace:
			break;
		default:
			if ((((event->keyval) > 0x02f) & ((event->keyval) <
							  0x07b))
			    || (event->keyval == GDK_space)) {
				search_char =
				    g_strdup(gdk_keyval_name
					     (event->keyval));

				if (!strcmp(search_char, "space")) {
					g_free(search_char);
					search_char = g_strdup(" ");
				}
				pos = strlen(search_string);
				if (pos >= MAX_GAMENAME)
					pos = 0;
				search_string[pos] = search_char[0];
				search_string[pos+1] = '\0';
				g_free(search_char);
				gxmame_debug("pos: %d - search for %s",strlen(search_string),search_string);
				lookup_game(search_string);

				if (timeoutsearch)
					gtk_timeout_remove(timeoutsearch);
				timeoutsearch =
				    gtk_timeout_add(SEARCH_TIMEOUT,
						    (GtkFunction) remove_search, search_string);
			}
			break;
		}
	}
	return FALSE;
}


/* misc */
gboolean remove_search(gchar *search_string)
{
	search_string[0]= '\0',
	gtk_timeout_remove(timeoutsearch);
	return FALSE;
}




