/*
 * GXMame
 *
 * 2002, 2003, Stphane Pontier <shadow_walker@users.sourceforge.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "gxmame.h"
#include "gui.h"
#include "mameio.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "directories.h"
#include "progression_window.h"

GtkWidget *directories_selection;
GList *xmame_execs_path_list = NULL;
GList *roms_path_list = NULL;
GList *samples_path_list = NULL;
gpointer the_current_entry;

GtkWidget *xmame_execs_path_entry;
GtkWidget *xmame_execs_path_gtk_list;

GtkWidget *roms_path_entry;
GtkWidget *roms_path_gtk_list;

GtkWidget *cfg_directory_entry;
GtkWidget *nvram_directory_entry;
GtkWidget *memcard_directory_entry;
GtkWidget *state_directory_entry;
GtkWidget *ctrlr_directory_entry;
GtkWidget *inipath_entry;
GtkWidget *diffdir_path_entry;

GtkWidget *samples_path_entry;
GtkWidget *samples_path_gtk_list;

GtkWidget *snapshot_path_entry;
GtkWidget *artwork_path_entry;
GtkWidget *input_path_entry;
GtkWidget *cheatfile_path_entry;
GtkWidget *hiscorefile_path_entry;
GtkWidget *historyfile_path_entry;
GtkWidget *mameinfofile_path_entry;
GtkWidget *hiscore_path_entry;
GtkWidget *xmame_exe_path_entry;
GtkWidget *common_path_entry;
GtkWidget *current_sample_list_item;
GtkWidget *current_rom_list_item;
GtkWidget *current_xmame_execs_list_item;

GtkWidget *flyer_path_entry;
GtkWidget *cabinet_path_entry;
GtkWidget *marquee_path_entry;
GtkWidget *title_path_entry;
GtkWidget *icons_path_entry;

GtkWidget *directories_selection_apply_button;

GtkWidget*
create_folderselection (gpointer current_entry,gboolean directory_selection_box)
{
	GtkWidget *folderselection;
	GtkWidget *ok_button_folderselection;
	GtkWidget *cancel_button_folderselection;
	gchar *temp_text;

	the_current_entry = current_entry;
	if (directory_selection_box)
		folderselection = gtk_file_selection_new (_("Browse for Folder"));
	else
		folderselection = gtk_file_selection_new (_("Browse for File"));
	gtk_object_set_data (GTK_OBJECT (folderselection), "folderselection", folderselection);
	gtk_container_set_border_width (GTK_CONTAINER (folderselection), 10);
	GTK_WINDOW (folderselection)->type = GTK_WINDOW_DIALOG;
	gtk_window_set_position (GTK_WINDOW (folderselection), GTK_WIN_POS_MOUSE);
	gtk_window_set_modal (GTK_WINDOW (folderselection), TRUE);
	gtk_window_set_policy (GTK_WINDOW (folderselection), FALSE, FALSE, FALSE);
	gtk_window_set_transient_for(GTK_WINDOW(folderselection),GTK_WINDOW(directories_selection));
	gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (folderselection));
	if (directory_selection_box)
		gtk_widget_hide (GTK_WIDGET (GTK_FILE_SELECTION(folderselection)->file_list)->parent);
	
	temp_text = gtk_editable_get_chars (GTK_EDITABLE(current_entry),0,-1);
	gtk_file_selection_set_filename(GTK_FILE_SELECTION(folderselection),temp_text);
	g_free(temp_text);
	ok_button_folderselection = GTK_FILE_SELECTION (folderselection)->ok_button;
	gtk_object_set_data (GTK_OBJECT (folderselection), "ok_button_folderselection", ok_button_folderselection);
	gtk_widget_show (ok_button_folderselection);
	GTK_WIDGET_SET_FLAGS (ok_button_folderselection, GTK_CAN_DEFAULT);

	cancel_button_folderselection = GTK_FILE_SELECTION (folderselection)->cancel_button;
	gtk_object_set_data (GTK_OBJECT (folderselection), "cancel_button_folderselection", cancel_button_folderselection);
	gtk_widget_show (cancel_button_folderselection);
	GTK_WIDGET_SET_FLAGS (cancel_button_folderselection, GTK_CAN_DEFAULT);

	gtk_signal_connect (GTK_OBJECT (ok_button_folderselection), "clicked",
			    GTK_SIGNAL_FUNC (on_ok_button_folderselection_clicked),
			    (gpointer) folderselection);
	gtk_signal_connect (GTK_OBJECT (cancel_button_folderselection), "clicked",
			    GTK_SIGNAL_FUNC (on_cancel_button_folderselection_clicked),
			    (gpointer) folderselection);
	
	gtk_widget_show(folderselection);
	if (directory_selection_box)
		set_icon(folderselection,_("Folder Selection"));
	else
		set_icon(folderselection,_("File Selection"));
	return folderselection;
}
GtkWidget*
create_directories_selection (void)
{
	GtkWidget *main_buttons_hbox;
	GtkWidget *directories_notebook;
	GtkWidget *gxmame_tab_label;
	GtkWidget *xmame_tab_label1;
	GtkWidget *xmame_tab_label2;
	GtkWidget *main_vbox;
	GtkWidget *gxmame_directory_frame;
	GtkWidget *gxmame_directory_table;
	GtkWidget *xmame_directory_frame;
	GtkWidget *xmame_directory_table;
	GtkWidget *xmame_addit_path_frame;
	GtkWidget *xmame_addit_path_table;
	GtkWidget *xmame_user_path_frame;
	GtkWidget *xmame_user_path_table;
	
	/* GXMame */
	GtkWidget *common_browse_button;
	GtkWidget *catver_path_label;
	GtkWidget *xmame_execs_path_entry_vbox;
	GtkWidget *xmame_execs_path_scrolledwindow;
	GtkWidget *xmame_execs_path_viewport;
	GtkWidget *xmame_execs_label;
	GtkWidget *xmame_execs_path_button_vbox;
	GtkWidget *xmame_execs_browse_button;
	GtkWidget *xmame_execs_add_button;
	GtkWidget *xmame_execs_remove_button;
	GtkWidget *icons_browse_button;
	GtkWidget *icons_label;
	GtkWidget *flyer_label;
	GtkWidget *flyer_browse_button;
	GtkWidget *cabinet_label;
	GtkWidget *cabinet_browse_button;
	GtkWidget *marquee_label;
	GtkWidget *marquee_browse_button;
	GtkWidget *title_label;
	GtkWidget *title_browse_button;
	
	/* XMame basic */
	GtkWidget *rompath_entry_vbox;
	GtkWidget *rompath_scrolledwindow;
	GtkWidget *rompath_viewport;
	GtkWidget *rompath_label;
	GtkWidget *rompath_button_vbox;
	GtkWidget *roms_browse_button;
	GtkWidget *roms_add_button;
	GtkWidget *roms_remove_button;
	GtkWidget *samplepath_entry_vbox;
	GtkWidget *samplepath_scrolledwindow;
	GtkWidget *samplepath_button_vbox;
	GtkWidget *samples_browse_button;
	GtkWidget *samples_add_button;
	GtkWidget *samples_remove_button;
	GtkWidget *samplepath_viewport;
	GtkWidget *sample_label;
	GtkWidget *diffdirs_browse_button;
	GtkWidget *diffdir_label;
	GtkWidget *artwork_label;
	GtkWidget *artwork_browse_button;
	
	/* XMame additional */
	GtkWidget *snapshot_label;
	GtkWidget *snapshots_browse_button;
	GtkWidget *historyfile_label;
	GtkWidget *historyfile_browse_button;
	GtkWidget *mameinfofile_label;
	GtkWidget *mameinfofile_browse_button;
	
	GtkWidget *cheatfile_label;
	GtkWidget *cheatfile_browse_button;
	GtkWidget *hiscorefile_label;
	GtkWidget *hiscorefile_browse_button;
	GtkWidget *ctrlr_directory_label;
	GtkWidget *ctrlr_directory_browse_button;
	
	/* User resources */
	GtkWidget *hiscore_label;
	GtkWidget *hiscore_browse_button;
	GtkWidget *input_label;
	GtkWidget *input_browse_button;
	GtkWidget *cfg_directory_label;
	GtkWidget *cfg_directory_browse_button;
	GtkWidget *nvram_directory_label;
	GtkWidget *nvram_directory_browse_button;
	GtkWidget *memcard_directory_label;
	GtkWidget *memcard_directory_browse_button;
	GtkWidget *state_directory_label;
	GtkWidget *state_directory_browse_button;
	GtkWidget *inipath_label;
	GtkWidget *inipath_browse_button;
	
	
	GtkWidget *vseparator1;
	GtkWidget *vseparator2;
	/*GtkWidget *hseparator1;*/
	guint directories_selection_apply_button_key;
	guint directories_selection_ok_button_key;
	guint directories_selection_cancel_button_key;
	
	GtkWidget *directories_selection_cancel_button;
	GtkWidget *directories_selection_ok_button;
	gint i;
	GtkAccelGroup *accel_group;
	

	accel_group = gtk_accel_group_new ();

	if (roms_path_list == NULL)
	{
		for (i=0;gui_prefs.RomPath[i]!=NULL;i++)
		{
			roms_path_list = g_list_append (roms_path_list,gtk_list_item_new_with_label(gui_prefs.RomPath[i]));
		}
	}
	if (samples_path_list == NULL)
	{
		for (i=0;gui_prefs.SamplePath[i]!=NULL;i++)
		{
			samples_path_list = g_list_append (samples_path_list,gtk_list_item_new_with_label(gui_prefs.SamplePath[i]));
		}
	}
	if (xmame_execs_path_list == NULL)
	{
		for (i=0;xmame_executables_array[i]!=NULL;i++)
		{
			xmame_execs_path_list = g_list_append (xmame_execs_path_list,gtk_list_item_new_with_label(xmame_executables_array[i]));
		}
	}
	

	directories_selection = gtk_window_new (GTK_WINDOW_DIALOG);
	gtk_object_set_data (GTK_OBJECT (directories_selection), "directories_selection", directories_selection);
	gtk_window_set_title (GTK_WINDOW (directories_selection), _("Directories Selection"));
	gtk_window_set_transient_for(GTK_WINDOW(directories_selection),GTK_WINDOW(MainWindow));
	gtk_widget_set_usize(directories_selection, 600,400);
	gtk_window_set_modal(GTK_WINDOW(directories_selection),TRUE);

			
	main_vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (main_vbox);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "main_vbox", main_vbox,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (main_vbox);
	gtk_container_add (GTK_CONTAINER (directories_selection), main_vbox);

	directories_notebook = gtk_notebook_new ();
	gtk_widget_ref (directories_notebook);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "directories_notebook", directories_notebook,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (directories_notebook);
	gtk_box_pack_start (GTK_BOX (main_vbox), directories_notebook, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (directories_notebook), 8);
	
	gxmame_directory_frame = gtk_frame_new (_("Path to xmame and various support files"));
	gtk_widget_ref (gxmame_directory_frame);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "gxmame_directory_frame", gxmame_directory_frame,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (gxmame_directory_frame);
	gtk_container_set_border_width (GTK_CONTAINER (gxmame_directory_frame), 5);

	gxmame_tab_label = gtk_label_new (_("GXMame Directories"));
	gtk_widget_ref (gxmame_tab_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "gxmame_tab_label", gxmame_tab_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (gxmame_tab_label);
	
	gtk_notebook_append_page        (GTK_NOTEBOOK(directories_notebook),
                                             gxmame_directory_frame,
                                             gxmame_tab_label);
	
	gxmame_directory_table = gtk_table_new (3, 3, FALSE);
	gtk_widget_ref (gxmame_directory_table);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "gxmame_directory_table", gxmame_directory_table,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (gxmame_directory_table);
	gtk_container_add (GTK_CONTAINER (gxmame_directory_frame), gxmame_directory_table);
	gtk_container_set_border_width (GTK_CONTAINER (gxmame_directory_table), 2);
	gtk_table_set_row_spacings (GTK_TABLE (gxmame_directory_table), 5);
	gtk_table_set_col_spacings (GTK_TABLE (gxmame_directory_table), 5);

/*	xmame_title_frame_label = gtk_label_new (_("Path to xmame and various config files"));
	gtk_widget_ref (xmame_title_frame_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_title_frame_label", xmame_title_frame_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (xmame_title_frame_label);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), xmame_title_frame_label, 0, 6, 0, 1,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (xmame_title_frame_label), 0.5, 0.5);
*/	

/* Xmame executables Path */
	xmame_execs_label = gtk_label_new (_("Xmame Executables"));
	gtk_widget_ref (xmame_execs_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_execs_label", xmame_execs_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (xmame_execs_label);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), xmame_execs_label, 0, 1, 0, 3,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (xmame_execs_label), 0, 0);
	gtk_misc_set_padding (GTK_MISC (xmame_execs_label), 0, 5);

	xmame_execs_path_entry_vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (xmame_execs_path_entry_vbox);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_execs_path_entry_vbox", xmame_execs_path_entry_vbox,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (xmame_execs_path_entry_vbox);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), xmame_execs_path_entry_vbox, 1, 2, 0, 3,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

	xmame_execs_path_entry = gtk_entry_new ();
	gtk_widget_ref (xmame_execs_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_execs_path_entry", xmame_execs_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_box_pack_start (GTK_BOX (xmame_execs_path_entry_vbox), xmame_execs_path_entry, FALSE, FALSE, 0);
	gtk_widget_show (xmame_execs_path_entry);

	xmame_execs_path_scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_ref (xmame_execs_path_scrolledwindow);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_execs_path_scrolledwindow", xmame_execs_path_scrolledwindow,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (xmame_execs_path_scrolledwindow);
	gtk_box_pack_start (GTK_BOX (xmame_execs_path_entry_vbox), xmame_execs_path_scrolledwindow, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (xmame_execs_path_scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	xmame_execs_path_viewport = gtk_viewport_new (NULL, NULL);
	gtk_widget_ref (xmame_execs_path_viewport);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_execs_path_viewport", xmame_execs_path_viewport,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (xmame_execs_path_viewport);
	gtk_container_add (GTK_CONTAINER (xmame_execs_path_scrolledwindow), xmame_execs_path_viewport);

	xmame_execs_path_gtk_list = gtk_list_new ();
	gtk_widget_ref (xmame_execs_path_gtk_list);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_execs_path_gtk_list", xmame_execs_path_gtk_list,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_list_append_items(GTK_LIST(xmame_execs_path_gtk_list) ,xmame_execs_path_list);
	gtk_widget_show_all (xmame_execs_path_gtk_list);
	gtk_container_add (GTK_CONTAINER (xmame_execs_path_viewport), xmame_execs_path_gtk_list);

	xmame_execs_path_button_vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (xmame_execs_path_button_vbox);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_execs_path_button_vbox", xmame_execs_path_button_vbox,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (xmame_execs_path_button_vbox);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), xmame_execs_path_button_vbox, 2, 3, 0, 3,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (GTK_SHRINK | GTK_FILL), 0, 0);

	xmame_execs_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (xmame_execs_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_execs_browse_button", xmame_execs_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (xmame_execs_browse_button);
	gtk_box_pack_start (GTK_BOX (xmame_execs_path_button_vbox), xmame_execs_browse_button, FALSE, FALSE, 2);

	xmame_execs_add_button = gtk_button_new_with_label (_("Add"));
	gtk_widget_ref (xmame_execs_add_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_execs_add_button", xmame_execs_add_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (xmame_execs_add_button);
	gtk_box_pack_start (GTK_BOX (xmame_execs_path_button_vbox), xmame_execs_add_button, FALSE, FALSE, 2);

	xmame_execs_remove_button = gtk_button_new_with_label (_("Remove"));
	gtk_widget_ref (xmame_execs_remove_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_execs_remove_button", xmame_execs_remove_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (xmame_execs_remove_button);
	gtk_box_pack_start (GTK_BOX (xmame_execs_path_button_vbox), xmame_execs_remove_button, FALSE, FALSE, 2);

/* common path (catver)*/
	catver_path_label = gtk_label_new (_("catver path"));
	gtk_widget_ref (catver_path_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "catver_path_label", catver_path_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (catver_path_label);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), catver_path_label, 0, 1, 3, 4,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (catver_path_label), 0, 0.5);

	common_path_entry = gtk_entry_new ();
	gtk_widget_ref (common_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "common_path_entry", common_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (common_path_entry), gui_prefs.catverDirectory);
	gtk_widget_show (common_path_entry);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), common_path_entry, 1, 2, 3, 4,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	common_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (common_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "common_browse_button", common_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (common_browse_button);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), common_browse_button, 2, 3, 3, 4,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);


/* separators */	
	vseparator2 = gtk_hseparator_new();
	gtk_widget_show(vseparator2);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), vseparator2, 0, 3, 4, 5,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (GTK_FILL), 0, 0);
	
/* Flyers directory */
	flyer_label = gtk_label_new (_("Flyers Path"));
	gtk_widget_ref (flyer_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "flyer_label", flyer_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (flyer_label);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), flyer_label, 0, 1, 5, 6,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (flyer_label), 0, 0.5);

	flyer_path_entry = gtk_entry_new ();
	gtk_widget_ref (flyer_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "flyer_path_entry", flyer_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (flyer_path_entry), gui_prefs.FlyerDirectroy);
	gtk_widget_show (flyer_path_entry);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), flyer_path_entry, 1, 2, 5, 6,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	flyer_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (flyer_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "flyer_browse_button", flyer_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (flyer_browse_button);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), flyer_browse_button, 2, 3, 5, 6,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

/* Cabinets directory */
	cabinet_label = gtk_label_new (_("Cabinets Path"));
	gtk_widget_ref (cabinet_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "cabinet_label", cabinet_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (cabinet_label);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), cabinet_label, 0, 1, 6, 7,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (cabinet_label), 0, 0.5);

	cabinet_path_entry = gtk_entry_new ();
	gtk_widget_ref (cabinet_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "cabinet_path_entry", cabinet_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (cabinet_path_entry), gui_prefs.CabinetDirectory);
	gtk_widget_show (cabinet_path_entry);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), cabinet_path_entry, 1, 2, 6, 7,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	cabinet_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (cabinet_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "cabinet_browse_button", cabinet_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (cabinet_browse_button);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), cabinet_browse_button, 2, 3, 6, 7,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

/* Marquees directory */
	marquee_label = gtk_label_new (_("Marquees Path"));
	gtk_widget_ref (marquee_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "marquee_label", marquee_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (marquee_label);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), marquee_label, 0, 1, 7, 8,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (marquee_label), 0, 0.5);

	marquee_path_entry = gtk_entry_new ();
	gtk_widget_ref (marquee_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "marquee_path_entry", marquee_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (marquee_path_entry), gui_prefs.MarqueeDirectory);
	gtk_widget_show (marquee_path_entry);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), marquee_path_entry, 1, 2, 7, 8,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	marquee_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (marquee_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "marquee_browse_button", marquee_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (marquee_browse_button);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), marquee_browse_button, 2, 3, 7, 8,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

/* Titles directory */
	title_label = gtk_label_new (_("Title screenshots Path"));
	gtk_widget_ref (title_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "title_label", title_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (title_label);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), title_label, 0, 1, 8, 9,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (title_label), 0, 0.5);

	title_path_entry = gtk_entry_new ();
	gtk_widget_ref (title_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "title_path_entry", title_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (title_path_entry), gui_prefs.TitleDirectory);
	gtk_widget_show (title_path_entry);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), title_path_entry, 1, 2, 8, 9,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	title_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (title_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "title_browse_button", title_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (title_browse_button);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), title_browse_button, 2, 3, 8, 9,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

/* Icons directory */
	icons_label = gtk_label_new (_("Icons Path"));
	gtk_widget_ref (icons_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "icons_label", icons_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (icons_label);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), icons_label, 0, 1, 9, 10,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (icons_label), 0, 0.5);

	icons_path_entry = gtk_entry_new ();
	gtk_widget_ref (icons_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "icons_path_entry", icons_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (icons_path_entry), gui_prefs.IconDirectory);
	gtk_widget_show (icons_path_entry);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), icons_path_entry, 1, 2, 9, 10,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	icons_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (icons_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "icons_browse_button", icons_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (icons_browse_button);
	gtk_table_attach (GTK_TABLE (gxmame_directory_table), icons_browse_button, 2, 3, 9, 10,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);


/*  second frame */	
	xmame_directory_frame = gtk_frame_new (_("paths to files needed for x-mame"));
	gtk_widget_ref (xmame_directory_frame);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_directory_frame", xmame_directory_frame,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (xmame_directory_frame);
	gtk_container_set_border_width (GTK_CONTAINER (xmame_directory_frame), 5);

	xmame_directory_table = gtk_table_new (4, 3, FALSE);
	gtk_widget_ref (xmame_directory_table);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_directory_table", xmame_directory_table,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (xmame_directory_table);
	gtk_container_add (GTK_CONTAINER (xmame_directory_frame), xmame_directory_table);
	gtk_container_set_border_width (GTK_CONTAINER (xmame_directory_table), 2);
	gtk_table_set_row_spacings (GTK_TABLE (xmame_directory_table), 5);
	gtk_table_set_col_spacings (GTK_TABLE (xmame_directory_table), 5);

/*	wmame_title_frame_label = gtk_label_new (_("path to files needed for x-mame"));
	gtk_widget_ref (wmame_title_frame_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "wmame_title_frame_label", wmame_title_frame_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (wmame_title_frame_label);
	gtk_table_attach (GTK_TABLE (xmame_directory_table), wmame_title_frame_label, 0, 7, 0, 1,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (wmame_title_frame_label), 0.5, 0.5);
*/	
	xmame_tab_label1 = gtk_label_new (_("XMame basic paths"));
	gtk_widget_ref (xmame_tab_label1);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_tab_label1", xmame_tab_label1,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (xmame_tab_label1);
	
	gtk_notebook_append_page        (GTK_NOTEBOOK(directories_notebook),
                                             xmame_directory_frame,
                                             xmame_tab_label1);

/* Romspath directories*/
	rompath_label = gtk_label_new (_("Roms Paths"));
	gtk_widget_ref (rompath_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "rompath_label", rompath_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (rompath_label);
	gtk_table_attach (GTK_TABLE (xmame_directory_table), rompath_label, 0, 1, 1, 5,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (rompath_label), 0, 0);
	gtk_misc_set_padding (GTK_MISC (rompath_label), 0, 5);

	rompath_entry_vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (rompath_entry_vbox);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "rompath_entry_vbox", rompath_entry_vbox,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (rompath_entry_vbox);
	gtk_table_attach (GTK_TABLE (xmame_directory_table), rompath_entry_vbox, 1, 2, 1, 5,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

	roms_path_entry = gtk_entry_new ();
	gtk_widget_ref (roms_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "roms_path_entry", roms_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (roms_path_entry);
	gtk_box_pack_start (GTK_BOX (rompath_entry_vbox), roms_path_entry, FALSE, FALSE, 0);

	rompath_scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_ref (rompath_scrolledwindow);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "rompath_scrolledwindow", rompath_scrolledwindow,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (rompath_scrolledwindow);
	gtk_box_pack_start (GTK_BOX (rompath_entry_vbox), rompath_scrolledwindow, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (rompath_scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	rompath_viewport = gtk_viewport_new (NULL, NULL);
	gtk_widget_ref (rompath_viewport);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "rompath_viewport", rompath_viewport,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (rompath_viewport);
	gtk_container_add (GTK_CONTAINER (rompath_scrolledwindow), rompath_viewport);

	roms_path_gtk_list = gtk_list_new ();
	gtk_widget_ref (roms_path_gtk_list);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "roms_path_gtk_list", roms_path_gtk_list,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_list_append_items(GTK_LIST(roms_path_gtk_list) ,roms_path_list);
	gtk_widget_show_all (roms_path_gtk_list);
	gtk_container_add (GTK_CONTAINER (rompath_viewport), roms_path_gtk_list);

	rompath_button_vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (rompath_button_vbox);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "rompath_button_vbox", rompath_button_vbox,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (rompath_button_vbox);
	gtk_table_attach (GTK_TABLE (xmame_directory_table), rompath_button_vbox, 2, 3, 1, 5,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (GTK_SHRINK | GTK_FILL), 0, 0);

	roms_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (roms_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "roms_browse_button", roms_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (roms_browse_button);
	gtk_box_pack_start (GTK_BOX (rompath_button_vbox), roms_browse_button, FALSE, FALSE, 2);

	roms_add_button = gtk_button_new_with_label (_("Add"));
	gtk_widget_ref (roms_add_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "roms_add_button", roms_add_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (roms_add_button);
	gtk_box_pack_start (GTK_BOX (rompath_button_vbox), roms_add_button, FALSE, FALSE, 2);

	roms_remove_button = gtk_button_new_with_label (_("Remove"));
	gtk_widget_ref (roms_remove_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "roms_remove_button", roms_remove_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (roms_remove_button);
	gtk_box_pack_start (GTK_BOX (rompath_button_vbox), roms_remove_button, FALSE, FALSE, 2);

/* samplespath directories*/
	sample_label = gtk_label_new (_("Samples Paths"));
	gtk_widget_ref (sample_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "sample_label", sample_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (sample_label);
	gtk_table_attach (GTK_TABLE (xmame_directory_table), sample_label, 0, 1, 6, 10,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (sample_label), 0, 0);
	gtk_misc_set_padding (GTK_MISC (sample_label), 0, 5);

	samplepath_entry_vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (samplepath_entry_vbox);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "samplepath_entry_vbox", samplepath_entry_vbox,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (samplepath_entry_vbox);
	gtk_table_attach (GTK_TABLE (xmame_directory_table), samplepath_entry_vbox, 1, 2, 6, 10,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

	samples_path_entry = gtk_entry_new ();
	gtk_widget_ref (samples_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "samples_path_entry", samples_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (samples_path_entry);
	gtk_box_pack_start (GTK_BOX (samplepath_entry_vbox), samples_path_entry, FALSE, FALSE, 0);

	samplepath_scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_ref (samplepath_scrolledwindow);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "samplepath_scrolledwindow", samplepath_scrolledwindow,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (samplepath_scrolledwindow);
	gtk_box_pack_start (GTK_BOX (samplepath_entry_vbox), samplepath_scrolledwindow, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (samplepath_scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	samplepath_viewport = gtk_viewport_new (NULL, NULL);
	gtk_widget_ref (samplepath_viewport);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "samplepath_viewport", samplepath_viewport,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (samplepath_viewport);
	gtk_container_add (GTK_CONTAINER (samplepath_scrolledwindow), samplepath_viewport);

	samples_path_gtk_list = gtk_list_new ();
	gtk_widget_ref (samples_path_gtk_list);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "samples_path_gtk_list", samples_path_gtk_list,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_list_append_items(GTK_LIST(samples_path_gtk_list) ,samples_path_list);
	gtk_widget_show_all (samples_path_gtk_list);
	gtk_container_add (GTK_CONTAINER (samplepath_viewport), samples_path_gtk_list);

	samplepath_button_vbox = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (samplepath_button_vbox);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "samplepath_button_vbox", samplepath_button_vbox,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (samplepath_button_vbox);
	gtk_table_attach (GTK_TABLE (xmame_directory_table), samplepath_button_vbox, 2, 3, 6, 10,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (GTK_SHRINK | GTK_FILL), 0, 0);

	samples_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (samples_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "samples_browse_button", samples_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (samples_browse_button);
	gtk_box_pack_start (GTK_BOX (samplepath_button_vbox), samples_browse_button, FALSE, FALSE, 2);

	samples_add_button = gtk_button_new_with_label (_("Add"));
	gtk_widget_ref (samples_add_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "samples_add_button", samples_add_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (samples_add_button);
	gtk_box_pack_start (GTK_BOX (samplepath_button_vbox), samples_add_button, FALSE, FALSE, 2);

	samples_remove_button = gtk_button_new_with_label (_("Remove"));
	gtk_widget_ref (samples_remove_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "samples_remove_button", samples_remove_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (samples_remove_button);
	gtk_box_pack_start (GTK_BOX (samplepath_button_vbox), samples_remove_button, FALSE, FALSE, 2);

/* Artwork directory */
	artwork_label = gtk_label_new (_("Artwork Path"));
	gtk_widget_ref (artwork_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "artwork_label", artwork_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (artwork_label);
	gtk_table_attach (GTK_TABLE (xmame_directory_table), artwork_label, 0, 1, 10, 11,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (artwork_label), 0, 0.5);

	artwork_path_entry = gtk_entry_new ();
	gtk_widget_ref (artwork_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "artwork_path_entry", artwork_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (artwork_path_entry), gui_prefs.ArtworkDirectory);
	gtk_widget_show (artwork_path_entry);
	gtk_table_attach (GTK_TABLE (xmame_directory_table), artwork_path_entry, 1, 2, 10, 11,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	artwork_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (artwork_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "artwork_browse_button", artwork_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (artwork_browse_button);
	gtk_table_attach (GTK_TABLE (xmame_directory_table), artwork_browse_button, 2, 3, 10, 11,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	
/* separators */	
/*	
	hseparator1 = gtk_hseparator_new();
	gtk_widget_show(hseparator1);
	gtk_table_attach (GTK_TABLE (xmame_directory_table), hseparator1, 0, 7, 5, 6,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	
*/	
/*  third frame */	
	xmame_addit_path_frame = gtk_frame_new (_("paths to additional files needed for x-mame"));
	gtk_widget_ref (xmame_addit_path_frame);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_addit_path_frame", xmame_addit_path_frame,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (xmame_addit_path_frame);
	gtk_container_set_border_width (GTK_CONTAINER (xmame_addit_path_frame), 5);

	xmame_addit_path_table = gtk_table_new (4, 3, FALSE);
	gtk_widget_ref (xmame_addit_path_table);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_addit_path_table", xmame_addit_path_table,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (xmame_addit_path_table);
	gtk_container_add (GTK_CONTAINER (xmame_addit_path_frame), xmame_addit_path_table);
	gtk_container_set_border_width (GTK_CONTAINER (xmame_addit_path_table), 2);
	gtk_table_set_row_spacings (GTK_TABLE (xmame_addit_path_table), 5);
	gtk_table_set_col_spacings (GTK_TABLE (xmame_addit_path_table), 5);

	xmame_tab_label2 = gtk_label_new (_("XMame additional paths"));
	gtk_widget_ref (xmame_tab_label2);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_tab_label2", xmame_tab_label2,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (xmame_tab_label2);
	
	gtk_notebook_append_page        (GTK_NOTEBOOK(directories_notebook),
                                             xmame_addit_path_frame,
                                             xmame_tab_label2);
		
/* Snapshot directory */
	snapshot_label = gtk_label_new (_("Snapshots Path"));
	gtk_widget_ref (snapshot_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "snapshot_label", snapshot_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (snapshot_label);
	gtk_table_attach (GTK_TABLE (xmame_addit_path_table), snapshot_label, 0, 1, 0, 1,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (snapshot_label), 0, 0.5);

	snapshot_path_entry = gtk_entry_new ();
	gtk_widget_ref (snapshot_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "snapshot_path_entry", snapshot_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (snapshot_path_entry), gui_prefs.SnapshotDirectory);
	gtk_widget_show (snapshot_path_entry);
	gtk_table_attach (GTK_TABLE (xmame_addit_path_table), snapshot_path_entry, 1, 2, 0, 1,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	snapshots_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (snapshots_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "snapshots_browse_button", snapshots_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (snapshots_browse_button);
	gtk_table_attach (GTK_TABLE (xmame_addit_path_table), snapshots_browse_button, 2, 3, 0, 1,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	
/* historyfile File */
	historyfile_label = gtk_label_new (_("History File"));
	gtk_widget_ref (historyfile_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "historyfile_label", historyfile_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (historyfile_label);
	gtk_table_attach (GTK_TABLE (xmame_addit_path_table), historyfile_label, 0, 1, 1, 2,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (historyfile_label), 0, 0.5);

	historyfile_path_entry = gtk_entry_new ();
	gtk_widget_ref (historyfile_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "historyfile_path_entry", historyfile_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (historyfile_path_entry), gui_prefs.HistoryFile);
	gtk_widget_show (historyfile_path_entry);
	gtk_table_attach (GTK_TABLE (xmame_addit_path_table), historyfile_path_entry, 1, 2, 1, 2,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	historyfile_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (historyfile_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "historyfile_browse_button", historyfile_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (historyfile_browse_button);
	gtk_table_attach (GTK_TABLE (xmame_addit_path_table), historyfile_browse_button, 2, 3, 1, 2,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

/* mameinfofile File */
	mameinfofile_label = gtk_label_new (_("Mame Info File"));
	gtk_widget_ref (mameinfofile_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "mameinfofile_label", mameinfofile_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (mameinfofile_label);
	gtk_table_attach (GTK_TABLE (xmame_addit_path_table), mameinfofile_label, 0, 1, 2, 3,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (mameinfofile_label), 0, 0.5);

	mameinfofile_path_entry = gtk_entry_new ();
	gtk_widget_ref (mameinfofile_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "mameinfofile_path_entry", mameinfofile_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (mameinfofile_path_entry), gui_prefs.MameInfoFile);
	gtk_widget_show (mameinfofile_path_entry);
	gtk_table_attach (GTK_TABLE (xmame_addit_path_table), mameinfofile_path_entry, 1, 2, 2, 3,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	mameinfofile_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (mameinfofile_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "mameinfofile_browse_button", mameinfofile_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (mameinfofile_browse_button);
	gtk_table_attach (GTK_TABLE (xmame_addit_path_table), mameinfofile_browse_button, 2, 3, 2, 3,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	vseparator1 = gtk_hseparator_new();
	gtk_widget_show(vseparator1);
	gtk_table_attach (GTK_TABLE (xmame_addit_path_table), vseparator1, 0, 3, 3, 4,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

/* Cheat File */
	cheatfile_label = gtk_label_new (_("Cheat File"));
	gtk_widget_ref (cheatfile_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "cheatfile_label", cheatfile_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (cheatfile_label);
	gtk_table_attach (GTK_TABLE (xmame_addit_path_table), cheatfile_label, 0, 1, 4, 5,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (cheatfile_label), 0, 0.5);

	cheatfile_path_entry = gtk_entry_new ();
	gtk_widget_ref (cheatfile_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "cheatfile_path_entry", cheatfile_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (cheatfile_path_entry), gui_prefs.CheatFile);
	gtk_widget_show (cheatfile_path_entry);
	gtk_table_attach (GTK_TABLE (xmame_addit_path_table), cheatfile_path_entry, 1, 2, 4, 5,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	cheatfile_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (cheatfile_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "cheatfile_browse_button", cheatfile_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (cheatfile_browse_button);
	gtk_table_attach (GTK_TABLE (xmame_addit_path_table), cheatfile_browse_button, 2, 3, 4, 5,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

/* hiscorefile File */
	hiscorefile_label = gtk_label_new (_("HiScore File"));
	gtk_widget_ref (hiscorefile_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "hiscorefile_label", hiscorefile_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hiscorefile_label);
	gtk_table_attach (GTK_TABLE (xmame_addit_path_table), hiscorefile_label, 0, 1, 5, 6,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (hiscorefile_label), 0, 0.5);

	hiscorefile_path_entry = gtk_entry_new ();
	gtk_widget_ref (hiscorefile_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "hiscorefile_path_entry", hiscorefile_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (hiscorefile_path_entry), gui_prefs.HiscoreFile);
	gtk_widget_show (hiscorefile_path_entry);
	gtk_table_attach (GTK_TABLE (xmame_addit_path_table), hiscorefile_path_entry, 1, 2, 5, 6,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	hiscorefile_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (hiscorefile_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "hiscorefile_browse_button", hiscorefile_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hiscorefile_browse_button);
	gtk_table_attach (GTK_TABLE (xmame_addit_path_table), hiscorefile_browse_button, 2, 3, 5, 6,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

/* ctrlr Directory */
	ctrlr_directory_label = gtk_label_new (_("Controler Directory"));
	gtk_widget_ref (ctrlr_directory_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "ctrlr_directory_label", ctrlr_directory_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (ctrlr_directory_label);
	gtk_table_attach (GTK_TABLE (xmame_addit_path_table), ctrlr_directory_label, 0, 1, 6, 7,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (ctrlr_directory_label), 0, 0.5);

	ctrlr_directory_entry = gtk_entry_new ();
	gtk_widget_ref (ctrlr_directory_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "ctrlr_directory_entry", ctrlr_directory_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (ctrlr_directory_entry), gui_prefs.CtrlrDirectory);
	gtk_widget_show (ctrlr_directory_entry);
	gtk_table_attach (GTK_TABLE (xmame_addit_path_table), ctrlr_directory_entry, 1, 2, 6, 7,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	ctrlr_directory_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (ctrlr_directory_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "ctrlr_directory_browse_button", ctrlr_directory_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (ctrlr_directory_browse_button);
	gtk_table_attach (GTK_TABLE (xmame_addit_path_table), ctrlr_directory_browse_button, 2, 3, 6, 7,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	
	
/*  fourth frame */	
	xmame_user_path_frame = gtk_frame_new (_("paths to user resources"));
	gtk_widget_ref (xmame_user_path_frame);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_user_path_frame", xmame_user_path_frame,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (xmame_user_path_frame);
	gtk_container_set_border_width (GTK_CONTAINER (xmame_user_path_frame), 5);

	xmame_user_path_table = gtk_table_new (4, 3, FALSE);
	gtk_widget_ref (xmame_user_path_table);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_user_path_table", xmame_user_path_table,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (xmame_user_path_table);
	gtk_container_add (GTK_CONTAINER (xmame_user_path_frame), xmame_user_path_table);
	gtk_container_set_border_width (GTK_CONTAINER (xmame_user_path_table), 2);
	gtk_table_set_row_spacings (GTK_TABLE (xmame_user_path_table), 5);
	gtk_table_set_col_spacings (GTK_TABLE (xmame_user_path_table), 5);

	xmame_tab_label2 = gtk_label_new (_("User resources"));
	gtk_widget_ref (xmame_tab_label2);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_tab_label2", xmame_tab_label2,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (xmame_tab_label2);
	
	gtk_notebook_append_page        (GTK_NOTEBOOK(directories_notebook),
                                             xmame_user_path_frame,
                                             xmame_tab_label2);
		
/* hiscores directory */
	hiscore_label = gtk_label_new (_("High Score Path"));
	gtk_widget_ref (hiscore_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "hiscore_label", hiscore_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hiscore_label);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), hiscore_label, 0, 1, 0, 1,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (hiscore_label), 0, 0.5);

	hiscore_path_entry = gtk_entry_new ();
	gtk_widget_ref (hiscore_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "hiscore_path_entry", hiscore_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (hiscore_path_entry), gui_prefs.HiscoreDirectory);
	gtk_widget_show (hiscore_path_entry);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), hiscore_path_entry, 1, 2, 0, 1,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	hiscore_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (hiscore_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "hiscore_browse_button", hiscore_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hiscore_browse_button);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), hiscore_browse_button, 2, 3, 0, 1,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	
/* input directory */
	input_label = gtk_label_new (_("Recorded games Path"));
	gtk_widget_ref (input_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "input_label", input_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (input_label);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), input_label, 0, 1, 1, 2,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (input_label), 0, 0.5);

	input_path_entry = gtk_entry_new ();
	gtk_widget_ref (input_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "input_path_entry", input_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (input_path_entry), gui_prefs.InputDirectory);
	gtk_widget_show (input_path_entry);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), input_path_entry, 1, 2, 1, 2,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	input_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (input_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "input_browse_button", input_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (input_browse_button);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), input_browse_button, 2, 3, 1, 2,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

/* cfg_directory directory */
	cfg_directory_label = gtk_label_new (_("Games configuration Path"));
	gtk_widget_ref (cfg_directory_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "cfg_directory_label", cfg_directory_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (cfg_directory_label);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), cfg_directory_label, 0, 1, 2, 3,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (cfg_directory_label), 0, 0.5);

	cfg_directory_entry = gtk_entry_new ();
	gtk_widget_ref (cfg_directory_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "cfg_directory_entry", cfg_directory_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (cfg_directory_entry), gui_prefs.ConfigDirectory);
	gtk_widget_show (cfg_directory_entry);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), cfg_directory_entry, 1, 2, 2, 3,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	cfg_directory_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (cfg_directory_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "cfg_directory_browse_button", cfg_directory_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (cfg_directory_browse_button);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), cfg_directory_browse_button, 2, 3, 2, 3,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

/* nvram_directory directory */
	nvram_directory_label = gtk_label_new (_("NVRam Path"));
	gtk_widget_ref (nvram_directory_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "nvram_directory_label", nvram_directory_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (nvram_directory_label);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), nvram_directory_label, 0, 1, 3, 4,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (nvram_directory_label), 0, 0.5);

	nvram_directory_entry = gtk_entry_new ();
	gtk_widget_ref (nvram_directory_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "nvram_directory_entry", nvram_directory_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (nvram_directory_entry), gui_prefs.NVRamDirectory);
	gtk_widget_show (nvram_directory_entry);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), nvram_directory_entry, 1, 2, 3, 4,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	nvram_directory_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (nvram_directory_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "nvram_directory_browse_button", nvram_directory_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (nvram_directory_browse_button);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), nvram_directory_browse_button, 2, 3, 3, 4,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

/* memcard_directory directory */
	memcard_directory_label = gtk_label_new (_("Memory cards Path"));
	gtk_widget_ref (memcard_directory_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "memcard_directory_label", memcard_directory_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (memcard_directory_label);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), memcard_directory_label, 0, 1, 4, 5,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (memcard_directory_label), 0, 0.5);

	memcard_directory_entry = gtk_entry_new ();
	gtk_widget_ref (memcard_directory_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "memcard_directory_entry", memcard_directory_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (memcard_directory_entry), gui_prefs.MemCardDirectory);
	gtk_widget_show (memcard_directory_entry);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), memcard_directory_entry, 1, 2, 4, 5,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	memcard_directory_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (memcard_directory_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "memcard_directory_browse_button", memcard_directory_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (memcard_directory_browse_button);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), memcard_directory_browse_button, 2, 3, 4, 5,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

/* state_directory directory */
	state_directory_label = gtk_label_new (_("Game Status Path"));
	gtk_widget_ref (state_directory_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "state_directory_label", state_directory_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (state_directory_label);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), state_directory_label, 0, 1, 5, 6,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (state_directory_label), 0, 0.5);

	state_directory_entry = gtk_entry_new ();
	gtk_widget_ref (state_directory_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "state_directory_entry", state_directory_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (state_directory_entry), gui_prefs.StateDirectory);
	gtk_widget_show (state_directory_entry);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), state_directory_entry, 1, 2, 5, 6,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	state_directory_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (state_directory_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "state_directory_browse_button", state_directory_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (state_directory_browse_button);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), state_directory_browse_button, 2, 3, 5, 6,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	
/* inipath */
	inipath_label = gtk_label_new (_("Ini files Path"));
	gtk_widget_ref (inipath_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "inipath_label", inipath_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (inipath_label);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), inipath_label, 0, 1, 6, 7,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (inipath_label), 0, 0.5);

	inipath_entry = gtk_entry_new ();
	gtk_widget_ref (inipath_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "inipath_entry", inipath_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (inipath_entry), gui_prefs.inipath);
	gtk_widget_show (inipath_entry);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), inipath_entry, 1, 2, 6, 7,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	inipath_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (inipath_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "inipath_browse_button", inipath_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (inipath_browse_button);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), inipath_browse_button, 2, 3, 6, 7,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	
/* diffdir directory */
	diffdir_label = gtk_label_new (_("Diff Path"));
	gtk_widget_ref (diffdir_label);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "diffdir_label", diffdir_label,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (diffdir_label);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), diffdir_label, 0, 1, 7, 8,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (diffdir_label), 0, 0.5);

	diffdir_path_entry = gtk_entry_new ();
	gtk_widget_ref (diffdir_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "diffdir_path_entry", diffdir_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (diffdir_path_entry), gui_prefs.DiffDirectory);
	gtk_widget_show (diffdir_path_entry);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), diffdir_path_entry, 1, 2, 7, 8,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	diffdirs_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (diffdirs_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "diffdirs_browse_button", diffdirs_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (diffdirs_browse_button);
	gtk_table_attach (GTK_TABLE (xmame_user_path_table), diffdirs_browse_button, 2, 3, 7, 8,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	


	
	
	
/* OK Apply Cancel buttons */
	main_buttons_hbox = gtk_hbox_new (FALSE, 5);
	gtk_widget_ref (main_buttons_hbox);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "main_buttons_hbox", main_buttons_hbox,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (main_buttons_hbox);
	gtk_box_pack_start (GTK_BOX (main_vbox), main_buttons_hbox, FALSE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (main_buttons_hbox), 5);

	directories_selection_cancel_button = gtk_button_new_with_label ("");
	directories_selection_cancel_button_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (directories_selection_cancel_button)->child),
					 _("_Cancel"));
	gtk_widget_add_accelerator (directories_selection_cancel_button, "clicked", accel_group,
				    directories_selection_cancel_button_key, GDK_MOD1_MASK, (GtkAccelFlags) 0);
	gtk_widget_ref (directories_selection_cancel_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "directories_selection_cancel_button", directories_selection_cancel_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (directories_selection_cancel_button);
	gtk_box_pack_end (GTK_BOX (main_buttons_hbox), directories_selection_cancel_button, FALSE, FALSE, 0);
	gtk_widget_set_usize (directories_selection_cancel_button, 70, -2);

	directories_selection_apply_button = gtk_button_new_with_label ("");
	directories_selection_apply_button_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (directories_selection_apply_button)->child),
					 _("_Apply"));
	gtk_widget_add_accelerator (directories_selection_apply_button, "clicked", accel_group,
				    directories_selection_apply_button_key, GDK_MOD1_MASK, (GtkAccelFlags) 0);
	gtk_widget_ref (directories_selection_apply_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "directories_selection_apply_button", directories_selection_apply_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (directories_selection_apply_button);
	gtk_box_pack_end (GTK_BOX (main_buttons_hbox), directories_selection_apply_button, FALSE, FALSE, 0);
	gtk_widget_set_usize (directories_selection_apply_button, 70, -2);

	gtk_widget_set_sensitive(directories_selection_apply_button, FALSE);
	
	directories_selection_ok_button = gtk_button_new_with_label ("");
	directories_selection_ok_button_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (directories_selection_ok_button)->child),
					 _("_OK"));
	gtk_widget_add_accelerator (directories_selection_ok_button, "clicked", accel_group,
				    directories_selection_ok_button_key, GDK_MOD1_MASK, (GtkAccelFlags) 0);
	gtk_widget_ref (directories_selection_ok_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "directories_selection_ok_button", directories_selection_ok_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (directories_selection_ok_button);
	gtk_box_pack_end (GTK_BOX (main_buttons_hbox), directories_selection_ok_button, FALSE, FALSE, 0);
	gtk_widget_set_usize (directories_selection_ok_button, 70, -2);

	/* button clicked */
	/* GXMame */
	gtk_signal_connect (GTK_OBJECT (xmame_execs_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_file_browse_button_clicked),
			    xmame_execs_path_entry);
	gtk_signal_connect (GTK_OBJECT (xmame_execs_add_button), "clicked",
			    GTK_SIGNAL_FUNC (on_xmame_execs_add_button_clicked),
			    NULL);
	gtk_signal_connect (GTK_OBJECT (xmame_execs_remove_button), "clicked",
			    GTK_SIGNAL_FUNC (on_xmame_execs_remove_button_clicked),
			    NULL);
	gtk_signal_connect (GTK_OBJECT (common_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_dir_browse_button_clicked),
			    common_path_entry);
	gtk_signal_connect (GTK_OBJECT (flyer_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_dir_browse_button_clicked),
			    flyer_path_entry);
	gtk_signal_connect (GTK_OBJECT (cabinet_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_dir_browse_button_clicked),
			    cabinet_path_entry);
	gtk_signal_connect (GTK_OBJECT (marquee_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_dir_browse_button_clicked),
			    marquee_path_entry);
	gtk_signal_connect (GTK_OBJECT (title_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_dir_browse_button_clicked),
			    title_path_entry);
	gtk_signal_connect (GTK_OBJECT (icons_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_dir_browse_button_clicked),
			    icons_path_entry);
	
	/* XMame basic */
	gtk_signal_connect (GTK_OBJECT (samples_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_dir_browse_button_clicked),
			    samples_path_entry);
	gtk_signal_connect (GTK_OBJECT (samples_add_button), "clicked",
			    GTK_SIGNAL_FUNC (on_samples_add_button_clicked),
			    NULL);
	gtk_signal_connect (GTK_OBJECT (samples_remove_button), "clicked",
			    GTK_SIGNAL_FUNC (on_samples_remove_button_clicked),
			    NULL);
	gtk_signal_connect (GTK_OBJECT (roms_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_dir_browse_button_clicked),
			    roms_path_entry);
	gtk_signal_connect (GTK_OBJECT (roms_add_button), "clicked",
			    GTK_SIGNAL_FUNC (on_roms_add_button_clicked),
			    NULL);
	gtk_signal_connect (GTK_OBJECT (roms_remove_button), "clicked",
			    GTK_SIGNAL_FUNC (on_roms_remove_button_clicked),
			    NULL);
	gtk_signal_connect (GTK_OBJECT (diffdirs_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_dir_browse_button_clicked),
			    diffdir_path_entry);
	gtk_signal_connect (GTK_OBJECT (artwork_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_dir_browse_button_clicked),
			    artwork_path_entry);
	
	/* XMame additional */
	gtk_signal_connect (GTK_OBJECT (snapshots_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_dir_browse_button_clicked),
			    snapshot_path_entry);
	gtk_signal_connect (GTK_OBJECT (historyfile_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_file_browse_button_clicked),
			    historyfile_path_entry);
	gtk_signal_connect (GTK_OBJECT (mameinfofile_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_file_browse_button_clicked),
			    mameinfofile_path_entry);
	gtk_signal_connect (GTK_OBJECT (cheatfile_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_file_browse_button_clicked),
			    cheatfile_path_entry);
	gtk_signal_connect (GTK_OBJECT (hiscorefile_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_file_browse_button_clicked),
			    hiscorefile_path_entry);
	gtk_signal_connect (GTK_OBJECT (ctrlr_directory_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_file_browse_button_clicked),
			    ctrlr_directory_entry);
	
	/* User Resources */
	gtk_signal_connect (GTK_OBJECT (hiscore_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_dir_browse_button_clicked),
			    hiscore_path_entry);
	gtk_signal_connect (GTK_OBJECT (input_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_dir_browse_button_clicked),
			    input_path_entry);
	gtk_signal_connect (GTK_OBJECT (cfg_directory_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_file_browse_button_clicked),
			    cfg_directory_entry);
	gtk_signal_connect (GTK_OBJECT (nvram_directory_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_file_browse_button_clicked),
			    nvram_directory_entry);
	gtk_signal_connect (GTK_OBJECT (memcard_directory_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_file_browse_button_clicked),
			    memcard_directory_entry);
	gtk_signal_connect (GTK_OBJECT (state_directory_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_file_browse_button_clicked),
			    state_directory_entry);
	gtk_signal_connect (GTK_OBJECT (inipath_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_file_browse_button_clicked),
			    inipath_entry);
	
	
	gtk_signal_connect (GTK_OBJECT (directories_selection_cancel_button), "clicked",
			    GTK_SIGNAL_FUNC (directories_selection_close_window),
			    directories_selection);
	gtk_signal_connect (GTK_OBJECT (directories_selection_apply_button), "clicked",
			    GTK_SIGNAL_FUNC (directories_selection_save_changes),
			    directories_selection);
	gtk_signal_connect (GTK_OBJECT (directories_selection_ok_button), "clicked",
			    GTK_SIGNAL_FUNC (directories_selection_save_and_close),
			    directories_selection);
	gtk_signal_connect (GTK_OBJECT (samples_path_gtk_list), "select-child",
			    GTK_SIGNAL_FUNC (on_gtk_list_sample_directory_selected),
			    samples_path_entry);
	gtk_signal_connect (GTK_OBJECT (roms_path_gtk_list), "select-child",
			    GTK_SIGNAL_FUNC (on_gtk_list_rom_directory_selected),
			    roms_path_entry);
	gtk_signal_connect (GTK_OBJECT (xmame_execs_path_gtk_list), "select-child",
			    GTK_SIGNAL_FUNC (on_gtk_list_xmame_execs_directory_selected),
			    xmame_execs_path_entry);
	/* window */
	gtk_signal_connect (GTK_OBJECT (directories_selection), "delete-event",
			    GTK_SIGNAL_FUNC (directories_selection_close_window),
			    directories_selection);
	
	/* entry changed */
	/* GXMame */
	gtk_signal_connect_object (GTK_OBJECT (flyer_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (cabinet_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (marquee_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (title_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (icons_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (common_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));

	/* XMame basic */
	gtk_signal_connect_object (GTK_OBJECT (diffdir_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (artwork_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	/* XMame additional */
	gtk_signal_connect_object (GTK_OBJECT (snapshot_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (mameinfofile_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (historyfile_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (hiscorefile_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (cheatfile_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (ctrlr_directory_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	
	/* User Resources */
	gtk_signal_connect_object (GTK_OBJECT (hiscore_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (input_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (cfg_directory_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (nvram_directory_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (memcard_directory_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (state_directory_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (inipath_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));

	gtk_window_add_accel_group (GTK_WINDOW (directories_selection), accel_group);
	
	return directories_selection;
}




void
on_dir_browse_button_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
	create_folderselection(user_data, TRUE);
}


void
on_file_browse_button_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
	create_folderselection(user_data, FALSE);
}


void
on_samples_add_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	gchar *temp_text;
	GList *temp_list = NULL,*list_ptr = NULL; 
	temp_text = gtk_editable_get_chars (GTK_EDITABLE(samples_path_entry),0,-1);
	gxmame_debug("adding:%s to samples folders",temp_text);
	/* I need to copy the list otherwise, I can't add it again to the gtk list*/
	for(list_ptr = g_list_first(samples_path_list);list_ptr!=NULL;list_ptr=g_list_next(list_ptr))
	{
		temp_list = g_list_append(temp_list,list_ptr->data);
	}
	gtk_list_remove_items_no_unref(GTK_LIST(samples_path_gtk_list) ,samples_path_list);
	temp_list = g_list_append (temp_list,gtk_list_item_new_with_label(temp_text));
	gtk_list_append_items(GTK_LIST(samples_path_gtk_list) ,temp_list);
	gtk_widget_show_all (samples_path_gtk_list);
	g_free(temp_text);
	samples_path_list = temp_list;
	gtk_widget_set_sensitive(directories_selection_apply_button, TRUE);
}


void
on_samples_remove_button_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
	GList *temp_list = NULL,*list_ptr = NULL; 
	/* I need to copy the list otherwise, I can't add it again to the gtk list*/
	for(list_ptr = g_list_first(samples_path_list);list_ptr!=NULL;list_ptr=g_list_next(list_ptr))
	{
		temp_list = g_list_append(temp_list,list_ptr->data);
	}
	temp_list = g_list_remove(temp_list,current_sample_list_item);
	gtk_list_remove_items_no_unref(GTK_LIST(samples_path_gtk_list) ,samples_path_list);
	gtk_list_append_items(GTK_LIST(samples_path_gtk_list) ,temp_list);
	gtk_widget_show_all (samples_path_gtk_list);
	samples_path_list = temp_list;
	gtk_widget_set_sensitive(directories_selection_apply_button, TRUE);
}


void
on_roms_add_button_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	gchar *temp_text;
	GList *temp_list = NULL,*list_ptr = NULL; 
	temp_text = gtk_editable_get_chars (GTK_EDITABLE(roms_path_entry),0,-1);
	gxmame_debug("adding:%s to roms folders",temp_text);
	/* I need to copy the list otherwise, I can't add it again to the gtk list*/
	for(list_ptr = g_list_first(roms_path_list);list_ptr!=NULL;list_ptr=g_list_next(list_ptr))
	{
		temp_list = g_list_append(temp_list,list_ptr->data);
	}
	gtk_list_remove_items_no_unref(GTK_LIST(roms_path_gtk_list) ,roms_path_list);
	temp_list = g_list_append (temp_list,gtk_list_item_new_with_label(temp_text));
	gtk_list_append_items(GTK_LIST(roms_path_gtk_list) ,temp_list);
	gtk_widget_show_all (roms_path_gtk_list);
	g_free(temp_text);
	roms_path_list = temp_list;
	gtk_widget_set_sensitive(directories_selection_apply_button, TRUE);
}


void
on_roms_remove_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	GList *temp_list = NULL,*list_ptr = NULL; 
	/* I need to copy the list otherwise, I can't add it again to the gtk list*/
	for(list_ptr = g_list_first(roms_path_list);list_ptr!=NULL;list_ptr=g_list_next(list_ptr))
	{
		temp_list = g_list_append(temp_list,list_ptr->data);
	}
	temp_list = g_list_remove(temp_list,current_rom_list_item);
	gtk_list_remove_items_no_unref(GTK_LIST(roms_path_gtk_list) ,roms_path_list);
	gtk_list_append_items(GTK_LIST(roms_path_gtk_list) ,temp_list);
	gtk_widget_show_all (roms_path_gtk_list);
	roms_path_list = temp_list;
	gtk_widget_set_sensitive(directories_selection_apply_button, TRUE);
}


void
on_xmame_execs_add_button_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	gchar *temp_text;
	GList *temp_list = NULL,*list_ptr = NULL; 
	temp_text = gtk_editable_get_chars (GTK_EDITABLE(xmame_execs_path_entry),0,-1);
	gxmame_debug("adding:%s to xmame_execs folders",temp_text);
	/* I need to copy the list otherwise, I can't add it again to the gtk list*/
	for(list_ptr = g_list_first(xmame_execs_path_list);list_ptr!=NULL;list_ptr=g_list_next(list_ptr))
	{
		temp_list = g_list_append(temp_list,list_ptr->data);
	}
	gtk_list_remove_items_no_unref(GTK_LIST(xmame_execs_path_gtk_list) ,xmame_execs_path_list);
	temp_list = g_list_append (temp_list,gtk_list_item_new_with_label(temp_text));
	gtk_list_append_items(GTK_LIST(xmame_execs_path_gtk_list) ,temp_list);
	gtk_widget_show_all (xmame_execs_path_gtk_list);
	g_free(temp_text);
	xmame_execs_path_list = temp_list;
	gtk_widget_set_sensitive(directories_selection_apply_button, TRUE);
}


void
on_xmame_execs_remove_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	GList *temp_list = NULL,*list_ptr = NULL; 
	/* I need to copy the list otherwise, I can't add it again to the gtk list*/
	for(list_ptr = g_list_first(xmame_execs_path_list);list_ptr!=NULL;list_ptr=g_list_next(list_ptr))
	{
		temp_list = g_list_append(temp_list,list_ptr->data);
	}
	temp_list = g_list_remove(temp_list,current_xmame_execs_list_item);
	gtk_list_remove_items_no_unref(GTK_LIST(xmame_execs_path_gtk_list) ,xmame_execs_path_list);
	gtk_list_append_items(GTK_LIST(xmame_execs_path_gtk_list) ,temp_list);
	gtk_widget_show_all (xmame_execs_path_gtk_list);
	xmame_execs_path_list = temp_list;
	gtk_widget_set_sensitive(directories_selection_apply_button, TRUE);
}


void
directories_selection_close_window
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_list_remove_items(GTK_LIST(roms_path_gtk_list) ,roms_path_list);
	gtk_list_remove_items(GTK_LIST(samples_path_gtk_list) ,samples_path_list);
	gtk_list_remove_items(GTK_LIST(xmame_execs_path_gtk_list) ,xmame_execs_path_list);
	/* test if its called from the cancel button or if the window was destroyed*/
	if (GTK_IS_WIDGET(user_data))
		gtk_widget_destroy(user_data);
	else
		gtk_widget_destroy(GTK_WIDGET(button));
	roms_path_list = NULL;
	samples_path_list = NULL;
	xmame_execs_path_list = NULL;
}


gboolean remove_hash_value(gpointer key,
                              gpointer value,
                              gpointer user_data)
{
	g_free(value);
	return TRUE;
}
	
void
directories_selection_save_changes
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
	GList *list_ptr;
	int size;
	gchar *message = NULL;
	/* Do I need this?*/
	GtkWidget *Loading_error = NULL;
	
	gtk_widget_set_sensitive(directories_selection_apply_button, FALSE);
	
	/* free the main config paths*/
	/* GXMame */
	g_free (gui_prefs.catverDirectory);
	g_free (gui_prefs.FlyerDirectroy);
	g_free (gui_prefs.CabinetDirectory);
	g_free (gui_prefs.MarqueeDirectory);
	g_free (gui_prefs.TitleDirectory);
	g_free (gui_prefs.IconDirectory);
	/* XMame basic */
	g_strfreev (gui_prefs.RomPath);
	g_strfreev (gui_prefs.SamplePath);
	g_free (gui_prefs.DiffDirectory);
	g_free (gui_prefs.ArtworkDirectory);
	/* XMame additional */
	g_free (gui_prefs.SnapshotDirectory);
	g_free (gui_prefs.HistoryFile);
	g_free (gui_prefs.MameInfoFile);
	g_free (gui_prefs.CheatFile);
	g_free (gui_prefs.CtrlrDirectory);
	g_free (gui_prefs.HiscoreFile);
	/* User Resources */
	g_free (gui_prefs.HiscoreDirectory);
	g_free (gui_prefs.InputDirectory);
	g_free (gui_prefs.ConfigDirectory);
	g_free (gui_prefs.NVRamDirectory);
	g_free (gui_prefs.MemCardDirectory);
	g_free (gui_prefs.StateDirectory);
	g_free (gui_prefs.inipath);
	
	
	/* and replace them by the new ones*/
	/* GXMame */
	gui_prefs.catverDirectory = gtk_editable_get_chars (GTK_EDITABLE(common_path_entry),0,-1);
	gui_prefs.FlyerDirectroy = gtk_editable_get_chars (GTK_EDITABLE(flyer_path_entry),0,-1);
	gui_prefs.CabinetDirectory = gtk_editable_get_chars (GTK_EDITABLE(cabinet_path_entry),0,-1);
	gui_prefs.MarqueeDirectory = gtk_editable_get_chars (GTK_EDITABLE(marquee_path_entry),0,-1);
	gui_prefs.TitleDirectory = gtk_editable_get_chars (GTK_EDITABLE(title_path_entry),0,-1);
	gui_prefs.IconDirectory = gtk_editable_get_chars (GTK_EDITABLE(icons_path_entry),0,-1);
	
	/* XMame additional */
	gui_prefs.SnapshotDirectory = gtk_editable_get_chars (GTK_EDITABLE(snapshot_path_entry),0,-1);
	gui_prefs.HistoryFile = gtk_editable_get_chars (GTK_EDITABLE(historyfile_path_entry),0,-1);
	gui_prefs.MameInfoFile = gtk_editable_get_chars (GTK_EDITABLE(mameinfofile_path_entry),0,-1);
	gui_prefs.HiscoreFile = gtk_editable_get_chars (GTK_EDITABLE(hiscorefile_path_entry),0,-1);
	gui_prefs.CheatFile = gtk_editable_get_chars (GTK_EDITABLE(cheatfile_path_entry),0,-1);
	gui_prefs.CtrlrDirectory = gtk_editable_get_chars (GTK_EDITABLE(ctrlr_directory_entry),0,-1);
	
	/* User Resources */
	gui_prefs.HiscoreDirectory = gtk_editable_get_chars (GTK_EDITABLE(hiscore_path_entry),0,-1);
	gui_prefs.InputDirectory = gtk_editable_get_chars (GTK_EDITABLE(input_path_entry),0,-1);
	gui_prefs.ConfigDirectory = gtk_editable_get_chars (GTK_EDITABLE(cfg_directory_entry),0,-1);
	gui_prefs.NVRamDirectory = gtk_editable_get_chars (GTK_EDITABLE(nvram_directory_entry),0,-1);
	gui_prefs.MemCardDirectory = gtk_editable_get_chars (GTK_EDITABLE(memcard_directory_entry),0,-1);
	gui_prefs.StateDirectory = gtk_editable_get_chars (GTK_EDITABLE(state_directory_entry),0,-1);
	gui_prefs.inipath = gtk_editable_get_chars (GTK_EDITABLE(inipath_entry),0,-1);
	

	/* XMame basic */
	gui_prefs.DiffDirectory = gtk_editable_get_chars (GTK_EDITABLE(diffdir_path_entry),0,-1);
	gui_prefs.ArtworkDirectory = gtk_editable_get_chars (GTK_EDITABLE(artwork_path_entry),0,-1);
	
	/* Transform the list content into a array of char terminated by a NULL element*/
	size = g_list_length(samples_path_list)+1;
	gui_prefs.SamplePath = g_new(gchar *, size);
	for (size=0,list_ptr = g_list_first(samples_path_list);
	     (list_ptr != NULL);
	      list_ptr= g_list_next(list_ptr),size++)
	{
		gui_prefs.SamplePath[size] = g_strdup(GTK_LABEL(GTK_BIN(list_ptr->data)->child)->label);
	}
	gui_prefs.SamplePath[size]=NULL;
	
	size = g_list_length(roms_path_list)+1;
	gui_prefs.RomPath = g_new(gchar *, size);
	for (size=0,list_ptr = g_list_first(roms_path_list);
	     (list_ptr != NULL);
	      list_ptr= g_list_next(list_ptr),size++)
	{
		gui_prefs.RomPath[size] = g_strdup(GTK_LABEL(GTK_BIN(list_ptr->data)->child)->label);
	}
	gui_prefs.RomPath[size]=NULL;

	
		
	/* free the value part of the hashtable*/
	if (xmame_executables_hash_table!=NULL)
	{
	g_hash_table_foreach_remove(xmame_executables_hash_table,
			(GHRFunc)remove_hash_value,NULL);
	g_hash_table_destroy(xmame_executables_hash_table);
	}
	/* then free the key part that was common with the array */
	g_strfreev (xmame_executables_array);
	/* recreate the array */
	size = g_list_length(xmame_execs_path_list)+1;
	xmame_executables_array = g_new(gchar *, size);
	for (size=0,list_ptr = g_list_first(xmame_execs_path_list);
	     (list_ptr != NULL);
	      list_ptr= g_list_next(list_ptr),size++)
	{
		xmame_executables_array[size] = g_strdup(GTK_LABEL(GTK_BIN(list_ptr->data)->child)->label);
	}
	xmame_executables_array[size]=NULL;
	/* Now fill the hashtable */
	check_executable_list();
	/* check if the executable is still valid and set versions.* variables */
	get_mame_version(mame_executable,EXECUTABLE);
	
	/*destroy the dropbox to rebuild a new one
	  I don't know if all element of the dropbox are destroyed or if I have to destroy all
	  maybe I'd better remove all items of the box to add new ones?*/
	gtk_widget_destroy(main_gui.xmame_executables_combo);
	/* remove and destroy the submenu */
	add_exec_dropbox();
	/* here we should already have a valid executable except if mame is not present in path */
	if (!strcmp(executable_version.name,"Not executable"))
	{
		message = g_strdup_printf(_("%s is not a valid executable"),mame_executable);
		Loading_error = GXMame_error_message(ERROR,message);
		g_free(message);
		message = g_strdup_printf(_("GXMame V%s"),
					VERSION);
		gtk_window_set_title (GTK_WINDOW (MainWindow), message);
		gtk_widget_set_sensitive(GTK_WIDGET(main_gui.audit_all_games_menu),FALSE);
		g_free(message);
	} else
	{	
		if (list_version.version==NULL || !strcmp(list_version.version,"unknown") || !strcmp(list_version.version,"none"))
		{
			Loading_error = GXMame_error_message(ERROR,_("Gamelist not available,\nyou need to build a new one\ngo to Option and rebuild a game list"));
		} else if (gui_prefs.VersionCheck && (strcmp(executable_version.name,list_version.name)||strcmp(executable_version.version,list_version.version)))
		{
			message = g_strdup_printf(_("the gamelist is from:\n%s (%s) %s\nand the current executable is:\n%s (%s) %s\nyou may want rebuild the game list"),
					list_version.name,
					list_version.target,
					list_version.version,
					executable_version.name,
					executable_version.target,
					executable_version.version);
			Loading_error = GXMame_error_message(WARNING,message);
			g_free(message);
		}
		message = g_strdup_printf(_("GXMame V%s running %s (%s) %s"),
					VERSION,
					executable_version.name,
					executable_version.target,
					executable_version.version);
		
		gtk_window_set_title (GTK_WINDOW (MainWindow), message);
		gtk_widget_set_sensitive(GTK_WIDGET(main_gui.audit_all_games_menu),TRUE);
		g_free(message);
		
		/* get mame options */
		get_mame_options();
	}
}




void
directories_selection_save_and_close
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
	directories_selection_save_changes(NULL,user_data);
	directories_selection_close_window(NULL,user_data);
}



void
on_ok_button_folderselection_clicked        (GtkButton       *button,
                                             gpointer         user_data)
{
	gxmame_debug("selected: {%s}",gtk_file_selection_get_filename (GTK_FILE_SELECTION(user_data)));
	gtk_entry_set_text (GTK_ENTRY (the_current_entry),gtk_file_selection_get_filename (GTK_FILE_SELECTION(user_data)));
	gtk_widget_destroy(user_data);
	gtk_widget_set_sensitive(directories_selection_apply_button, TRUE);
}


void
on_cancel_button_folderselection_clicked    (GtkButton       *button,
                                             gpointer         user_data)
{
	gtk_widget_destroy(user_data);
}

	
void
on_gtk_list_rom_directory_selected         (GtkList *list,
                                            GtkWidget *widget,
                                            gpointer user_data)
{
	current_rom_list_item = widget;
	gtk_entry_set_text (GTK_ENTRY (user_data), GTK_LABEL(GTK_BIN(widget)->child)->label);
}

void
on_gtk_list_sample_directory_selected      (GtkList *list,
                                            GtkWidget *widget,
                                            gpointer user_data)
{
	current_sample_list_item = widget;
	gtk_entry_set_text (GTK_ENTRY (user_data), GTK_LABEL(GTK_BIN(widget)->child)->label);
}

		
void
on_gtk_list_xmame_execs_directory_selected	(GtkList *list,
						 GtkWidget *widget,
						 gpointer user_data)
{
	current_xmame_execs_list_item = widget;
	gtk_entry_set_text (GTK_ENTRY (user_data), GTK_LABEL(GTK_BIN(widget)->child)->label);
}

		
		
