/*
 * GXMame
 *
 * 2002, 2003, Stphane Pontier <shadow_walker@users.sourceforge.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Original code taken from:
 * gtkjoy.c
 *
 * general routines for handling joystick events
 *
 * Copyright 1999 David Boynton.
 * http://trylinux.com/projects/joystick/
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#ifndef NO_JOYSTICK
#include <linux/joystick.h>
#endif

#include "gtkjoy.h"
#include "gxmame.h"
#include "gui.h"

#define JOY_TIMEOUT 30

guint timeoutjoy;
gboolean joystick_focus;

void joy_left_repetition (void);
void joy_right_repetition (void);
void joy_up_repetition (void);
void joy_down_repetition (void);
void select_row_down (void);
void select_row_up (int new_row);

#ifndef NO_JOYSTICK
static void 
joystick_event(gpointer data, gint fdsource, GdkInputCondition cond)
{
	struct js_event js;
	int readrc;
	JoyData * joydata;

	joydata=data;
	if (!joydata) {
		gxmame_debug("joystick_event: improper callback registration, %u", __LINE__);
		return;
	}

	readrc=read(fdsource, &js, sizeof(struct js_event));
	if (readrc == sizeof(struct js_event)) {
		switch ((js.type&3)) {
			case JS_EVENT_BUTTON:
				if ( js.number < JOYSTICK_MAXB ) 
					joydata->buttons[js.number]=js.value;
				break;
			case JS_EVENT_AXIS:
				if ( js.number < JOYSTICK_MAXA ) 
					joydata->axis[js.number]=js.value;
				break;
		}
		if ( joydata->callback )
			joydata->callback(js.type, js.number, joydata->callback_data);
	} else { 
		gxmame_debug("source(%d), %d bytes read", fdsource, readrc);
	}
}
#endif	

JoyData *   
joystick_new(char * joystick, void * owner, void * data)
{
#ifndef NO_JOYSTICK
	JoyData * joydata;

	joydata=(JoyData *)g_malloc(sizeof(JoyData));
	if (!joydata)
		return NULL;

	memset(joydata, 0, sizeof(JoyData));

	{ /* open joystick device */
		char *devname="/dev/js0";

		if ( joystick )
			devname=joystick;
		joydata->joystickfd= open(devname, O_RDONLY);

		if ( joydata->joystickfd < 0 ) {
			if ( errno == ENODEV ) {
				gxmame_debug("Device not configured - did you load the module?");
			} else {
				gxmame_debug("Couldn't open '%s'", devname);
				perror(devname);
			}
			return NULL;
		}
	}

	joydata->callback=owner; /* button callback */
	joydata->callback_data=data;

	{ /* load properties */
		char name[128] = "Unknown";

		fcntl(joydata->joystickfd, F_SETFL, O_NONBLOCK);
		ioctl(joydata->joystickfd, JSIOCGVERSION, &joydata->version);
		ioctl(joydata->joystickfd, JSIOCGAXES, &joydata->num_axis);
		ioctl(joydata->joystickfd, JSIOCGBUTTONS, &joydata->num_buttons);
		ioctl(joydata->joystickfd, JSIOCGNAME(128), name);
		joydata->device_name = strdup(name);
		joydata->gdktag=
				gdk_input_add(joydata->joystickfd, GDK_INPUT_READ, joystick_event, joydata);
	}

	return joydata;
#endif
#ifdef NO_JOYSTICK
	return NULL;
#endif	
}

void
joystick_close(JoyData * joydata)
{
#ifndef NO_JOYSTICK
	if (joydata)
	{
		if ( joydata->gdktag ) 
			gdk_input_remove(joydata->gdktag);
		if ( joydata->joystickfd != -1 )
			close(joydata->joystickfd);
		if ( joydata->device_name )
			g_free(joydata->device_name);
		g_free(joydata);
	}
#endif	
}


void 
joystick_events(gint eventtype, gint button, gpointer data)
{
	gboolean return_val;
	GdkEventKey event;
	/*the focus is not set: another windows has been opened */
	if (!joystick_focus)
		return;
	/*gxmame_debug("event nb %i button %i",eventtype,button);*/
/* JOYSTICK BUTTON */
	if ( (eventtype == 1) && (button < JOYSTICK_MAXB ) )
	{
		gxmame_debug("Joystick button %i %s",button,(joydata->buttons[button])?"pressed":"released");
		/* button 0 released*/
		if (button==0 && joydata->buttons[button]==0 && visible_games>0)
			play_game(gui_prefs.DefaultGame_p);
	}
/* JOYSTICK AXIS */
	else if ( ( eventtype == 2 ) && (button < JOYSTICK_MAXA ) )
	{
/* LEFT and RIGHT directions*/
		if ((button/2) == ((gfloat)button/2.0))
		{	
			if (joydata->axis[button] > (AXIS_MAX/2))
			{	
				gxmame_debug("Joystick axis %i pushed right",button);
				event.keyval = GDK_Page_Down;
				event.state = 0;
				gtk_signal_emit_by_name (GTK_OBJECT(main_gui.displayed_list),
						"key_press_event",
						&event,
						&return_val);
				if (timeoutjoy)
				  	gtk_timeout_remove(timeoutjoy);
				timeoutjoy =
				    gtk_timeout_add(10*JOY_TIMEOUT,
				  		    (GtkFunction) joy_right_repetition, NULL);
			} else if (joydata->axis[button] < (AXIS_MIN/2))
			{
				gxmame_debug("Joystick axis %i pushed left",button);
				event.keyval = GDK_Page_Up;
				event.state = 0;
				gtk_signal_emit_by_name (GTK_OBJECT(main_gui.displayed_list),
						"key_press_event",
						&event,
						&return_val);
				if (timeoutjoy)
				  	gtk_timeout_remove(timeoutjoy);
				timeoutjoy =
				    gtk_timeout_add(10*JOY_TIMEOUT,
				  		    (GtkFunction) joy_left_repetition, NULL);
			} else
			{
				gxmame_debug("Joystick axis %i released",button);
				if (timeoutjoy)
				  	gtk_timeout_remove(timeoutjoy);
			}
		}
		else
/* UP and DOWN directions*/
		{
			gint row;
			if (visible_games>0)
				row = gui_prefs.DefaultGame_p->position;
			else 
				row=-1;
			if (joydata->axis[button] > (AXIS_MAX/2))
			{	
				gxmame_debug("Joystick axis %i pushed down",button);
				event.keyval = GDK_Down;
				event.state = 0;
				gtk_signal_emit_by_name (GTK_OBJECT(main_gui.displayed_list),
						"key_press_event",
						&event,
						&return_val);
				if (timeoutjoy)
				  	gtk_timeout_remove(timeoutjoy);
				timeoutjoy =
				    gtk_timeout_add(10*JOY_TIMEOUT,
				  		    (GtkFunction) joy_down_repetition, NULL);
				
			} else if (joydata->axis[button] < (AXIS_MIN/2))
			{
				gxmame_debug("Joystick axis %i pushed up",button);
				event.keyval = GDK_Up;
				event.state = 0;
				gtk_signal_emit_by_name (GTK_OBJECT(main_gui.displayed_list),
						"key_press_event",
						&event,
						&return_val);
				if (timeoutjoy)
				  	gtk_timeout_remove(timeoutjoy);
				timeoutjoy =
				    gtk_timeout_add(10*JOY_TIMEOUT,
				  		    (GtkFunction) joy_up_repetition, NULL);
						
			} else
			{
				gxmame_debug("Joystick axis %i released",button);
				 if (timeoutjoy)
				 	 gtk_timeout_remove(timeoutjoy);
			}
		}
	}
}
		
		
void joy_left_repetition (void)
{
	gboolean return_val;
	GdkEventKey event;
	event.keyval = GDK_Page_Up;
	event.state = 0;
	gtk_signal_emit_by_name (GTK_OBJECT(main_gui.displayed_list),
			"key_press_event",
			&event,
			&return_val);
	if (timeoutjoy)
	  	gtk_timeout_remove(timeoutjoy);
	timeoutjoy = gtk_timeout_add(JOY_TIMEOUT,(GtkFunction) joy_left_repetition, NULL);
}

void joy_right_repetition (void)
{
	gboolean return_val;
	GdkEventKey event;
	event.keyval = GDK_Page_Down;
	event.state = 0;
	gtk_signal_emit_by_name (GTK_OBJECT(main_gui.displayed_list),
			"key_press_event",
			&event,
			&return_val);
	if (timeoutjoy)
	  	gtk_timeout_remove(timeoutjoy);
	timeoutjoy = gtk_timeout_add(JOY_TIMEOUT,(GtkFunction) joy_right_repetition, NULL);
}

void joy_up_repetition (void)
{
	gboolean return_val;
	GdkEventKey event;
	event.keyval = GDK_Up;
	event.state = 0;
	gtk_signal_emit_by_name (GTK_OBJECT(main_gui.displayed_list),
			"key_press_event",
			&event,
			&return_val);
	if (timeoutjoy)
	  	gtk_timeout_remove(timeoutjoy);
	timeoutjoy = gtk_timeout_add(JOY_TIMEOUT,(GtkFunction) joy_up_repetition, NULL);
}

void joy_down_repetition (void)
{
	gboolean return_val;
	GdkEventKey event;
	event.keyval = GDK_Down;
	event.state = 0;
	gtk_signal_emit_by_name (GTK_OBJECT(main_gui.displayed_list),
			"key_press_event",
			&event,
			&return_val);
	if (timeoutjoy)
	  	gtk_timeout_remove(timeoutjoy);
	timeoutjoy = gtk_timeout_add(JOY_TIMEOUT,(GtkFunction) joy_down_repetition, NULL);
}


void joy_focus_on(void)
{
	joystick_focus = TRUE;
}
void joy_focus_off(void)
{
	joystick_focus = FALSE;
}
/* old version of repetition joykeys*/
/*
void
select_row_down (void)
{
	gint row;
	if (visible_games>0)
		row = gui_prefs.DefaultGame_p->position;
	else 
		row=-1;
	if ((row < visible_games-1) && visible_games>0)
	{
		gtk_clist_select_row(GTK_CLIST(main_gui.displayed_list),row+1,0);
		if(gtk_clist_row_is_visible(GTK_CLIST(main_gui.displayed_list),row+1)!=GTK_VISIBILITY_FULL)
			gtk_clist_moveto(GTK_CLIST(main_gui.displayed_list),row+1,0,1,0);
		gui_prefs.DefaultGame_p = gtk_clist_get_row_data(GTK_CLIST(main_gui.displayed_list),row+1);
				if (timeoutjoy)
					gtk_timeout_remove(timeoutjoy);
				timeoutjoy =
				    gtk_timeout_add(JOY_TIMEOUT,
						    (GtkFunction) select_row_down, NULL);
	} else if (timeoutjoy)
	{
		gtk_timeout_remove(timeoutjoy);
	}
	
}


void
select_row_up (int new_row)
{
	if ((new_row < visible_games-1) && visible_games>0)
	{
		gtk_clist_select_row(GTK_CLIST(main_gui.displayed_list),new_row-1,0);
		if(gtk_clist_row_is_visible(GTK_CLIST(main_gui.displayed_list),new_row-1)!=GTK_VISIBILITY_FULL)
			gtk_clist_moveto(GTK_CLIST(main_gui.displayed_list),new_row-1,0,0,0);
				if (timeoutjoy)
					gtk_timeout_remove(timeoutjoy);
				timeoutjoy =
				    gtk_timeout_add(JOY_TIMEOUT,
						    (GtkFunction) select_row_up,GINT_TO_POINTER(new_row-1) );
	} else if (timeoutjoy)
	{
		gtk_timeout_remove(timeoutjoy);
	}
	
}
*/
