/*
 * GXMame
 *
 * 2002, 2003, Stphane Pontier <shadow_walker@users.sourceforge.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "gxmame.h"
#include "gui.h"
#include "mameio.h"
#include "icones.h"
#include "progression_window.h"
#include <stdio.h>
#include "unzip.h"

#include "io.h"

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gdk-pixbuf/gdk-pixbuf-loader.h>

void create_Toolbar (void);
void select_game(RomEntry *rom);
gboolean set_game_history(const RomEntry *rom);
gboolean set_game_info(const RomEntry *rom);
void update_screenshot_panel(RomEntry *rom);

GList *xmame_executables_combo_items = NULL;
GtkWidget *xmame_executables_entry;
GSList *exec_group_group = NULL;

guint timeout_icon;

/* these two prototype are declared here cause of the cross referencing that allows menu to be updated when drop box change and vice-versa */
/* executable changed, called when text is changed in executable dropbox (or by on_executable_selected() when changed from the menu) */
void xecutable_changed(void);
/* executable selected from the menu */
void
on_executable_selected			(GtkCheckMenuItem	*menuitem,
					 gpointer		 user_data);

gboolean adjustement_scrolled_delayed          (void)
{
	RomEntry *tmprom;
	GList *listpointer;
	gchar **Texte;
	GtkCTreeNode *node;

	GdkPixmap *testpix = NULL;
	GdkPixmap *testmask = NULL;
	ZIP *zip;
	gchar *zipfile;
	Texte = g_malloc(sizeof(gchar *));
	
	/*open the zip file only at the begining */
	if(gui_prefs.current_mode == INDENTED)
	{
		zipfile = g_strdup_printf("%sicons.zip",gui_prefs.IconDirectory);
	        if(!access(zipfile,R_OK))
	        {
			zip = openzip(zipfile);
	        }
		else
			zip= NULL;
	
	
		for (listpointer = g_list_first(roms);(listpointer != NULL);listpointer = g_list_next(listpointer))
		{
			tmprom = (RomEntry *) listpointer->data;
			tmprom->position = gtk_clist_find_row_from_data(GTK_CLIST(main_gui.displayed_list),tmprom);
			if (gtk_clist_row_is_visible(GTK_CLIST(main_gui.displayed_list),tmprom->position) != GTK_VISIBILITY_NONE &&
					tmprom->has_roms == 1 &&
					tmprom->status &&
					tmprom->icon_pixbuf == NULL)
			{
				node = gtk_ctree_node_nth(GTK_CTREE(main_gui.displayed_list),tmprom->position);
				if (tmprom->icon_pixbuf == NULL)
				{
					tmprom->icon_pixbuf = get_icon_for_rom(tmprom, gui_prefs.ListFontHeight, zip);
				}
				if (tmprom->icon_pixbuf != NULL)
				{
					gdk_pixbuf_render_pixmap_and_mask(tmprom->icon_pixbuf,
								&testpix,
								&testmask,
								127);
					gtk_ctree_node_get_pixtext(GTK_CTREE(main_gui.displayed_list),
								node,
 								0,
 								Texte,
 								NULL,
 								NULL,
 								NULL);
					gtk_ctree_node_set_pixtext(GTK_CTREE(main_gui.displayed_list),
									node,
 									0,
 									Texte[0],
 									5,
 									testpix,
 									testmask);
				}
			}
		}
		if (zip)
			closezip(zip);
		g_free(zipfile);
	}
	else
	{
		zipfile = g_strdup_printf("%sicons.zip",gui_prefs.IconDirectory);
	        if(!access(zipfile,R_OK))
	        {
			zip = openzip(zipfile);
	        }
		else
			zip= NULL;
	
	
		for (listpointer = g_list_first(roms);(listpointer != NULL);listpointer = g_list_next(listpointer))
		{
			tmprom = (RomEntry *) listpointer->data;
			tmprom->position = gtk_clist_find_row_from_data(GTK_CLIST(main_gui.displayed_list),tmprom);
			if (gtk_clist_row_is_visible(GTK_CLIST(main_gui.displayed_list),tmprom->position) != GTK_VISIBILITY_NONE &&
					tmprom->has_roms == 1 &&
					tmprom->status &&
					tmprom->icon_pixbuf == NULL)
			{
				if (tmprom->icon_pixbuf == NULL)
				{
					tmprom->icon_pixbuf = get_icon_for_rom(tmprom, gui_prefs.ListFontHeight, zip);
				}
				if (tmprom->icon_pixbuf != NULL)
				{
					gdk_pixbuf_render_pixmap_and_mask(tmprom->icon_pixbuf,
								&testpix,
								&testmask,
								127);
					gtk_clist_get_pixtext(GTK_CLIST(main_gui.displayed_list),
					              tmprom->position,
					              0,
					              Texte,
					              NULL,
					              NULL,
					              NULL);
					gtk_clist_set_pixtext(GTK_CLIST(main_gui.displayed_list),
					              tmprom->position,
					              0,
					              Texte[0],
					              5,
					              testpix,
					              testmask);
				}
			}
		}
		if (zip)
			closezip(zip);
		g_free(zipfile);
	}
	g_free(Texte);
	return FALSE;
}

void adjustement_scrolled                  (GtkAdjustment *adjustment,
                                            gpointer user_data)
{
		if (timeout_icon)
			gtk_timeout_remove(timeout_icon);
		timeout_icon =
		    gtk_timeout_add(ICON_TIMEOUT,
				    (GtkFunction) adjustement_scrolled_delayed, NULL);

}
	
	
void init_gui()
{
	GdkPixmap *testpix = NULL;
	GdkPixmap *testmask = NULL;
	GdkColor transparent;
	char *window_title;
	gchar *message;
	GtkWidget *Loading_error = NULL;
	
	GtkAccelGroup *popup_menu_accels;
	guint tmp_key;
	GtkWidget *separator0;
	GtkWidget *separator1;
	GtkWidget *separator2;
	GtkWidget *select_random_game;
	GtkWidget *play_record_menu;
	GtkWidget *playback_menu;
	GtkWidget *separator3;
	GtkWidget *properties;
	GtkAccelGroup *accel_group;
	GtkTooltips *tooltips;

	tooltips = gtk_tooltips_new ();

	accel_group = gtk_accel_group_new ();

	MainWindow = create_MainWindow ();
	/* realize the main window to be able to create pixmaps with colormaps */
	gtk_widget_realize(MainWindow);
	
	if (list_version.version==NULL)
	{
		Loading_error = GXMame_error_message(ERROR,_("problem to recognise the gamelist version \nyou'd better build a new one\ngo to Option and rebuild a game list"));
	} else if (!strcmp(list_version.version,"none"))
	{
		Loading_error = GXMame_error_message(ERROR,_("Gamelist not available,\nyou need to build a new one\ngo to Option and rebuild a game list"));
	} else if (!strcmp(list_version.version,"unknown"))
	{
		Loading_error = GXMame_error_message(ERROR,_("Could not recognize format of gamelist.\nYou need to build a new gamelist."));
	} else if (!strcmp(list_version.version,"too old"))
	{
		Loading_error = GXMame_error_message(ERROR,_("Gamelist from too old GXMame\nThe format of the gamelist has changed in this version.\nYou need to build a new gamelist."));
	} else if (!strcmp(executable_version.name,"Not executable"))
	{
		message = g_strdup_printf(_("%s is not a valid executable"),mame_executable);
		Loading_error = GXMame_error_message(ERROR,message);
	} else if (gui_prefs.VersionCheck && (strcmp(executable_version.name,list_version.name)||strcmp(executable_version.version,list_version.version)))
	{
		message = g_strdup_printf(_("the gamelist is from:\n%s (%s) %s\nand the current executable is:\n%s (%s) %s\nyou may want rebuild the game list"),
				list_version.name,
				list_version.target,
				list_version.version,
				executable_version.name,
				executable_version.target,
				executable_version.version);
		Loading_error = GXMame_error_message(WARNING,message);
		g_free(message);
	}
	
	/* menu */
	main_gui.play_menu = GTK_MENU_ITEM(lookup_widget(MainWindow, "play"));
	main_gui.properties_menu = GTK_MENU_ITEM(lookup_widget(MainWindow, "properties"));
	main_gui.audit_all_games_menu = GTK_MENU_ITEM(lookup_widget(MainWindow, "audit_all_games"));
	main_gui.add_to_favorites = GTK_MENU_ITEM(lookup_widget(MainWindow, "add_to_favorites"));
	main_gui.remove_from_favorites = GTK_MENU_ITEM(lookup_widget(MainWindow, "remove_from_favorites"));
	main_gui.folder_list_menu = GTK_CHECK_MENU_ITEM(lookup_widget(MainWindow, "folder_list"));
	main_gui.screen_shot_menu = GTK_CHECK_MENU_ITEM(lookup_widget(MainWindow, "screen_shot"));
	main_gui.toolbar_view_menu = GTK_CHECK_MENU_ITEM(lookup_widget(MainWindow, "toolbar_view_menu"));
	main_gui.status_bar_view_menu = GTK_CHECK_MENU_ITEM(lookup_widget(MainWindow, "status_bar_view_menu"));
	
	main_gui.large_icons_view_menu = GTK_CHECK_MENU_ITEM(lookup_widget(MainWindow, "large_icons_view_menu"));
	main_gui.small_icons_view_menu = GTK_CHECK_MENU_ITEM(lookup_widget(MainWindow, "small_icons_view_menu"));
	main_gui.list_view_menu = GTK_CHECK_MENU_ITEM(lookup_widget(MainWindow, "list_view_menu"));
	main_gui.details_view_menu = GTK_CHECK_MENU_ITEM(lookup_widget(MainWindow, "details_view_menu"));
	main_gui.indented_view_menu = GTK_CHECK_MENU_ITEM(lookup_widget(MainWindow, "indented_view_menu"));

	main_gui.modify_the_menu = GTK_CHECK_MENU_ITEM(lookup_widget(MainWindow, "the_prefix"));
	
	main_gui.executables_title = GTK_MENU_ITEM(lookup_widget(MainWindow, "executable_title"));
	
	/* left pane (filters) */
	main_gui.hpanedLeft = GTK_PANED(lookup_widget(MainWindow, "hpanedLeft"));
	main_gui.scrolled_window_filters = GTK_WIDGET(lookup_widget(MainWindow, "scrolledwindowFilters"));
	/* central scrolled window (games list) */
	main_gui.scrolled_window_games = GTK_WIDGET(lookup_widget(MainWindow, "scrolledwindowGames"));
	main_gui.hpanedRight = GTK_PANED(lookup_widget(MainWindow, "hpanedRight"));
	main_gui.hseparator2 = GTK_WIDGET(lookup_widget(MainWindow, "hseparator2"));
	main_gui.toolbar = GTK_TOOLBAR(lookup_widget(MainWindow, "toolbar1"));
	main_gui.hseparator1 = GTK_WIDGET(lookup_widget(MainWindow, "hseparator1"));
	main_gui.ctree_filters = GTK_CTREE(lookup_widget(MainWindow, "ctreeFilters"));
	main_gui.screenshot_hist_frame = GTK_FRAME(lookup_widget(MainWindow, "ScreenShot_hist_frame"));
	main_gui.screenshot_hist_vbox = GTK_BOX(lookup_widget(MainWindow, "ScreenShot_hist_vbox"));
	main_gui.tri_status_bar = GTK_WIDGET(lookup_widget(MainWindow, "tri_status_bar"));
	main_gui.combo_progress_bar = GTK_WIDGET(lookup_widget(MainWindow, "combo_progress_bar"));
	main_gui.status_progress_bar = GTK_STATUSBAR(lookup_widget(MainWindow, "status_progress_bar"));
	main_gui.progress_progress_bar = GTK_PROGRESS_BAR(lookup_widget(MainWindow, "progress_progress_bar"));
	main_gui.statusbar1 = GTK_STATUSBAR(lookup_widget(MainWindow, "statusbar1"));
	main_gui.statusbar2 = GTK_STATUSBAR(lookup_widget(MainWindow, "statusbar2"));
	main_gui.statusbar3 = GTK_STATUSBAR(lookup_widget(MainWindow, "statusbar3"));
	
	/* need to create the menu to have all button in the toolbar ??? not really needed */
	create_Toolbar ();
	
	main_gui.filterShowButton = GTK_TOGGLE_BUTTON(lookup_widget(MainWindow, "filterShowButton"));
	main_gui.snapShowButton = GTK_TOGGLE_BUTTON(lookup_widget(MainWindow, "snapShowButton"));
	main_gui.large_icons_view_button = GTK_TOGGLE_BUTTON(lookup_widget(MainWindow, "large_icons_view_button"));
	main_gui.small_icons_view_button = GTK_TOGGLE_BUTTON(lookup_widget(MainWindow, "small_icons_view_button"));
	main_gui.list_view_button = GTK_TOGGLE_BUTTON(lookup_widget(MainWindow, "list_view_button"));
	main_gui.details_view_button = GTK_TOGGLE_BUTTON(lookup_widget(MainWindow, "details_view_button"));	
	main_gui.indented_view_button = GTK_TOGGLE_BUTTON(lookup_widget(MainWindow, "indented_view_button"));	
	
	main_gui.modify_the_button = GTK_TOGGLE_BUTTON(lookup_widget(MainWindow, "modify_the_button"));	

	/* if the ListFont is empty or not loadable, use default font */
	if (! gui_prefs.ListFont || !gdk_font_load(gui_prefs.ListFont)) {
		gxmame_debug("Could not find the font.");
		gui_prefs.ListFontStruct = NULL;
		gui_prefs.ListFontHeight = 16;
	}
	else
	{
		gint ascent,descent,lbearing,rbearing;
		gui_prefs.ListFontStruct = gdk_font_load(gui_prefs.ListFont);
		gxmame_debug("find the font.");
		gdk_string_extents(gui_prefs.ListFontStruct,
                                             "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz",
                                             &lbearing,
                                             &rbearing,
                                             NULL,
                                             &ascent,
                                             &descent);
		gui_prefs.ListFontHeight = descent + gdk_string_height(gui_prefs.ListFontStruct,
				               "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz");
	}
	
	create_filtersTreeContent();
	gtk_paned_set_position (main_gui.hpanedLeft, gui_prefs.Splitters[0]);
	gtk_paned_set_position (main_gui.hpanedRight, gui_prefs.Splitters[1]);

#ifndef DEBUG	
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.large_icons_view_menu),FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.list_view_menu),FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.large_icons_view_button),FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.list_view_button),FALSE);
#endif
	if (!strcmp(executable_version.name,"Not executable"))
	{
		gtk_widget_set_sensitive(GTK_WIDGET(main_gui.audit_all_games_menu),FALSE);
	}	
	gtk_widget_hide (GTK_WIDGET(main_gui.combo_progress_bar));
	
	/* set which items of the menu are active */
	/* need to block the call otherwise its trying to change the position but it has been already done */
	gtk_signal_handler_block_by_func(GTK_OBJECT(main_gui.modify_the_menu),GTK_SIGNAL_FUNC (on_the_prefix_activate),NULL);
	gtk_check_menu_item_set_active(main_gui.modify_the_menu,gui_prefs.ModifyThe);
	gtk_signal_handler_unblock_by_func(GTK_OBJECT(main_gui.modify_the_menu),GTK_SIGNAL_FUNC (on_the_prefix_activate),NULL);
	switch(gui_prefs.current_mode)
	{
		case(LARGE_ICONS):
			gtk_check_menu_item_set_active(main_gui.large_icons_view_menu,TRUE);
			break;
		case(SMALL_ICONS):
			gtk_check_menu_item_set_active(main_gui.small_icons_view_menu,TRUE);
			break;
		case(LIST):
			gtk_check_menu_item_set_active(main_gui.list_view_menu,TRUE);
			break;
		case(DETAILS):
			gtk_check_menu_item_set_active(main_gui.details_view_menu,TRUE);
			break;
		case(INDENTED):
			gtk_check_menu_item_set_active(main_gui.indented_view_menu,TRUE);
			break;
	}
	
	gtk_check_menu_item_set_active(main_gui.toolbar_view_menu,gui_prefs.ShowToolBar);
	gtk_check_menu_item_set_active(main_gui.status_bar_view_menu,gui_prefs.ShowStatusBar);
	main_gui.scrolled_window_filters->allocation.width = gui_prefs.Splitters[0];
	gtk_check_menu_item_set_active(main_gui.folder_list_menu,gui_prefs.ShowFolderList);
	/* need this line or GTK will allocate a value for the gamelist that will mess the screenshot display*/
	main_gui.scrolled_window_games->allocation.width = gui_prefs.Splitters[1];
	gtk_check_menu_item_set_active(main_gui.screen_shot_menu,gui_prefs.ShowScreenShot);
	
	gtk_window_set_default_size(GTK_WINDOW(MainWindow),
				    gui_prefs.GUIWidth,
				    gui_prefs.GUIHeight);
	gtk_widget_set_uposition(MainWindow,
				    gui_prefs.GUIPosX,
				    gui_prefs.GUIPosY);
	
	/* display the default picture */
	testpix = gdk_pixmap_create_from_xpm_d(MainWindow->window,
					&testmask,
					&transparent,
					gxmame_xpm);
	main_gui.main_screenshot = gtk_pixmap_new (testpix,testmask);
	main_gui.screenshoot_event_box = gtk_event_box_new();
	gtk_box_pack_start(main_gui.screenshot_hist_vbox,main_gui.screenshoot_event_box,TRUE,TRUE,5);
	gtk_container_add(GTK_CONTAINER(main_gui.screenshoot_event_box),GTK_WIDGET(main_gui.main_screenshot));
	gtk_widget_show (main_gui.screenshoot_event_box);
	gtk_widget_show (main_gui.main_screenshot);
		
	/* here we create the history box that will be filled later */
	main_gui.history_scrollwin = gtk_scrolled_window_new (NULL, NULL);
	gtk_container_set_border_width (GTK_CONTAINER (main_gui.history_scrollwin), 5);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (main_gui.history_scrollwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_box_pack_end(main_gui.screenshot_hist_vbox,main_gui.history_scrollwin,TRUE,TRUE,5);
	
	main_gui.history_box = gtk_text_new(NULL,NULL);
	gtk_text_set_word_wrap(GTK_TEXT(main_gui.history_box), TRUE);
	gtk_text_set_line_wrap(GTK_TEXT(main_gui.history_box), TRUE);
	gtk_container_add (GTK_CONTAINER (main_gui.history_scrollwin), main_gui.history_box);
	
	gtk_widget_show (main_gui.history_scrollwin);
	gtk_widget_show (main_gui.history_box);


/* build the popup menu */
	main_gui.popup_menu = GTK_MENU(gtk_menu_new ());
	gtk_widget_ref (GTK_WIDGET(main_gui.popup_menu));
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "popup_menu", main_gui.popup_menu,
				  (GtkDestroyNotify) gtk_widget_unref);

	popup_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (main_gui.popup_menu));

	main_gui.popup_play_menu = gtk_menu_item_new_with_label ("");
	tmp_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (main_gui.popup_play_menu )->child),
					 _("_Play"));
	gtk_widget_add_accelerator (main_gui.popup_play_menu, "activate_item", popup_menu_accels,
				    tmp_key, 0, 0);
	gtk_widget_ref (main_gui.popup_play_menu);
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "play", main_gui.popup_play_menu,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (main_gui.popup_play_menu);
	gtk_container_add (GTK_CONTAINER (main_gui.popup_menu), main_gui.popup_play_menu);
	gtk_tooltips_set_tip (tooltips, main_gui.popup_play_menu, _("Play currently selected game"), NULL);
	gtk_widget_add_accelerator (main_gui.popup_play_menu, "activate", accel_group,
				    GDK_Return, 0, GTK_ACCEL_LOCKED);
  	gtk_signal_connect (GTK_OBJECT (main_gui.popup_play_menu), "activate",
                      	    GTK_SIGNAL_FUNC (on_play_activate),
                            NULL);

	separator0 = gtk_menu_item_new ();
	gtk_widget_ref (separator0);
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "separator0", separator0,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (separator0);
	gtk_container_add (GTK_CONTAINER (main_gui.popup_menu), separator0);
	gtk_widget_set_sensitive (separator0, FALSE);

	play_record_menu = gtk_menu_item_new_with_label ("");
	tmp_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (play_record_menu)->child),
					 _("Play and _Record Input..."));
	gtk_widget_add_accelerator (play_record_menu, "activate_item", popup_menu_accels,
				    tmp_key, 0, GTK_ACCEL_LOCKED);
	gtk_widget_ref (play_record_menu);
	gtk_widget_show (play_record_menu);
	gtk_container_add (GTK_CONTAINER (main_gui.popup_menu), play_record_menu);
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "play_record_menu", play_record_menu,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_tooltips_set_tip (tooltips, play_record_menu, _("Record a game for later playback"), NULL);
	gtk_signal_connect (GTK_OBJECT (play_record_menu), "activate",
			    GTK_SIGNAL_FUNC (on_play_and_record_input_activate),
			    NULL);

	playback_menu = gtk_menu_item_new_with_label ("");
	tmp_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (playback_menu)->child),
					 _("P_layback Input..."));
	gtk_widget_add_accelerator (playback_menu, "activate_item", popup_menu_accels,
				    tmp_key, 0, GTK_ACCEL_LOCKED);
	gtk_widget_ref (playback_menu);
	gtk_widget_show (playback_menu);
	gtk_container_add (GTK_CONTAINER (main_gui.popup_menu), playback_menu);
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "playback_menu", playback_menu,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_tooltips_set_tip (tooltips, playback_menu, _("Playback a recorded game"), NULL);
	gtk_signal_connect (GTK_OBJECT (playback_menu), "activate",
			    GTK_SIGNAL_FUNC (on_playback_input_activate),
			    NULL);

	separator1 = gtk_menu_item_new ();
	gtk_widget_ref (separator1);
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "separator1", separator1,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (separator1);
	gtk_container_add (GTK_CONTAINER (main_gui.popup_menu), separator1);
	gtk_widget_set_sensitive (separator1, FALSE);

	main_gui.popup_add_to_favorites = gtk_menu_item_new_with_label ("");
	tmp_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (main_gui.popup_add_to_favorites)->child),
					 _("A_dd to 'Favorites'"));
	gtk_widget_add_accelerator (main_gui.popup_add_to_favorites, "activate_item", popup_menu_accels,
				    tmp_key, 0, 0);
	gtk_widget_ref (main_gui.popup_add_to_favorites);
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "add_to_favorites", main_gui.popup_add_to_favorites,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (main_gui.popup_add_to_favorites);
	gtk_container_add (GTK_CONTAINER (main_gui.popup_menu), main_gui.popup_add_to_favorites);
	gtk_tooltips_set_tip (tooltips, main_gui.popup_add_to_favorites, _("Add this game to your 'Favorites' game folder"), NULL);
	gtk_signal_connect (GTK_OBJECT (main_gui.popup_add_to_favorites), "activate",
			    GTK_SIGNAL_FUNC (on_add_to_favorites_activate),
			    NULL);

	main_gui.popup_remove_from_favorites = gtk_menu_item_new_with_label ("");
	tmp_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (main_gui.popup_remove_from_favorites)->child),
					 _("R_emove from 'Favorites'"));
	gtk_widget_add_accelerator (main_gui.popup_remove_from_favorites, "activate_item", popup_menu_accels,
				    tmp_key, 0, 0);
	gtk_widget_ref (main_gui.popup_remove_from_favorites);
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "remove_from_favorites", main_gui.popup_remove_from_favorites,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (main_gui.popup_remove_from_favorites);
	gtk_container_add (GTK_CONTAINER (main_gui.popup_menu), main_gui.popup_remove_from_favorites);
	gtk_tooltips_set_tip (tooltips, main_gui.popup_remove_from_favorites, _("Remove this game from your 'Favorites' game folder"), NULL);
	gtk_signal_connect (GTK_OBJECT (main_gui.popup_remove_from_favorites), "activate",
			    GTK_SIGNAL_FUNC (on_remove_from_favorites_activate),
			    NULL);

	separator2 = gtk_menu_item_new ();
	gtk_widget_ref (separator2);
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "separator2", separator2,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (separator2);
	gtk_container_add (GTK_CONTAINER (main_gui.popup_menu), separator2);
	gtk_widget_set_sensitive (separator2, FALSE);

	select_random_game = gtk_menu_item_new_with_label ("");
	tmp_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (select_random_game)->child),
					 _("_Select Random Game"));
	gtk_widget_add_accelerator (select_random_game, "activate_item", popup_menu_accels,
				    tmp_key, 0, 0);
	gtk_widget_ref (select_random_game);
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "select_random_game", select_random_game,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (select_random_game);
	gtk_container_add (GTK_CONTAINER (main_gui.popup_menu), select_random_game);
	gtk_tooltips_set_tip (tooltips, select_random_game, _("Randomly select a game"), NULL);
	gtk_signal_connect (GTK_OBJECT (select_random_game), "activate",
			    GTK_SIGNAL_FUNC (on_select_random_game_activate),
			    NULL);

	separator3 = gtk_menu_item_new ();
	gtk_widget_ref (separator3);
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "separator3", separator3,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (separator3);
	gtk_container_add (GTK_CONTAINER (main_gui.popup_menu), separator3);
	gtk_widget_set_sensitive (separator3, FALSE);

	properties = gtk_menu_item_new_with_label ("");
	tmp_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (properties)->child),
					 _("Pr_operties"));
	gtk_widget_add_accelerator (properties, "activate_item", popup_menu_accels,
				    tmp_key, 0, 0);
	gtk_widget_ref (properties);
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "properties", properties,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (properties);
	gtk_container_add (GTK_CONTAINER (main_gui.popup_menu), properties);
	gtk_tooltips_set_tip (tooltips, properties, _("Display the properties of the selected items"), NULL);
	gtk_signal_connect (GTK_OBJECT (properties), "activate",
			    GTK_SIGNAL_FUNC (on_properties_activate),
			    NULL);
	
	
	/* Main window title and icon */	
	if (!strcmp(executable_version.name,"Not executable"))
	{
		window_title = g_strdup_printf(_("GXMame V%s"),
					VERSION);
		gtk_window_set_title (GTK_WINDOW (MainWindow), window_title);
	}
	else
	{
		window_title = g_strdup_printf(_("GXMame V%s running %s (%s) %s"),
					VERSION,
					executable_version.name,
					executable_version.target,
					executable_version.version);
		gtk_window_set_title (GTK_WINDOW (MainWindow), window_title);
	}
	g_free(window_title);
	set_icon(MainWindow,"GXMame");
	
	/* need to show the window before creating the CList
	   or the first moveto wont apear correctly */
	gtk_widget_show (MainWindow);
	if (Loading_error != NULL )
		gdk_window_raise(Loading_error->window);
	
	switch (gui_prefs.current_mode)
	{
		case (LARGE_ICONS):
			gxmame_debug("large icons mode not implemented yet");
			break;
		case (SMALL_ICONS):
			gxmame_debug("Display small icons mode");
			display_small_icon_list();
			break;
		case (LIST):
			gxmame_debug("list mode not implemented yet");
			gxmame_debug("test of indented list using clist not ctree");
			display_indented2_list();
			break;
		case (DETAILS):
			gxmame_debug("Display details mode");
			display_details_list();
			break;
		case (INDENTED):
			gxmame_debug("Display indented mode");
			display_indented_list();
			break;
	}	

	/*need to set the size here otherwise it move when we create the gamelist*/
	if (gui_prefs.ShowScreenShot)
		gtk_paned_set_position (main_gui.hpanedRight, gui_prefs.Splitters[1]);
	gtk_widget_grab_focus(main_gui.displayed_list);
	
        gtk_signal_connect (GTK_OBJECT (gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(main_gui.scrolled_window_games))), "changed",
        		    GTK_SIGNAL_FUNC (adjustement_scrolled),
        		    NULL);
        gtk_signal_connect (GTK_OBJECT (gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(main_gui.scrolled_window_games))), "value-changed",
        		    GTK_SIGNAL_FUNC (adjustement_scrolled),
        		    NULL);
	

}

/* select the user value from the dropbox and store it in the mame_executable variable */
void
xmame_executables_combo_comp (gchar *key,gchar *value)
{	
	gchar *used_text;
	used_text = gtk_editable_get_chars(GTK_EDITABLE(xmame_executables_entry),0,-1);
	if (!strcmp(value,used_text))
	{
		GtkWidget *temp_menu_item;
		g_free(mame_executable);
		mame_executable = g_strdup(key);
		temp_menu_item = lookup_widget(MainWindow, value);
		gtk_signal_handler_block_by_func(GTK_OBJECT(temp_menu_item),GTK_SIGNAL_FUNC (on_executable_selected),NULL);
		/* select the executable in the menu */
		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (temp_menu_item), TRUE);	
		gtk_signal_handler_unblock_by_func(GTK_OBJECT(temp_menu_item),GTK_SIGNAL_FUNC (on_executable_selected),NULL);
	}
	g_free(used_text);
}

/* executable changed, called when text is changed in executable dropbox (or by on_executable_selected() when changed from the menu) */
void xecutable_changed(void)
{
	if (xmame_executables_hash_table)
	{
		gchar *message;
		g_hash_table_foreach(xmame_executables_hash_table,
             	                (GHFunc)xmame_executables_combo_comp,
             	                NULL);
		gxmame_debug("executable change to %s",mame_executable);
		
		/* check if the executable is still valid and set versions.* variables */
		get_mame_version(mame_executable,EXECUTABLE);
		if (!strcmp(executable_version.name,"Not executable"))
		{
			message = g_strdup_printf(_("%s is not a valid executable"),mame_executable);
			GXMame_error_message(ERROR,message);
			g_free(message);
			message = g_strdup_printf(_("GXMame V%s"),
						VERSION);
			gtk_window_set_title (GTK_WINDOW (MainWindow), message);
			gtk_widget_set_sensitive(GTK_WIDGET(main_gui.audit_all_games_menu),FALSE);
			g_free(message);
		} else
		{
			if (list_version.version==NULL || !strcmp(list_version.version,"unknown") || !strcmp(list_version.version,"none"))
			{
				GXMame_error_message(ERROR,_("Gamelist not available,\nyou need to build a new one\ngo to Option and rebuild a game list"));
			} else if (gui_prefs.VersionCheck && (strcmp(executable_version.name,list_version.name)||strcmp(executable_version.version,list_version.version)))
			{
				message = g_strdup_printf(_("the gamelist is from:\n%s (%s) %s\nand the current executable is:\n%s (%s) %s\nyou may want rebuild the game list"),
						list_version.name,
						list_version.target,
						list_version.version,
						executable_version.name,
						executable_version.target,
						executable_version.version);
				GXMame_error_message(WARNING,message);
				g_free(message);
			}
			message = g_strdup_printf(_("GXMame V%s running %s (%s) %s"),
						VERSION,
						executable_version.name,
						executable_version.target,
						executable_version.version);
	
			gtk_window_set_title (GTK_WINDOW (MainWindow), message);
			gtk_widget_set_sensitive(GTK_WIDGET(main_gui.audit_all_games_menu),TRUE);
			g_free(message);
	
			/* get mame options */
			get_mame_options();
		}
	}
}

/* executable selected from the menu */
void
on_executable_selected			(GtkCheckMenuItem	*menuitem,
					 gpointer		 user_data)
{
	gchar *temp_text;
	if (menuitem->active)
	{
		gtk_label_get(GTK_LABEL(GTK_BIN(menuitem)->child),&temp_text);
		gxmame_debug("selected: %s",temp_text);
		xmame_executables_entry = GTK_COMBO (main_gui.xmame_executables_combo)->entry;
		/* need to block the handler here and call the xecutable_changed() by hand otherwise, it's been called twice */
		gtk_signal_handler_block_by_func(GTK_OBJECT(xmame_executables_entry),GTK_SIGNAL_FUNC (xecutable_changed),NULL);
		gtk_entry_set_text (GTK_ENTRY (xmame_executables_entry), temp_text);
		xecutable_changed();
		gtk_signal_handler_unblock_by_func(GTK_OBJECT(xmame_executables_entry),GTK_SIGNAL_FUNC (xecutable_changed),NULL);
	}
	else
	{
		gtk_label_get(GTK_LABEL(GTK_BIN(menuitem)->child),&temp_text);
		gxmame_debug("unselected: %s",temp_text);
	}
	
}

/* add executable choices to dsp dropbox option */
void
xmame_executables_combo_add (gchar *key,gchar *value)
{
	GtkWidget *temp_menu_item;
	
	/* to the combo box */
	xmame_executables_combo_items = g_list_append (xmame_executables_combo_items, (gpointer)g_strdup(value));
	
	/* to the menu */
	temp_menu_item = gtk_radio_menu_item_new_with_label (exec_group_group, (gpointer)g_strdup(value));
	exec_group_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (temp_menu_item));
	gtk_widget_ref (temp_menu_item);
	/* the menu is  referenced by the name of the executable */
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), g_strdup(value), temp_menu_item,
	 			  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (temp_menu_item);
	gtk_container_add (GTK_CONTAINER (main_gui.executable_menu), temp_menu_item);
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (temp_menu_item), !strcmp(key,mame_executable));
	gtk_signal_connect (GTK_OBJECT (temp_menu_item), "activate",
			GTK_SIGNAL_FUNC (on_executable_selected),
			NULL);

	gxmame_debug("\t %s exec added checked to %i",value,!strcmp(key,mame_executable));
}


void add_exec_dropbox(void)
{
	/* recreate the menu */
	if (main_gui.executable_menu==NULL)
		main_gui.executable_menu = gtk_menu_new ();
	else
	{
		gtk_container_foreach(GTK_CONTAINER (main_gui.executable_menu),(GtkCallback)gtk_widget_destroy,NULL);
		/* need to set the GList to NULL otherwise segfault as soon as I set new executables 
		   are all the other really needed ? */
		gtk_menu_item_remove_submenu (GTK_MENU_ITEM(main_gui.executables_title));
		gtk_widget_destroy(main_gui.executable_menu);
		main_gui.executable_menu = gtk_menu_new ();
		exec_group_group = NULL;
	}
  gtk_widget_ref (GTK_WIDGET(main_gui.executable_menu));
  gtk_object_set_data_full (GTK_OBJECT (MainWindow), "executable_menu", main_gui.executable_menu,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (main_gui.executables_title), main_gui.executable_menu);
  
	/* recreate the combo box */
	main_gui.xmame_executables_combo = gtk_combo_new ();
	gtk_widget_ref (main_gui.xmame_executables_combo);
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "xmame_executables_combo", main_gui.xmame_executables_combo,
				  (GtkDestroyNotify) gtk_widget_unref);
	/* should not use gtk_combo_set_value_in_list otherwise, is the entry has not a good value, we cant change directories*/
	/* gtk_combo_set_value_in_list (GTK_COMBO (xmame_executables_combo), TRUE, FALSE);*/
	gtk_widget_set_usize (main_gui.xmame_executables_combo, 270, -2);
	/* need to set the glist null here otherwise, got an strange element that will crach the app */
	xmame_executables_combo_items = NULL;
	if(xmame_executables_hash_table) 
	{
		g_hash_table_foreach(xmame_executables_hash_table,
        	                     (GHFunc)xmame_executables_combo_add,
        	                     NULL);
		/* test because we can have an existing but empty xmame_executables_hash_table,
		   that lead to a empty xmame_executables_combo_items */
		if (xmame_executables_combo_items!=NULL)
		{
			gtk_combo_set_popdown_strings (GTK_COMBO (main_gui.xmame_executables_combo), xmame_executables_combo_items);
			g_list_free (xmame_executables_combo_items);
		}
	}
	gtk_toolbar_append_widget (main_gui.toolbar, main_gui.xmame_executables_combo, _("Executable"), NULL);
	gtk_widget_show (main_gui.xmame_executables_combo);
		
	xmame_executables_entry = GTK_COMBO (main_gui.xmame_executables_combo)->entry;
	gtk_entry_set_editable (GTK_ENTRY (xmame_executables_entry), FALSE);
	gtk_widget_ref (xmame_executables_entry);
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "xmame_executables_entry", xmame_executables_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (xmame_executables_entry);

	/* xmame_executable should be correct since we ve set it in check_executable_list function (mameio.c)*/
	gtk_entry_set_text (GTK_ENTRY (xmame_executables_entry), mame_executable);
	/* if no valid exec was found, the table contain nothing or
	   if the mame_executable is not executable, the hashtable wont contain it,
	   we must test if the mame_executable is in the hash table */
	if (g_hash_table_lookup(xmame_executables_hash_table,mame_executable))
		gtk_entry_set_text (GTK_ENTRY (xmame_executables_entry), g_hash_table_lookup(xmame_executables_hash_table,mame_executable));
	/* If i'm not letting GTK runs here, I got a GTK-warning 
	   and if I try to open directory window then default option window, 
	   GTK lose a reference to a window and segfault */
	while (gtk_events_pending()) gtk_main_iteration();
        gtk_signal_connect (GTK_OBJECT (xmame_executables_entry), "changed",
        		    GTK_SIGNAL_FUNC (xecutable_changed),
        		    NULL);
}
void create_Toolbar (void)
{
	GtkWidget *tmp_toolbar_icon;
	GtkWidget *filterShowButton;
	GtkWidget *snapShowButton;
	GtkWidget *large_icons_view_button;
	GtkWidget *small_icons_view_button;
	GtkWidget *list_view_button;
	GtkWidget *details_view_button;
	GtkWidget *indented_view_button;
	GtkWidget *modify_the_button;
	GtkWidget *aboutButton;
	
	GdkPixmap *tmp_pixmap_icon;
	GdkBitmap *tmp_mask_icon;
	GdkColor  transparent;
	

/* addition of toolbar buttons */
/* filters and snapshots buttons */
	tmp_pixmap_icon = gdk_pixmap_create_from_xpm_d(MainWindow->window,
					&tmp_mask_icon,
					&transparent,
					filtermenu_xpm);
	tmp_toolbar_icon = gtk_pixmap_new (tmp_pixmap_icon,tmp_mask_icon);
        filterShowButton = gtk_toolbar_append_element (main_gui.toolbar,
        			      GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
        			      NULL,
        			      _("Show Filters"),
        			      _("Toggle Folder List"), NULL,
        			      tmp_toolbar_icon, NULL, NULL);
        gtk_widget_ref (filterShowButton);
        gtk_object_set_data_full (GTK_OBJECT (MainWindow), "filterShowButton", filterShowButton,
        			  (GtkDestroyNotify) gtk_widget_unref);
        gtk_widget_show (filterShowButton);
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (filterShowButton), gui_prefs.ShowFolderList);

	tmp_pixmap_icon = gdk_pixmap_create_from_xpm_d(MainWindow->window,
					&tmp_mask_icon,
					&transparent,
					snapmenu_xpm);
	tmp_toolbar_icon = gtk_pixmap_new (tmp_pixmap_icon,tmp_mask_icon);
        snapShowButton = gtk_toolbar_append_element (main_gui.toolbar,
        			      GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
        			      NULL,
        			      _("Show Snaps"),
        			      _("Toggle Screen Shot"), NULL,
        			      tmp_toolbar_icon, NULL, NULL);
        gtk_widget_ref (snapShowButton);
        gtk_object_set_data_full (GTK_OBJECT (MainWindow), "snapShowButton", snapShowButton,
        			  (GtkDestroyNotify) gtk_widget_unref);
        gtk_widget_show (snapShowButton);
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (snapShowButton), gui_prefs.ShowScreenShot);

        gtk_toolbar_append_space (main_gui.toolbar);

/* listing mode buttons */
	tmp_pixmap_icon = gdk_pixmap_create_from_xpm_d(MainWindow->window,
					&tmp_mask_icon,
					&transparent,
					viewlarge_xpm);
	tmp_toolbar_icon = gtk_pixmap_new (tmp_pixmap_icon,tmp_mask_icon);
        large_icons_view_button = gtk_toolbar_append_element (main_gui.toolbar,
        			      GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
        			      NULL,
        			      _("Large Icons"),
        			      _("Large Icons"), NULL,
        			      tmp_toolbar_icon, NULL, NULL);
        gtk_widget_ref (large_icons_view_button);
        gtk_object_set_data_full (GTK_OBJECT (MainWindow), "large_icons_view_button", large_icons_view_button,
        			  (GtkDestroyNotify) gtk_widget_unref);
        gtk_widget_show (large_icons_view_button);

	tmp_pixmap_icon = gdk_pixmap_create_from_xpm_d(MainWindow->window,
					&tmp_mask_icon,
					&transparent,
					viewsmall_xpm);
	tmp_toolbar_icon = gtk_pixmap_new (tmp_pixmap_icon,tmp_mask_icon);
        small_icons_view_button = gtk_toolbar_append_element (main_gui.toolbar,
        			      GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
        			      NULL,
        			      _("Small Icons"),
        			      _("Small Icons"), NULL,
        			      tmp_toolbar_icon, NULL, NULL);
        gtk_widget_ref (small_icons_view_button);
        gtk_object_set_data_full (GTK_OBJECT (MainWindow), "small_icons_view_button", small_icons_view_button,
        			  (GtkDestroyNotify) gtk_widget_unref);
        gtk_widget_show (small_icons_view_button);

	tmp_pixmap_icon = gdk_pixmap_create_from_xpm_d(MainWindow->window,
					&tmp_mask_icon,
					&transparent,
					viewlist_xpm);
	tmp_toolbar_icon = gtk_pixmap_new (tmp_pixmap_icon,tmp_mask_icon);
        list_view_button = gtk_toolbar_append_element (main_gui.toolbar,
        			      GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
        			      NULL,
        			      _("List"),
        			      _("List"), NULL,
        			      tmp_toolbar_icon, NULL, NULL);
        gtk_widget_ref (list_view_button);
        gtk_object_set_data_full (GTK_OBJECT (MainWindow), "list_view_button", list_view_button,
        			  (GtkDestroyNotify) gtk_widget_unref);
        gtk_widget_show (list_view_button);

	tmp_pixmap_icon = gdk_pixmap_create_from_xpm_d(MainWindow->window,
					&tmp_mask_icon,
					&transparent,
					viewdetail_xpm);
	tmp_toolbar_icon = gtk_pixmap_new (tmp_pixmap_icon,tmp_mask_icon);
        details_view_button = gtk_toolbar_append_element (main_gui.toolbar,
        			      GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
        			      NULL,
        			      _("Details"),
        			      _("Details"), NULL,
        			      tmp_toolbar_icon, NULL, NULL);
        gtk_widget_ref (details_view_button);
        gtk_object_set_data_full (GTK_OBJECT (MainWindow), "details_view_button", details_view_button,
        			  (GtkDestroyNotify) gtk_widget_unref);
        gtk_widget_show (details_view_button);

	tmp_pixmap_icon = gdk_pixmap_create_from_xpm_d(MainWindow->window,
					&tmp_mask_icon,
					&transparent,
					viewident_xpm);
	tmp_toolbar_icon = gtk_pixmap_new (tmp_pixmap_icon,tmp_mask_icon);
        indented_view_button = gtk_toolbar_append_element (main_gui.toolbar,
        			      GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
        			      NULL,
        			      _("Details"),
        			      _("Details"), NULL,
        			      tmp_toolbar_icon, NULL, NULL);
        gtk_widget_ref (indented_view_button);
        gtk_object_set_data_full (GTK_OBJECT (MainWindow), "indented_view_button", indented_view_button,
        			  (GtkDestroyNotify) gtk_widget_unref);
        gtk_widget_show (indented_view_button);

/* init the mode button */	
	switch(gui_prefs.current_mode)
	{
		case(LARGE_ICONS):
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(large_icons_view_button), TRUE);
			break;
 		case(SMALL_ICONS):
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(small_icons_view_button), TRUE);
			break;
 		case(LIST):
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(list_view_button), TRUE);
			break;
 		case(DETAILS):
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(details_view_button), TRUE);
			break;
 		case(INDENTED):
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(indented_view_button), TRUE);
			break;
	}

        gtk_toolbar_append_space (main_gui.toolbar);

/* the trailer button */
	tmp_pixmap_icon = gdk_pixmap_create_from_xpm_d(MainWindow->window,
					&tmp_mask_icon,
					&transparent,
					thetrail_xpm);
	tmp_toolbar_icon = gtk_pixmap_new (tmp_pixmap_icon,tmp_mask_icon);
        modify_the_button = gtk_toolbar_append_element (main_gui.toolbar,
        			      GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
        			      NULL,
        			      _("Modify The"),
        			      _("Modify The"), NULL,
        			      tmp_toolbar_icon, NULL , NULL);
        gtk_widget_ref (modify_the_button);
        gtk_object_set_data_full (GTK_OBJECT (MainWindow), "modify_the_button", modify_the_button,
        			  (GtkDestroyNotify) gtk_widget_unref);
        gtk_widget_show (modify_the_button);
	
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(modify_the_button), gui_prefs.ModifyThe);


        gtk_toolbar_append_space (main_gui.toolbar);

/* About button */
	tmp_pixmap_icon = gdk_pixmap_create_from_xpm_d(MainWindow->window,
					&tmp_mask_icon,
					&transparent,
					about_xpm);
	tmp_toolbar_icon = gtk_pixmap_new (tmp_pixmap_icon,tmp_mask_icon);
	gdk_pixmap_unref(tmp_pixmap_icon);
	gdk_pixmap_unref(tmp_mask_icon);
        aboutButton = gtk_toolbar_append_element (main_gui.toolbar,
        			      GTK_TOOLBAR_CHILD_BUTTON,
        			      NULL,
        			      _("About"),
        			      _("About"), NULL,
        			      tmp_toolbar_icon, NULL, NULL);
        gtk_widget_ref (aboutButton);
        gtk_object_set_data_full (GTK_OBJECT (MainWindow), "aboutButton", aboutButton,
        			  (GtkDestroyNotify) gtk_widget_unref);
        gtk_widget_show (aboutButton);
 
/* executable dropbox*/
	gtk_toolbar_append_space (main_gui.toolbar);
	
	add_exec_dropbox();

/* Connection of toolbar buttons signals*/
        gtk_signal_connect (GTK_OBJECT (filterShowButton), "toggled",
        		    GTK_SIGNAL_FUNC (on_filterShowButton_toggled),
        		    NULL);
        gtk_signal_connect (GTK_OBJECT (snapShowButton), "toggled",
        		    GTK_SIGNAL_FUNC (on_snapShowButton_toggled),
        		    NULL);
        gtk_signal_connect (GTK_OBJECT (large_icons_view_button), "clicked",
        		    GTK_SIGNAL_FUNC (on_mode_button_clicked),
        		    GINT_TO_POINTER(LARGE_ICONS));
        gtk_signal_connect (GTK_OBJECT (small_icons_view_button), "clicked",
        		    GTK_SIGNAL_FUNC (on_mode_button_clicked),
        		    GINT_TO_POINTER(SMALL_ICONS));
        gtk_signal_connect (GTK_OBJECT (list_view_button), "clicked",
        		    GTK_SIGNAL_FUNC (on_mode_button_clicked),
        		    GINT_TO_POINTER(LIST));
        gtk_signal_connect (GTK_OBJECT (details_view_button), "clicked",
        		    GTK_SIGNAL_FUNC (on_mode_button_clicked),
        		    GINT_TO_POINTER(DETAILS));
        gtk_signal_connect (GTK_OBJECT (indented_view_button), "clicked",
        		    GTK_SIGNAL_FUNC (on_mode_button_clicked),
        		    GINT_TO_POINTER(INDENTED));
        gtk_signal_connect (GTK_OBJECT (modify_the_button), "toggled",
        		    GTK_SIGNAL_FUNC(on_ModifyThe_toggled),
        		    GINT_TO_POINTER(DETAILS));
        gtk_signal_connect (GTK_OBJECT (aboutButton), "clicked",
        		    GTK_SIGNAL_FUNC (on_about_activate),
        		    NULL);
}


void create_filtersTreeContent (void)
{
	GtkCTreeNode *node;
	GtkCTreeNode *node2;
	GtkCTreeNode *current_node_selected;
	gchar *text[1];
	GList *listpointer;
	gint folder_ID_selected;
	static gboolean firstrun = TRUE;
	
	GdkPixmap *pixmap_folder_open;
	GdkBitmap *mask_folder_open;
	GdkPixmap *pixmap_folder_close;
	GdkBitmap *mask_folder_close;
	
	GdkPixbuf *icon_pixbuf;
	GtkStyle *liststyle;
	ZIP *zip;
	gchar *zipfile;
	
	simple_filter *folder_filter;
	
	folder_ID_selected = NUMBER_FOLDER;
			
	/* try to get icons in the zip file */
	zipfile = g_strdup_printf("%sicons.zip",gui_prefs.IconDirectory);
	if(!access(zipfile,R_OK))
	{
		zip = openzip(zipfile);
	}
	else
		zip= NULL;
	
	/* if a font as already be designated, use it */
	if (gui_prefs.ListFontStruct ) {
		liststyle = gtk_style_copy(gtk_widget_get_style(GTK_WIDGET(main_gui.ctree_filters)));
		if (liststyle == NULL) {
			gtk_style_unref(liststyle);
			liststyle = gtk_style_new();
			gxmame_debug("creating new style");
		}
		gxmame_debug("use style.");
		liststyle->font = gui_prefs.ListFontStruct;
		gtk_widget_set_style(GTK_WIDGET(main_gui.ctree_filters), liststyle);
	}
	gtk_clist_set_row_height(GTK_CLIST(main_gui.ctree_filters),gui_prefs.ListFontHeight);
		
	gtk_ctree_set_indent(GTK_CTREE(main_gui.ctree_filters),gui_prefs.ListFontHeight + 5);
	
	gtk_clist_freeze(GTK_CLIST(main_gui.ctree_filters));
	
	/* disconnect signal of previous CTree to prevent a crash when I clean this Ctree
	   should not be done during the first run
	  I'm just adding a static variable, I'm sure I can do by an other way but I''m too tired to search*/
	if (!firstrun)
	{
		gtk_signal_disconnect_by_func(GTK_OBJECT (main_gui.ctree_filters),
					GTK_SIGNAL_FUNC (on_folder_selected),
					NULL);
		gtk_signal_disconnect_by_func(GTK_OBJECT (main_gui.ctree_filters),
					GTK_SIGNAL_FUNC (on_folder_unselected),
					NULL);
	}
	firstrun=FALSE;

	/* clean the CTree*/
	gtk_ctree_post_recursive(main_gui.ctree_filters,
				NULL,
				destroy_simple_filter,
				NULL);

	/* we will need to sort year, category, version and manufacturer*/
	gtk_clist_set_compare_func(GTK_CLIST(main_gui.ctree_filters),NULL);
	
	
	gtk_ctree_set_line_style(main_gui.ctree_filters, GTK_CTREE_LINES_DOTTED);
	gtk_clist_set_selection_mode(GTK_CLIST(main_gui.ctree_filters), GTK_SELECTION_BROWSE);
	
	/* begin to fill */
 	icon_pixbuf = get_icon_for_name("foldopen", gui_prefs.ListFontHeight, zip, foldopen_xpm);
	gdk_pixbuf_render_pixmap_and_mask(icon_pixbuf,
					&pixmap_folder_open,
					&mask_folder_open,
					127);
	g_free(icon_pixbuf);
		
 	icon_pixbuf = get_icon_for_name("folder", gui_prefs.ListFontHeight, zip, folder_xpm);
	gdk_pixbuf_render_pixmap_and_mask(icon_pixbuf,
					&pixmap_folder_close,
					&mask_folder_close,
					127);
	g_free(icon_pixbuf);
	
	/* Filter's Tree*/
	/* ALL GAMES */
 	text[0] = g_strdup_printf(_("All Games"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = DRIVER;
	folder_filter->is = FALSE;
	folder_filter->value = (gpointer *)g_strdup_printf(" ");
	folder_filter->FolderID = ALL;
	/* default folder selected */
	current_filter=	folder_filter;
	current_node_selected = node;
	/* but expand it only if it was the last selected*/
	if (gui_prefs.FolderID==folder_filter->FolderID)
		gtk_ctree_expand(main_gui.ctree_filters,node);
	
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* AVAILABLES */
 	icon_pixbuf = get_icon_for_name("foldavail", gui_prefs.ListFontHeight, zip, foldavail_xpm);
	gdk_pixbuf_render_pixmap_and_mask(icon_pixbuf,
					&pixmap_folder_close,
					&mask_folder_close,
					127);
	g_free(icon_pixbuf);
	
 	text[0] = g_strdup_printf(_("Available"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = HAS_ROMS;
	folder_filter->is = TRUE;
	folder_filter->value = GINT_TO_POINTER(1);
	folder_filter->FolderID = AVAILABLE;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* UNAVAILABLE */
 	icon_pixbuf = get_icon_for_name("foldunav", gui_prefs.ListFontHeight, zip, foldunav_xpm);
	gdk_pixbuf_render_pixmap_and_mask(icon_pixbuf,
					&pixmap_folder_close,
					&mask_folder_close,
					127);
	g_free(icon_pixbuf);
	
 	text[0] = g_strdup_printf(_("Unavailable"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = HAS_ROMS;
	folder_filter->is = FALSE;
	folder_filter->value = GINT_TO_POINTER(1);
	folder_filter->FolderID = UNAVAILABLE;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* NEOGEO */
 	icon_pixbuf = get_icon_for_name("neo-geo", gui_prefs.ListFontHeight, zip, neogeo_xpm);
	gdk_pixbuf_render_pixmap_and_mask(icon_pixbuf,
					&pixmap_folder_close,
					&mask_folder_close,
					127);
	g_free(icon_pixbuf);
	
 	text[0] = g_strdup_printf(_("Neo-Geo"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = DRIVER;
	folder_filter->is = TRUE;
	folder_filter->value = (gpointer *)g_strdup_printf("neogeo");
	folder_filter->FolderID = NEOGEO;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* CPS1 */
 	icon_pixbuf = get_icon_for_name("cps", gui_prefs.ListFontHeight, zip, cps_xpm);
	gdk_pixbuf_render_pixmap_and_mask(icon_pixbuf,
					&pixmap_folder_close,
					&mask_folder_close,
					127);
	g_free(icon_pixbuf);

	text[0] = g_strdup_printf(_("CPS1"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = DRIVER;
	folder_filter->is = TRUE;
	folder_filter->value = (gpointer *)g_strdup_printf("cps1");
	folder_filter->FolderID = CPS1;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* CPS2 */
 	text[0] = g_strdup_printf(_("CPS2"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = DRIVER;
	folder_filter->is = TRUE;
	folder_filter->value = (gpointer *)g_strdup_printf("cps2");
	folder_filter->FolderID = CPS2;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* MANUFACTURERS */
 	icon_pixbuf = get_icon_for_name("foldmanu", gui_prefs.ListFontHeight, zip, foldmanu_xpm);
	gdk_pixbuf_render_pixmap_and_mask(icon_pixbuf,
					&pixmap_folder_close,
					&mask_folder_close,
					127);
	g_free(icon_pixbuf);
 	
	text[0] = g_strdup_printf(_("Manufacturer"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = DRIVER;
	folder_filter->is = FALSE;
	folder_filter->value = (gpointer *)g_strdup_printf(" ");
	folder_filter->FolderID = MANUFACTURERS;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);
	
	/* MANUFACTURERS LIST */
 	icon_pixbuf = get_icon_for_name("manufact", gui_prefs.ListFontHeight, zip, manufact_xpm);
	gdk_pixbuf_render_pixmap_and_mask(icon_pixbuf,
					&pixmap_folder_close,
					&mask_folder_close,
					127);
	g_free(icon_pixbuf);
	
/* insert all manufacturers */
	for(listpointer = g_list_first(manufacturers);
			(listpointer != NULL);
			listpointer = g_list_next(listpointer))
	{
		simple_filter *manufilter;
		text[0] = g_strdup_printf("%s",(gchar *)listpointer->data);
 		node2 = gtk_ctree_insert_node(main_gui.ctree_filters, node, NULL,
 					      text, 5,
 					      pixmap_folder_close,
 					      mask_folder_close,
 					      pixmap_folder_open,
 				     	      mask_folder_open,
 				 	      FALSE, FALSE);
		manufilter = g_new(simple_filter,1);
		manufilter->type = MANU;
		manufilter->is = TRUE;
		manufilter->value = listpointer->data;
		manufilter->FolderID = folder_ID_selected++;
		if (gui_prefs.FolderID==manufilter->FolderID)
		{
			current_filter = manufilter;
			current_node_selected = node2;
			gtk_ctree_expand(main_gui.ctree_filters,node);
			gtk_ctree_expand(main_gui.ctree_filters,node2);
		}
		gtk_ctree_node_set_row_data(main_gui.ctree_filters,node2,manufilter);
	}
	gtk_ctree_sort_node(main_gui.ctree_filters,node);

	/* YEARS */
 	icon_pixbuf = get_icon_for_name("foldyear", gui_prefs.ListFontHeight, zip, foldyear_xpm);
	gdk_pixbuf_render_pixmap_and_mask(icon_pixbuf,
					&pixmap_folder_close,
					&mask_folder_close,
					127);
	g_free(icon_pixbuf);
	
 	text[0] = g_strdup_printf(_("Year"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = DRIVER;
	folder_filter->is = FALSE;
	folder_filter->value = (gpointer *)g_strdup_printf(" ");
	folder_filter->FolderID = YEARS;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);
	
	/* YEARS LIST */
 	icon_pixbuf = get_icon_for_name("year", gui_prefs.ListFontHeight, zip, year_xpm);
	gdk_pixbuf_render_pixmap_and_mask(icon_pixbuf,
					&pixmap_folder_close,
					&mask_folder_close,
					127);
	g_free(icon_pixbuf);
/* insert all years */
	for(listpointer = g_list_last(years);
				(listpointer != NULL);
				listpointer = g_list_previous(listpointer))
	{
		simple_filter *yearfilter;
		if (listpointer->data == 0)
		{
			text[0] = g_strdup_printf(_("<unknown>"));
		}
		else
		{
			text[0] = g_strdup_printf("%d",GPOINTER_TO_INT(listpointer->data));
		}
 		node2 = gtk_ctree_insert_node(main_gui.ctree_filters, node, NULL,
 					      text, 5,
 					      pixmap_folder_close,
 					      mask_folder_close,
 					      pixmap_folder_open,
 					      mask_folder_open,
 					      FALSE, FALSE);
		yearfilter = g_new(simple_filter,1);
		yearfilter->type = YEAR;
		yearfilter->is = TRUE;
		yearfilter->value = listpointer->data;
		yearfilter->FolderID = folder_ID_selected++;
		if (gui_prefs.FolderID==yearfilter->FolderID)
		{
			current_filter = yearfilter;
			current_node_selected = node2;
			gtk_ctree_expand(main_gui.ctree_filters,node);
			gtk_ctree_expand(main_gui.ctree_filters,node2);
		}
		gtk_ctree_node_set_row_data(main_gui.ctree_filters,node2,yearfilter);
	}
	gtk_ctree_sort_node(main_gui.ctree_filters,node);

	/* WORKING */
 	icon_pixbuf = get_icon_for_name("working", gui_prefs.ListFontHeight, zip, working_xpm);
	gdk_pixbuf_render_pixmap_and_mask(icon_pixbuf,
					&pixmap_folder_close,
					&mask_folder_close,
					127);
	g_free(icon_pixbuf);
	
 	text[0] = g_strdup_printf(_("Working"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = STATUS;
	folder_filter->is = TRUE;
	folder_filter->value = CORRECT;
	folder_filter->FolderID = WORKING;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* NONWORKING */
 	icon_pixbuf = get_icon_for_name("nonwork", gui_prefs.ListFontHeight, zip, nonwork_xpm);
	gdk_pixbuf_render_pixmap_and_mask(icon_pixbuf,
					&pixmap_folder_close,
					&mask_folder_close,
					127);
	g_free(icon_pixbuf);
		
 	text[0] = g_strdup_printf(_("Non-Working"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = STATUS;
	folder_filter->is = FALSE;
	folder_filter->value = CORRECT;
	folder_filter->FolderID = NONWORKING;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* CUSTOM */
 	icon_pixbuf = get_icon_for_name("custom", gui_prefs.ListFontHeight, zip, custom_xpm);
	gdk_pixbuf_render_pixmap_and_mask(icon_pixbuf,
					&pixmap_folder_close,
					&mask_folder_close,
					127);
	g_free(icon_pixbuf);
		
 	text[0] = g_strdup_printf(_("Custom"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = DRIVER;
	folder_filter->is = FALSE;
	folder_filter->value = (gpointer *)g_strdup_printf(" ");
	folder_filter->FolderID = CUSTOMS;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);
	
	/* FAVORITES */
 	text[0] = g_strdup_printf(_("Favorites"));
 	node2 = gtk_ctree_insert_node(main_gui.ctree_filters, node, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = FAVORITE;
	folder_filter->is = TRUE;
	folder_filter->value = NULL;
	folder_filter->FolderID = FAVORITES;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node2;
		gtk_ctree_expand(main_gui.ctree_filters,node);
		gtk_ctree_expand(main_gui.ctree_filters,node2);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node2,folder_filter);

	/*PLAYED  */
	text[0] = g_strdup_printf(_("Played"));
	node2 = gtk_ctree_insert_node(main_gui.ctree_filters, node, NULL,
				      text, 5,
				      pixmap_folder_close,
				      mask_folder_close,
				      pixmap_folder_open,
				      mask_folder_open,
				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = TIMESPLAYED;
	folder_filter->is = FALSE;
	folder_filter->value = 0;
	folder_filter->FolderID = PLAYED;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node2;
		gtk_ctree_expand(main_gui.ctree_filters,node);
		gtk_ctree_expand(main_gui.ctree_filters,node2);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node2,folder_filter);

	/* ORIGINALS */
	icon_pixbuf = get_icon_for_name("folder", gui_prefs.ListFontHeight, zip,folder_xpm);
	gdk_pixbuf_render_pixmap_and_mask(icon_pixbuf,
					&pixmap_folder_close,
					&mask_folder_close,
					127);
	g_free(icon_pixbuf);

	text[0] = g_strdup_printf(_("Originals"));
	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = CLONE;
	folder_filter->is = TRUE;
	folder_filter->value = (gpointer *)g_strdup_printf("-");
	folder_filter->FolderID = ORIGINALS;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* CLONES */
 	text[0] = g_strdup_printf(_("Clones"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = CLONE;
	folder_filter->is = FALSE;
	folder_filter->value = (gpointer *)g_strdup_printf("-");
	folder_filter->FolderID = CLONES;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* RASTERS */
 	text[0] = g_strdup_printf(_("Raster"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = VECTOR;
	folder_filter->is = FALSE;
	folder_filter->value = NULL;
	folder_filter->FolderID = RASTERS;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	text[0] = g_strdup_printf(_("Vector"));
	/* VECTORS */
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = VECTOR;
	folder_filter->is = TRUE;
	folder_filter->value = NULL;
	folder_filter->FolderID = VECTORS;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* TRACKBALL */
 	text[0] = g_strdup_printf(_("Trackball"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = CONTROL;
	folder_filter->is = TRUE;
	folder_filter->value = (gpointer *)g_strdup_printf("trackball");
	folder_filter->FolderID = TRACKBALL;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* STEREO */
 	icon_pixbuf = get_icon_for_name("sound", gui_prefs.ListFontHeight, zip,sound_xpm);
	gdk_pixbuf_render_pixmap_and_mask(icon_pixbuf,
					&pixmap_folder_close,
					&mask_folder_close,
					127);
	g_free(icon_pixbuf);

	text[0] = g_strdup_printf(_("Stereo"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = CHANNELS;
	folder_filter->is = TRUE;
	folder_filter->value = GINT_TO_POINTER(2);
	folder_filter->FolderID = STEREO;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* CATEGORY */
 	icon_pixbuf = get_icon_for_name("foldcat", gui_prefs.ListFontHeight, zip,foldcat_xpm);
	gdk_pixbuf_render_pixmap_and_mask(icon_pixbuf,
					&pixmap_folder_close,
					&mask_folder_close,
					127);
	g_free(icon_pixbuf);

	text[0] = g_strdup_printf(_("Category"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = DRIVER;
	folder_filter->is = FALSE;
	folder_filter->value = (gpointer *)g_strdup_printf(" ");
	folder_filter->FolderID = CATEGORIES;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);
	/* insert all categories */
	for(listpointer = g_list_first(categories);
				(listpointer != NULL);
				listpointer = g_list_next(listpointer))
	{
		simple_filter *catfilter;
		text[0] = g_strdup(listpointer->data);
 		node2 = gtk_ctree_insert_node(main_gui.ctree_filters, node, NULL,
 					      text, 5,
 					      pixmap_folder_close,
 					      mask_folder_close,
 					      pixmap_folder_open,
 					      mask_folder_open,
 					      FALSE, FALSE);
		catfilter = g_new(simple_filter,1);
		catfilter->type = CATEGORY;
		catfilter->is = TRUE;
		catfilter->value = listpointer->data;
		catfilter->FolderID = folder_ID_selected++;
		if (gui_prefs.FolderID==catfilter->FolderID)
		{
			current_filter = catfilter;
			current_node_selected = node2;
			gtk_ctree_expand(main_gui.ctree_filters,node);
			gtk_ctree_expand(main_gui.ctree_filters,node2);
		}
		gtk_ctree_node_set_row_data(main_gui.ctree_filters,node2,catfilter);
	}
	gtk_ctree_sort_node(main_gui.ctree_filters,node);

	/* VERSION */
 	icon_pixbuf = get_icon_for_name("foldvers", gui_prefs.ListFontHeight, zip, foldvers_xpm);
	gdk_pixbuf_render_pixmap_and_mask(icon_pixbuf,
					&pixmap_folder_close,
					&mask_folder_close,
					127);
	g_free(icon_pixbuf);

	text[0] = g_strdup_printf(_("Version"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = DRIVER;
	folder_filter->is = FALSE;
	folder_filter->value = (gpointer *)g_strdup_printf(" ");
	folder_filter->FolderID = VERSIONS;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);
	/* insert all version */
	for(listpointer = g_list_first(versions);
				(listpointer != NULL);
				listpointer = g_list_next(listpointer))
	{
		simple_filter *verfilter;
		text[0] = g_strdup(listpointer->data);
 		node2 = gtk_ctree_insert_node(main_gui.ctree_filters, node, NULL,
 					      text, 5,
 					      pixmap_folder_close,
 					      mask_folder_close,
 					      pixmap_folder_open,
 					      mask_folder_open,
 					      FALSE, FALSE);
		verfilter = g_new(simple_filter,1);
		verfilter->type = MAMEVER;
		verfilter->is = TRUE;
		verfilter->value = listpointer->data;
		verfilter->FolderID = folder_ID_selected++;
		if (gui_prefs.FolderID==verfilter->FolderID)
		{
			current_filter = verfilter;
			current_node_selected = node2;
			gtk_ctree_expand(main_gui.ctree_filters,node);
			gtk_ctree_expand(main_gui.ctree_filters,node2);
		}
		gtk_ctree_node_set_row_data(main_gui.ctree_filters,node2,verfilter);
	}
	gtk_ctree_sort_node(main_gui.ctree_filters,node);

	
	
 	g_free(text[0]); 
	gdk_pixmap_unref(pixmap_folder_close);
	gdk_pixmap_unref(mask_folder_close);
	gdk_pixmap_unref(pixmap_folder_open);
	gdk_pixmap_unref(mask_folder_open);
	
	/* needed here or the size is not completly right */
	gtk_clist_thaw(GTK_CLIST(main_gui.ctree_filters));
	gtk_clist_columns_autosize(GTK_CLIST(main_gui.ctree_filters));
/*	while (gtk_events_pending()) gtk_main_iteration();*/

	/* need to connect the CTree only after because the gamelist is not yet created at this point */
	gtk_ctree_select (main_gui.ctree_filters,current_node_selected);
	/*FIXME where are you going ??
	   maybe need to expend tree before ?*/
	gtk_ctree_node_moveto (main_gui.ctree_filters,current_node_selected,0,0,0);
	
/* Signals Connection*/
	gtk_signal_connect (GTK_OBJECT (main_gui.ctree_filters), "tree_select_row",
        		    GTK_SIGNAL_FUNC (on_folder_selected),
        		    NULL);
        gtk_signal_connect (GTK_OBJECT (main_gui.ctree_filters), "tree_unselect_row",
        		    GTK_SIGNAL_FUNC (on_folder_unselected),
        		    NULL);

	if (zip)
		closezip(zip);
	g_free(zipfile);
}

void hide_filters (void)
{
	gui_prefs.ShowFolderList=0;
	gui_prefs.Splitters[0] = main_gui.scrolled_window_filters->allocation.width;
	gtk_paned_set_position (main_gui.hpanedLeft, 0);
	gtk_paned_set_gutter_size (main_gui.hpanedLeft, 0);
	gtk_container_remove(GTK_CONTAINER(main_gui.hpanedLeft),GTK_WIDGET(main_gui.scrolled_window_filters));
}

void show_filters (void)
{
	gui_prefs.ShowFolderList=1;
	gtk_paned_set_position (main_gui.hpanedLeft, gui_prefs.Splitters[0]);
	gtk_paned_set_gutter_size (main_gui.hpanedLeft, 6);
	gtk_paned_pack1 (main_gui.hpanedLeft, GTK_WIDGET(main_gui.scrolled_window_filters), FALSE, FALSE);
}

void hide_snaps (void)
{	
	gui_prefs.ShowScreenShot=0;
	gui_prefs.Splitters[1] = main_gui.scrolled_window_games->allocation.width;
	gtk_paned_set_position (main_gui.hpanedRight, -1);
	gtk_paned_set_gutter_size (main_gui.hpanedRight, 0);
	gtk_paned_set_handle_size(main_gui.hpanedRight, 0);
	gtk_container_remove(GTK_CONTAINER(main_gui.hpanedRight),GTK_WIDGET(main_gui.screenshot_hist_frame));
}

void show_snaps (void)
{
	gui_prefs.ShowScreenShot=1;
	gtk_paned_set_position (main_gui.hpanedRight, gui_prefs.Splitters[1]);
	gtk_paned_set_gutter_size (main_gui.hpanedRight, 6);
	gtk_paned_set_handle_size(main_gui.hpanedRight, 10);
	gtk_paned_pack2 (main_gui.hpanedRight, GTK_WIDGET(main_gui.screenshot_hist_frame), FALSE, FALSE);
}


void hide_toolbar (void)
{
	gui_prefs.ShowToolBar=0;
	gtk_widget_hide (GTK_WIDGET(main_gui.hseparator2));
	gtk_widget_hide (GTK_WIDGET(main_gui.toolbar));
	gtk_widget_hide (GTK_WIDGET(main_gui.hseparator1));
}


void show_toolbar (void)
{
	gui_prefs.ShowToolBar=1;
	gtk_widget_show (GTK_WIDGET(main_gui.hseparator2));
	gtk_widget_show (GTK_WIDGET(main_gui.toolbar));
	gtk_widget_show (GTK_WIDGET(main_gui.hseparator1));
}


void hide_status_bar (void)
{
	gui_prefs.ShowStatusBar=0;
	gtk_widget_hide (GTK_WIDGET(main_gui.tri_status_bar));
}


void show_status_bar (void)
{
	gui_prefs.ShowStatusBar=1;
	gtk_widget_show (GTK_WIDGET(main_gui.tri_status_bar));
}


/* get an icon for a rom, if not found, try the original game if the game is a clone */
GdkPixbuf *get_icon_for_rom(RomEntry *rom, guint size, ZIP *zip)
{
	GdkPixbuf *pixbuf = NULL, *scaled_pixbuf = NULL;
	gchar filename[MAX_ROMNAME +1], *filename2;
	
	if (rom!=NULL)
	{
		filename2 = g_strdup_printf("%s%s.ico",gui_prefs.IconDirectory,rom->romname);
		pixbuf = gdk_pixbuf_new_from_file (filename2);
		g_free(filename2);
		/* no picture found try parent game if any*/
		if ((pixbuf == NULL) && strcmp(rom->cloneof,"-"))
		{
			filename2 = g_strdup_printf("%s%s.ico",gui_prefs.IconDirectory,rom->cloneof);
			pixbuf = gdk_pixbuf_new_from_file (filename);
			g_free(filename2);
		}
		/* we havent found the picture in the directory, maybe we could try in a zipfile */
		if (pixbuf == NULL)
		{
			struct zipent* zipent;
			char *tmp_buffer;
			if (zip != 0)
			{
				GdkPixbufLoader *loader;
				rewindzip(zip);
				g_snprintf(filename, MAX_ROMNAME +1 , "%s.",rom->romname);
				while ((zipent = readzip(zip)) != 0)
				{
					/* this should allows to find any format of picture in the zip, not only bmp */
					if (!strncmp(filename,zipent->name,strlen(rom->romname)+1))
					{
						if ((tmp_buffer = g_malloc(zipent->uncompressed_size)) == 0)
							gxmame_debug("%s: out of memory!",zipent->name);
						else
						{	/* if the file successfully uncompress, try to load it in a pixbuf loader */
							if (readuncompresszip(zip, zipent, (char *)tmp_buffer) == 0)
							{
								loader = gdk_pixbuf_loader_new ();
								if (!gdk_pixbuf_loader_write (loader, tmp_buffer, zipent->uncompressed_size)) {
									gxmame_debug("Error while uncompressing %s ",zipent->name);
								}
								else
								{
									gdk_pixbuf_loader_close (loader);
									pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
								}
							}
							g_free(tmp_buffer);
							tmp_buffer = 0;
						}
					/* prevent to read all zip file if we have found the picture's game (uncompressed successfuly or not) */
					break;
					}
				}
				/* no picture found try parent game if any*/
				if ((pixbuf == NULL) && strcmp(rom->cloneof,"-"))
				{	
					rewindzip(zip);
					gxmame_debug("picture not found, trying clone: %s",rom->cloneof);
					g_snprintf(filename, MAX_ROMNAME +1 , "%s.",rom->cloneof);
					while ((zipent = readzip(zip)) != 0)
					{
						/* this should allows to find any format of picture in the zip, not only bmp */
						if (!strncmp(filename,zipent->name,strlen(rom->cloneof)+1))
						{
							if ((tmp_buffer = g_malloc(zipent->uncompressed_size)) == 0)
								gxmame_debug("%s: out of memory!",zipent->name);
							else
							{	/* if the file successfully uncompress, try to load it in a pixbuf loader */
								if (readuncompresszip(zip, zipent, (char *)tmp_buffer) == 0)
								{
									loader = gdk_pixbuf_loader_new ();
									if (!gdk_pixbuf_loader_write (loader, tmp_buffer, zipent->uncompressed_size)) {
										gxmame_debug("Error while uncompressing %s ",zipent->name);
									}
									else
									{
										gdk_pixbuf_loader_close (loader);
										pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
									}
								}
								g_free(tmp_buffer);
								tmp_buffer = 0;
							}
						/* prevent to read all zip file if we have found the picture's game (uncompressed successfuly or not) */
						break;
						}
					}
				}
			}
			
		}
		if (pixbuf != NULL)
		{
			scaled_pixbuf = gdk_pixbuf_scale_simple(pixbuf,
								size,
 			             				size,
 			             				GDK_INTERP_BILINEAR);
			gdk_pixbuf_unref (pixbuf);
		}
	}
	return scaled_pixbuf;
}


/* get an icon with a specific name */
GdkPixbuf *get_icon_for_name(gchar *name, guint size, ZIP *zip,  char **alternative)
{
	GdkPixbuf *pixbuf = NULL, *scaled_pixbuf = NULL;
	gchar filename[MAX_ROMNAME+1], *filename2;
	
	if (name!=NULL)
	{
		filename2 = g_strdup_printf("%s%s.ico",gui_prefs.IconDirectory,name);
		pixbuf = gdk_pixbuf_new_from_file (filename2);
		g_free(filename2);
		/* we havent found the picture in the directory, maybe we could try in a zipfile */
		if (pixbuf == NULL)
		{
			struct zipent* zipent;
			char *tmp_buffer;
			if (zip != 0)
			{
				GdkPixbufLoader *loader;
				rewindzip(zip);
				g_snprintf(filename, MAX_ROMNAME +1 , "%s.",name);
				while ((zipent = readzip(zip)) != 0)
				{
					/* this should allows to find any format of picture in the zip, not only bmp */
					if (!strncmp(filename,zipent->name,strlen(name)+1))
					{
						if ((tmp_buffer = g_malloc(zipent->uncompressed_size)) == 0)
							gxmame_debug("%s: out of memory!",zipent->name);
						else
						{	/* if the file successfully uncompress, try to load it in a pixbuf loader */
							if (readuncompresszip(zip, zipent, (char *)tmp_buffer) == 0)
							{
								loader = gdk_pixbuf_loader_new ();
								if (!gdk_pixbuf_loader_write (loader, tmp_buffer, zipent->uncompressed_size)) {
									gxmame_debug("Error while uncompressing %s ",zipent->name);
								}
								else
								{
									gdk_pixbuf_loader_close (loader);
									pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
								}
							}
							g_free(tmp_buffer);
							tmp_buffer = 0;
						}
					/* prevent to read all zip file if we have found the picture's game (uncompressed successfuly or not) */
					break;
					}
				}
			}
		}
		/* haven't found anything, draw back to the alternative pixmap */
		if (pixbuf == NULL)
		{
			pixbuf = gdk_pixbuf_new_from_xpm_data((const char **)alternative);
		}
		/*dont need to test since in any case we got a picture (from file or internal pixmap)*/
		scaled_pixbuf = gdk_pixbuf_scale_simple(pixbuf,
							size,
 		             				size,
 		             				GDK_INTERP_BILINEAR);
		gdk_pixbuf_unref (pixbuf);
	}
	return scaled_pixbuf;
}

/*this function check if an icon update is needed and get the icon from file or inlined xpm*/
void get_status_icons()
{
	ZIP *zip=NULL;
	gchar *zipfile=NULL;
	
	/* allocate and open zip fille if needed, ie dirty cache or null icon */
	if(dirty_icon_cache ||
		!Status_Icons[INCORRECT] ||
		!Status_Icons[CORRECT] ||
		!Status_Icons[UNKNOWN] ||
		!Status_Icons[PROBLEMS])
	{
		zipfile = g_strdup_printf("%sicons.zip",gui_prefs.IconDirectory);
		if(!access(zipfile,R_OK))
		{
			zip = openzip(zipfile);
		}
		else
			zip= NULL;
		
		if(dirty_icon_cache)
		{
			if(Status_Icons[INCORRECT]!=NULL)
			{
					gdk_pixbuf_unref(Status_Icons[INCORRECT]);
					Status_Icons[INCORRECT] = NULL;
			}
			if(Status_Icons[CORRECT]!=NULL)
			{
					gdk_pixbuf_unref(Status_Icons[CORRECT]);
					Status_Icons[CORRECT] = NULL;
			}
			if(Status_Icons[UNKNOWN]!=NULL)
			{
					gdk_pixbuf_unref(Status_Icons[UNKNOWN]);
					Status_Icons[UNKNOWN] = NULL;
			}
			if(Status_Icons[PROBLEMS]!=NULL)
			{
					gdk_pixbuf_unref(Status_Icons[PROBLEMS]);
					Status_Icons[PROBLEMS] = NULL;
			}
		}
		
	 	if(Status_Icons[CORRECT]==NULL)
			Status_Icons[CORRECT] = get_icon_for_name("roms", gui_prefs.ListFontHeight, zip, romL_xpm);
	 	if(Status_Icons[INCORRECT]==NULL)
			Status_Icons[INCORRECT] = get_icon_for_name("noroms", gui_prefs.ListFontHeight, zip, noromsL_xpm);
	 	if(Status_Icons[UNKNOWN]==NULL)
			Status_Icons[UNKNOWN] = get_icon_for_name("unknown", gui_prefs.ListFontHeight, zip, unknownL_xpm);
	 	if(Status_Icons[PROBLEMS]==NULL)
			Status_Icons[PROBLEMS] = get_icon_for_name("warning", gui_prefs.ListFontHeight, zip, redxL_xpm);

		if (zip)
			closezip(zip);
		g_free(zipfile);
	}

}

void display_small_icon_list (void)
{
/* Unable to use a gtkList here cause of the 32k pixels limitation
   forced to use a gtkCList but the gtkList should work with GTK 2
   perhaps could remplace it just by hidding columns other than game name
   in the details view (if this one was previously selected)
*/
	RomEntry *tmprom;
	GList *listpointer;
	GtkWidget *displayed_list;
	gchar **Texte;
	gchar *message;
	int i=0,j=0;
	int currentgame_row=-1;
	GdkPixmap *pixmap_unknown;
	GdkBitmap *mask_unknown;
	GdkPixmap *pixmap_rom;
	GdkBitmap *mask_rom;
	GdkPixmap *pixmap_norom;
	GdkBitmap *mask_norom;
	GdkPixmap *pixmap_redx;
	GdkBitmap *mask_redx;
	GdkPixmap *testpix = NULL;
	GdkPixmap *testmask = NULL;
	GtkStyle *liststyle;
	
	Texte = g_malloc(sizeof(gchar *));

	get_status_icons();
	gdk_pixbuf_render_pixmap_and_mask(Status_Icons[INCORRECT],
					&pixmap_norom,
					&mask_norom,
					127);
	gdk_pixbuf_render_pixmap_and_mask(Status_Icons[CORRECT],
					&pixmap_rom,
					&mask_rom,
					127);
	gdk_pixbuf_render_pixmap_and_mask(Status_Icons[UNKNOWN],
					&pixmap_unknown,
					&mask_unknown,
					127);
	gdk_pixbuf_render_pixmap_and_mask(Status_Icons[PROBLEMS],
					&pixmap_redx,
					&mask_redx,
					127);
	
	displayed_list = gtk_clist_new(1);
	gtk_clist_set_selection_mode(GTK_CLIST(displayed_list), GTK_SELECTION_BROWSE);
	gtk_container_add(GTK_CONTAINER(main_gui.scrolled_window_games), displayed_list);

	/* if a font as already be designated, use it */
	if (gui_prefs.ListFontStruct ) {
		liststyle = gtk_style_copy(gtk_widget_get_style(GTK_WIDGET(displayed_list)));
		if (liststyle == NULL) {
			gtk_style_unref(liststyle);
			liststyle = gtk_style_new();
			gxmame_debug("creating new style");
		}
		gxmame_debug("use style.");
		liststyle->font = gui_prefs.ListFontStruct;
		gtk_widget_set_style(GTK_WIDGET(displayed_list), liststyle);
	}
	gtk_clist_set_row_height(GTK_CLIST(displayed_list),gui_prefs.ListFontHeight);
	gtk_clist_freeze(GTK_CLIST(displayed_list));
		
	for (listpointer = g_list_first(roms);(listpointer != NULL);listpointer = g_list_next(listpointer))
	{
		tmprom = (RomEntry *) listpointer->data;
		/* the icon cache is dirty (change fontsize or display) */
		if (dirty_icon_cache && (tmprom->icon_pixbuf != NULL))
		{
			gdk_pixbuf_unref(tmprom->icon_pixbuf);
			tmprom->icon_pixbuf = NULL;
		}
			
		if (game_filtered(tmprom)){
			/* dont really need this string , an empty one could be enough
			   in that case mallog ans snprintf*/
			if (tmprom->the_trailer && gui_prefs.ModifyThe)
				Texte[0] = g_strstrip(g_strdup_printf("%s, The %s",tmprom->gamename,tmprom->gamenameext));
			else if (tmprom->the_trailer && !gui_prefs.ModifyThe)
				Texte[0] = g_strstrip(g_strdup_printf("The %s %s",tmprom->gamename,tmprom->gamenameext));
			else
				Texte[0] = g_strstrip(g_strdup_printf("%s %s",tmprom->gamename,tmprom->gamenameext));
			tmprom->position = i = gtk_clist_append(GTK_CLIST(displayed_list), Texte);
			if (tmprom->has_roms == 2)
				gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
	                                      i,
	                                      0,
	                                      Texte[0],
	                                      5,
	                                      pixmap_unknown,
	                                      mask_unknown);
			else if (tmprom->has_roms == 0)
				gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
	                                      i,
	                                      0,
	                                      Texte[0],
	                                      5,
	                                      pixmap_norom,
	                                      mask_norom);
			else if (!tmprom->status)
				gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
	                                      i,
	                                      0,
	                                      Texte[0],
	                                      5,
	                                      pixmap_redx,
	                                      mask_redx);
			else
			{
				if (tmprom->icon_pixbuf != NULL)
				{
					gdk_pixbuf_render_pixmap_and_mask(tmprom->icon_pixbuf,
								&testpix,
								&testmask,
								127);
					gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
					              i,
					              0,
					              Texte[0],
					              5,
					              testpix,
					              testmask);
				}else
				{
					gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
					              i,
					              0,
					              Texte[0],
					              5,
					              pixmap_rom,
					              mask_rom);
				}
			}
			gtk_clist_set_row_data(GTK_CLIST(displayed_list),i,tmprom);
	/* color for clones */
			if (strcmp(tmprom->cloneof,"-"))
			{
				gtk_clist_set_foreground (GTK_CLIST(displayed_list),
                                             		i,
							&(gui_prefs.clone_color));
			}
			if (!strncmp(Texte[0],gui_prefs.DefaultGame,MAX_GAMENAME))
				currentgame_row=i; 
			g_free(Texte[0]);
			/*allows to display the correct number of games*/
			i++;
		}
		else
			tmprom->position = -1;
	}
	gdk_pixmap_unref(pixmap_unknown);
	gdk_pixmap_unref(mask_unknown);
	gdk_pixmap_unref(pixmap_rom);
	gdk_pixmap_unref(mask_rom);
	gdk_pixmap_unref(pixmap_norom);
	gdk_pixmap_unref(mask_norom);
	gdk_pixmap_unref(pixmap_redx);
	gdk_pixmap_unref(mask_redx);

	visible_games = i;
	
	main_gui.displayed_list = displayed_list;
	/* 2 actif buttons */
	for (j = 0; j < 2; j++)
		gtk_clist_set_button_actions(GTK_CLIST(displayed_list), j, GTK_BUTTON_SELECTS);
	
	gtk_signal_connect (GTK_OBJECT (displayed_list), "select_row",
        		    GTK_SIGNAL_FUNC (on_row_selected),
        		    NULL);
	gtk_signal_connect (GTK_OBJECT(displayed_list), "key_press_event",
			    GTK_SIGNAL_FUNC (on_displayed_list_key_press_event),
			    NULL);
	gtk_signal_connect (GTK_OBJECT(displayed_list), "key_release_event",
			    GTK_SIGNAL_FUNC (on_displayed_list_key_release_event),
			    NULL);
	
	if (visible_games != 0)
		gtk_signal_connect (GTK_OBJECT (displayed_list), "button_press_event",
        		    	    GTK_SIGNAL_FUNC (on_row_right_clic),
        		   	    NULL);
	
	gtk_clist_columns_autosize(GTK_CLIST(displayed_list));
	gtk_clist_thaw(GTK_CLIST(displayed_list));

	gtk_widget_show_all(main_gui.scrolled_window_games);
	
	message = g_strdup_printf("%d %s",visible_games,visible_games>1?_("games"):_("game"));
	gtk_statusbar_pop (main_gui.statusbar3, 1);
	gtk_statusbar_push (main_gui.statusbar3, 1, message);
	g_free(message);
	
	/* allow to correctly update the page_size and page_increment of the GtkAdjustment of the GtkClist*/
	while (gtk_events_pending()) gtk_main_iteration();
	if (currentgame_row>0)
	{
	        gtk_clist_select_row(GTK_CLIST(displayed_list),currentgame_row,0);
		gtk_clist_moveto(GTK_CLIST(displayed_list),currentgame_row,0,0.5,0);
	}
	else
	{
	        gtk_clist_select_row(GTK_CLIST(displayed_list),0,0);
		gtk_clist_moveto(GTK_CLIST(displayed_list),0,0,0,0);
	}
	if (visible_games==0)
		select_game(NULL);
	dirty_icon_cache = FALSE;
}


void free_only_key (gpointer key, gpointer value, gpointer user_data)
{
	g_free(key);
}



void display_indented_list (void)
{
	RomEntry *tmprom;
	GList *listpointer;
	GtkWidget *displayed_list;
	gchar **Texte;
	gchar *message;
	int i=0,j=0;
	int currentgame_row=-1;
	RomEntry *currentgame_ptr=NULL;
	GdkPixmap *pixmap_unknown;
	GdkBitmap *mask_unknown;
	GdkPixmap *pixmap_rom;
	GdkBitmap *mask_rom;
	GdkPixmap *pixmap_norom;
	GdkBitmap *mask_norom;
	GdkPixmap *pixmap_redx;
	GdkBitmap *mask_redx;
	GtkCTreeNode *node, *node2;
	GHashTable *original_htable=NULL;
	gchar *originalgamekey;
	
	GdkPixmap *testpix = NULL;
	GdkPixmap *testmask = NULL;
	GtkStyle *liststyle;
	
	
	original_htable = g_hash_table_new (g_str_hash, g_str_equal);
	Texte = g_malloc(sizeof(gchar *));

	get_status_icons();
	gdk_pixbuf_render_pixmap_and_mask(Status_Icons[INCORRECT],
					&pixmap_norom,
					&mask_norom,
					127);
	gdk_pixbuf_render_pixmap_and_mask(Status_Icons[CORRECT],
					&pixmap_rom,
					&mask_rom,
					127);
	gdk_pixbuf_render_pixmap_and_mask(Status_Icons[UNKNOWN],
					&pixmap_unknown,
					&mask_unknown,
					127);
	gdk_pixbuf_render_pixmap_and_mask(Status_Icons[PROBLEMS],
					&pixmap_redx,
					&mask_redx,
					127);
	
	displayed_list = gtk_ctree_new(1,0);
	gtk_ctree_set_line_style (GTK_CTREE(displayed_list),GTK_CTREE_LINES_NONE);
	gtk_ctree_set_expander_style(GTK_CTREE(displayed_list),GTK_CTREE_EXPANDER_NONE);
	gtk_clist_set_selection_mode(GTK_CLIST(displayed_list), GTK_SELECTION_BROWSE);
	gtk_container_add(GTK_CONTAINER(main_gui.scrolled_window_games), displayed_list);

	/* if a font as already be designated, use it */
	if (gui_prefs.ListFontStruct ) {
		liststyle = gtk_style_copy(gtk_widget_get_style(GTK_WIDGET(displayed_list)));
		if (liststyle == NULL) {
			gtk_style_unref(liststyle);
			liststyle = gtk_style_new();
			gxmame_debug("creating new style");
		}
		gxmame_debug("use style.");
		liststyle->font = gui_prefs.ListFontStruct;
		gtk_widget_set_style(GTK_WIDGET(displayed_list), liststyle);
	}
	gtk_clist_set_row_height(GTK_CLIST(displayed_list),gui_prefs.ListFontHeight);
	gtk_ctree_set_indent(GTK_CTREE(displayed_list),gui_prefs.ListFontHeight + 5);
	
	gtk_clist_freeze(GTK_CLIST(displayed_list));
	
			
	/* first run for the original games */	
	for (listpointer = g_list_first(roms);(listpointer != NULL);listpointer = g_list_next(listpointer))
	{
		tmprom = (RomEntry *) listpointer->data;
		/* the icon cache is dirty (change fontsize or display) */
		if (dirty_icon_cache && (tmprom->icon_pixbuf != NULL))
		{
			gdk_pixbuf_unref(tmprom->icon_pixbuf);
			tmprom->icon_pixbuf = NULL;
		}
	    if (game_filtered(tmprom)){
		/* put the original games first */
		if (!strcmp(tmprom->cloneof,"-")){
			i++;
			/* dont really need this string , an empty one could be enough
			   in that case malloc and snprintf*/
			if (tmprom->the_trailer && gui_prefs.ModifyThe)
				Texte[0] = g_strstrip(g_strdup_printf("%s, The %s",tmprom->gamename,tmprom->gamenameext));
			else if (tmprom->the_trailer && !gui_prefs.ModifyThe)
				Texte[0] = g_strstrip(g_strdup_printf("The %s %s",tmprom->gamename,tmprom->gamenameext));
			else
				Texte[0] = g_strstrip(g_strdup_printf("%s %s",tmprom->gamename,tmprom->gamenameext));
			if (tmprom->has_roms == 2)
 				node = gtk_ctree_insert_node(GTK_CTREE(displayed_list), NULL, NULL,
 				      Texte, 5,
 				      NULL,
 				      NULL,
 				      pixmap_unknown,
 				      mask_unknown,
 				      FALSE, TRUE);
			else if (tmprom->has_roms == 0)
 				node = gtk_ctree_insert_node(GTK_CTREE(displayed_list), NULL, NULL,
 				      Texte, 5,
 				      NULL,
 				      NULL,
 				      pixmap_norom,
 				      mask_norom,
 				      FALSE, TRUE);
			else if (!tmprom->status)
			{
 				node = gtk_ctree_insert_node(GTK_CTREE(displayed_list), NULL, NULL,
 				      Texte, 5,
 				      NULL,
 				      NULL,
 				      pixmap_redx,
 				      mask_redx,
 				      FALSE, TRUE);
			}
			else
			{
				if (tmprom->icon_pixbuf != NULL)
				{
					gdk_pixbuf_render_pixmap_and_mask(tmprom->icon_pixbuf,
								&testpix,
								&testmask,
								10);
	
 					node = gtk_ctree_insert_node(GTK_CTREE(displayed_list), NULL, NULL,
 					      Texte, 5,
 					      NULL,
 					      NULL,
 					      testpix,
 					      testmask,
 					      FALSE, TRUE);
	
				}else
				{
 					node = gtk_ctree_insert_node(GTK_CTREE(displayed_list), NULL, NULL,
 					      Texte, 5,
 					      NULL,
 					      NULL,
 					      pixmap_rom,
 					      mask_rom,
 					      FALSE, TRUE);
				}
			}
			gtk_ctree_node_set_row_data(GTK_CTREE(displayed_list),node,tmprom);
			if (!strncmp(Texte[0],gui_prefs.DefaultGame,MAX_GAMENAME))
				currentgame_ptr = tmprom;
			g_free(Texte[0]);
			
			originalgamekey=g_strndup(tmprom->romname,strlen(tmprom->romname));
			g_hash_table_insert(original_htable,originalgamekey,node);
		}
	   }
	}
	/*second run for clone games now */
	for (listpointer = g_list_first(roms);(listpointer != NULL);listpointer = g_list_next(listpointer))
	{
		tmprom = (RomEntry *) listpointer->data;
	    if (game_filtered(tmprom)){
		if (strcmp(tmprom->cloneof,"-")){
			i++;
			node2 = g_hash_table_lookup(original_htable,tmprom->cloneof);
			/* dont really need this string , an empty one could be enough
			   in that case mallog ans snprintf*/
			if (tmprom->the_trailer && gui_prefs.ModifyThe)
				Texte[0] = g_strstrip(g_strdup_printf("%s, The %s",tmprom->gamename,tmprom->gamenameext));
			else if (tmprom->the_trailer && !gui_prefs.ModifyThe)
				Texte[0] = g_strstrip(g_strdup_printf("The %s %s",tmprom->gamename,tmprom->gamenameext));
			else
				Texte[0] = g_strstrip(g_strdup_printf("%s %s",tmprom->gamename,tmprom->gamenameext));
			if (tmprom->has_roms == 2)
 				node = gtk_ctree_insert_node(GTK_CTREE(displayed_list), node2, NULL,
 				      Texte, 5,
 				      pixmap_unknown,
 				      mask_unknown,
 				      NULL,
 				      NULL,
 				      TRUE, FALSE);
			else if (tmprom->has_roms == 0)
 				node = gtk_ctree_insert_node(GTK_CTREE(displayed_list), node2, NULL,
 				      Texte, 5,
 				      pixmap_norom,
 				      mask_norom,
 				      NULL,
 				      NULL,
 				      TRUE, FALSE);
			else if (!tmprom->status)
 				node = gtk_ctree_insert_node(GTK_CTREE(displayed_list), node2, NULL,
 				      Texte, 5,
 				      pixmap_redx,
 				      mask_redx,
 				      NULL,
 				      NULL,
 				      TRUE, FALSE);
			else
			{
				if (tmprom->icon_pixbuf != NULL)
				{
					gdk_pixbuf_render_pixmap_and_mask(tmprom->icon_pixbuf,
								&testpix,
								&testmask,
								127);
	
 					node = gtk_ctree_insert_node(GTK_CTREE(displayed_list), node2, NULL,
 					      Texte, 5,
 					      NULL,
 					      NULL,
 					      testpix,
 					      testmask,
 					      FALSE, TRUE);
	
				}else
				{
 					node = gtk_ctree_insert_node(GTK_CTREE(displayed_list), node2, NULL,
 					      Texte, 5,
 					      NULL,
 					      NULL,
 					      pixmap_rom,
 					      mask_rom,
 					      FALSE, TRUE);
				}
			}
			gtk_ctree_node_set_row_data(GTK_CTREE(displayed_list),node,tmprom);
			gtk_ctree_node_set_foreground (GTK_CTREE(displayed_list),
				node,
				&(gui_prefs.clone_color));
			if (!strncmp(Texte[0],gui_prefs.DefaultGame,MAX_GAMENAME))
				currentgame_ptr = tmprom;
			g_free(Texte[0]);
		}
	    }
	}
	/* third run to get position now and display icons if row is visible */
	visible_games = i;
	i=0;
	for (listpointer = g_list_first(roms);(listpointer != NULL);listpointer = g_list_next(listpointer))
	{
		tmprom = (RomEntry *) listpointer->data;
		tmprom->position = gtk_clist_find_row_from_data(GTK_CLIST(displayed_list),tmprom);
	}
	
	if (currentgame_ptr)
		currentgame_row = currentgame_ptr->position;
	else
		currentgame_row = 0;
	gdk_pixmap_unref(pixmap_unknown);
	gdk_pixmap_unref(mask_unknown);
	gdk_pixmap_unref(pixmap_rom);
	gdk_pixmap_unref(mask_rom);
	gdk_pixmap_unref(pixmap_norom);
	gdk_pixmap_unref(mask_norom);
	gdk_pixmap_unref(pixmap_redx);
	gdk_pixmap_unref(mask_redx);

	
	main_gui.displayed_list = displayed_list;
	/* 2 actif buttons */
	for (j = 0; j < 2; j++)
		gtk_clist_set_button_actions(GTK_CLIST(displayed_list), j, GTK_BUTTON_SELECTS);
	
	gtk_signal_connect (GTK_OBJECT (displayed_list), "select_row",
        		    GTK_SIGNAL_FUNC (on_row_selected),
        		    NULL);
	gtk_signal_connect (GTK_OBJECT(displayed_list), "key_press_event",
			    GTK_SIGNAL_FUNC (on_displayed_list_key_press_event),
			    NULL);
	gtk_signal_connect (GTK_OBJECT(displayed_list), "key_release_event",
			    GTK_SIGNAL_FUNC (on_displayed_list_key_release_event),
			    NULL);
	
	if (visible_games != 0)
		gtk_signal_connect (GTK_OBJECT (displayed_list), "button_press_event",
        		    	    GTK_SIGNAL_FUNC (on_row_right_clic),
        		   	    NULL);
	
	gtk_clist_columns_autosize(GTK_CLIST(displayed_list));
	gtk_clist_thaw(GTK_CLIST(displayed_list));

	gtk_widget_show_all(main_gui.scrolled_window_games);
	
	message = g_strdup_printf("%d %s",visible_games,visible_games>1?_("games"):_("game"));
	gtk_statusbar_pop (main_gui.statusbar3, 1);
	gtk_statusbar_push (main_gui.statusbar3, 1, message);
	g_free(message);
	
	/* allow to correctly update the page_size and page_increment of the GtkAdjustment of the GtkClist*/
	while (gtk_events_pending()) gtk_main_iteration();
	
	if (currentgame_row>0)
	{
	        gtk_clist_select_row(GTK_CLIST(displayed_list),currentgame_row,0);
		gtk_clist_moveto(GTK_CLIST(displayed_list),currentgame_row,0,0.5,0);
	}
	else
	{
	        gtk_clist_select_row(GTK_CLIST(displayed_list),0,0);
		gtk_clist_moveto(GTK_CLIST(displayed_list),0,0,0,0);
	}
	if (visible_games==0)
		select_game(NULL);
	
	g_hash_table_foreach(original_htable, (GHFunc)free_only_key, NULL);
	dirty_icon_cache = FALSE;
}


void display_details_list (void)
{
	RomEntry *tmprom;
	GList *listpointer,*current_column;
  	gint i;
	GtkWidget *displayed_list;
	gchar **Textes = NULL;
	int currentgame_row=-1;
	/*FIXME this is just to display the game position properly
	  when I got the strange 0.0 position bug*/
	static gboolean first_run = TRUE;

	GdkPixmap *pixmap_unknown;
	GdkBitmap *mask_unknown;
	GdkPixmap *pixmap_rom;
	GdkBitmap *mask_rom;
	GdkPixmap *pixmap_norom;
	GdkBitmap *mask_norom;
	GdkPixmap *pixmap_redx;
	GdkBitmap *mask_redx;
	GdkPixmap *testpix = NULL;
	GdkPixmap *testmask = NULL;

	gchar *message;
	int j=0;

	GtkStyle *liststyle;
	
	
	Textes = g_new(gchar *,g_list_length(Columns));
	
	get_status_icons();
	gdk_pixbuf_render_pixmap_and_mask(Status_Icons[INCORRECT],
					&pixmap_norom,
					&mask_norom,
					127);
	gdk_pixbuf_render_pixmap_and_mask(Status_Icons[CORRECT],
					&pixmap_rom,
					&mask_rom,
					127);
	gdk_pixbuf_render_pixmap_and_mask(Status_Icons[UNKNOWN],
					&pixmap_unknown,
					&mask_unknown,
					127);
	gdk_pixbuf_render_pixmap_and_mask(Status_Icons[PROBLEMS],
					&pixmap_redx,
					&mask_redx,
					127);
	for(current_column = g_list_first(Columns),i=0;
			(current_column != NULL);
			current_column = g_list_next(current_column),i++)
	{
		switch (GPOINTER_TO_INT(current_column->data))
		{
			case (GAMENAME):
				Textes[i] = g_strdup(_("Game"));
				break;
			case (HAS_ROMS):
				Textes[i] = g_strdup(_("ROMs"));
				break;
			case (HAS_SAMPLES):
				Textes[i] = g_strdup(_("Samples"));
				break;
			case (ROMNAME):
				Textes[i] = g_strdup(_("Directory"));
				break;
			case (VECTOR):
				Textes[i] = g_strdup(_("Type"));
				break;
			case (CONTROL):
				Textes[i] = g_strdup(_("Trackball"));
				break;
			case (TIMESPLAYED):
				Textes[i] = g_strdup(_("Played"));
				break;
			case (MANU):
				Textes[i] = g_strdup(_("Manufacturer"));
				break;
			case (YEAR):
				Textes[i] = g_strdup(_("Year"));
				break;
			case (CLONE):
				Textes[i] = g_strdup(_("Clone of"));
				break;
			case (DRIVER):
				Textes[i] = g_strdup(_("Driver"));
				break;
			case (ROMOF):
				Textes[i] = g_strdup(_("Rom of"));
				break;
			case (CATEGORY):
				Textes[i] = g_strdup(_("Category"));
				break;
			case (MAMEVER):
				Textes[i] = g_strdup(_("Version"));
				break;
		}
	}
	
	displayed_list = gtk_clist_new_with_titles(g_list_length(Columns), Textes);
	for (i=0 ; i<g_list_length(Columns) ; i++)
		g_free(Textes[i]);
	gtk_clist_set_selection_mode(GTK_CLIST(displayed_list), GTK_SELECTION_BROWSE);
	gtk_container_add(GTK_CONTAINER(main_gui.scrolled_window_games), displayed_list);

	/* if a font as already be designated, use it */
	if (gui_prefs.ListFontStruct ) {
		liststyle = gtk_style_copy(gtk_widget_get_style(GTK_WIDGET(displayed_list)));
		if (liststyle == NULL) {
			gtk_style_unref(liststyle);
			liststyle = gtk_style_new();
			gxmame_debug("creating new style");
		}
		gxmame_debug("use style.");
		liststyle->font = gui_prefs.ListFontStruct;
		gtk_widget_set_style(GTK_WIDGET(displayed_list), liststyle);
	}
	gtk_clist_set_row_height(GTK_CLIST(displayed_list),gui_prefs.ListFontHeight);
	gtk_clist_freeze(GTK_CLIST(displayed_list));

	for(listpointer = g_list_first(roms);
			(listpointer != NULL);
			 listpointer= g_list_next(listpointer))
	{
		tmprom = (RomEntry *) listpointer->data;
		/* the icon cache is dirty (change fontsize or display) for all game i.e. before applying filter */
		if (dirty_icon_cache && (tmprom->icon_pixbuf != NULL))
		{
			gdk_pixbuf_unref(tmprom->icon_pixbuf);
			tmprom->icon_pixbuf = NULL;
		}
		if (game_filtered(tmprom)){
			for(current_column = g_list_first(Columns),i=0;
					(current_column != NULL);
					current_column = g_list_next(current_column),i++)
			{
				switch (GPOINTER_TO_INT(current_column->data))
				{
					case (GAMENAME):
						if (tmprom->the_trailer && gui_prefs.ModifyThe)
							Textes[i] = g_strstrip(g_strdup_printf("%s, The %s",tmprom->gamename,tmprom->gamenameext));
						else if (tmprom->the_trailer && !gui_prefs.ModifyThe)
							Textes[i] = g_strstrip(g_strdup_printf("The %s %s",tmprom->gamename,tmprom->gamenameext));
						else
							Textes[i] = g_strstrip(g_strdup_printf("%s %s",tmprom->gamename,tmprom->gamenameext));
						break;
					case (HAS_ROMS):
						Textes[i] = g_strndup(tmprom->has_roms?_("Yes"):_("No"), MAX_GAMENAME);
						break;
					case (HAS_SAMPLES):
						if (tmprom->nb_samples==0)
							Textes[i] = g_strndup(" ", MAX_GAMENAME);
						else 
						{
							Textes[i] = g_strndup(tmprom->has_samples?_("Yes"):_("No"), MAX_GAMENAME);
						}
						break;
					case (ROMNAME):
						Textes[i] = g_strndup(tmprom->romname, MAX_ROMNAME);
						break;
					case (VECTOR):
						Textes[i] = g_strndup(tmprom->vector?_("Vector"):_("Raster"), MAX_ROMNAME);
						break;
					case (CONTROL):
						if (!strcmp(tmprom->control,"trackball"))
							Textes[i] = g_strndup(_("Yes"), MAX_ROMNAME);
						else
							Textes[i] = g_strndup(_("No"), MAX_ROMNAME);
						break;
					case (TIMESPLAYED):
						Textes[i] = g_strdup_printf("%d", tmprom->timesplayed);
						break;
					case (MANU):
						Textes[i] = g_strndup(tmprom->manu, MAX_MANU);
						break;
					case (YEAR):
						Textes[i] = g_strndup(tmprom->year, MAX_YEAR);
						break;
					case (CLONE):
						Textes[i] = g_strndup(tmprom->cloneof, MAX_ROMNAME);
						break;
					case (DRIVER):
						Textes[i] = g_strndup(tmprom->driver, MAX_ROMNAME);
						break;
					case (ROMOF):
						Textes[i] = g_strndup(tmprom->romof, MAX_ROMNAME);
						break;
					case (CATEGORY):
						Textes[i] = g_strndup(tmprom->category, MAX_CATEGORY);
						break;
					case (MAMEVER):
						Textes[i] = g_strndup(tmprom->mame_ver_added, MAX_MAMEVER);
						break;
				}
			}
			
			
			tmprom->position = j = gtk_clist_append(GTK_CLIST(displayed_list), Textes);
	/* Should be the first column and not move (perhaps remove this first column from the columns list?)*/
			if (tmprom->has_roms == 2)
				gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
	                                      j,
	                                      0,
	                                      Textes[0],
	                                      5,
	                                      pixmap_unknown,
	                                      mask_unknown);
			else if (tmprom->has_roms == 0)
				gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
	                                      j,
	                                      0,
	                                      Textes[0],
	                                      5,
	                                      pixmap_norom,
	                                      mask_norom);
			else if (!tmprom->status)
				gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
	                                      j,
	                                      0,
	                                      Textes[0],
	                                      5,
	                                      pixmap_redx,
	                                      mask_redx);
			else
			{
				if (tmprom->icon_pixbuf != NULL)
				{
					gdk_pixbuf_render_pixmap_and_mask(tmprom->icon_pixbuf,
								&testpix,
								&testmask,
								127);
					gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
					              j,
					              0,
					              Textes[0],
					              5,
					              testpix,
					              testmask);
				}else
				{	
					gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
					              j,
					              0,
					              Textes[0],
					              5,
					              pixmap_rom,
					              mask_rom);
				}
			}
			gtk_clist_set_row_data(GTK_CLIST(displayed_list),j,tmprom);
			
			if (!strncmp(Textes[0],gui_prefs.DefaultGame,MAX_GAMENAME))
				currentgame_row=j; 
			
			for (i=0 ; i<g_list_length(Columns) ; i++)
				g_free(Textes[i]);
		/* set clone color */
			if (strcmp(tmprom->cloneof,"-"))
			{
				gtk_clist_set_foreground (GTK_CLIST(displayed_list),
                                             		j,
							&(gui_prefs.clone_color));
			}
			/*allows to display the correct number of games*/
			j++;
		}	
		else
			tmprom->position = -1;
	}
	
	g_free(Textes);
	gdk_pixmap_unref(pixmap_unknown);
	gdk_pixmap_unref(mask_unknown);
	gdk_pixmap_unref(pixmap_rom);
	gdk_pixmap_unref(mask_rom);
	gdk_pixmap_unref(pixmap_norom);
	gdk_pixmap_unref(mask_norom);
	gdk_pixmap_unref(pixmap_redx);
	gdk_pixmap_unref(mask_redx);
	
	for(current_column = g_list_first(Columns),i=0;
			(current_column != NULL);
			current_column = g_list_next(current_column),i++)
	{
		switch (GPOINTER_TO_INT(current_column->data))
		{
			case (GAMENAME):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,200);
				break;
			case (HAS_ROMS):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,50);
				break;
			case (HAS_SAMPLES):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,50);
				break;
			case (ROMNAME):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,80);
				break;
			case (VECTOR):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,50);
				break;
			case (CONTROL):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,60);
				break;
			case (TIMESPLAYED):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,50);
				break;
			case (MANU):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,110);
				break;
			case (YEAR):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,50);
				break;
			case (CLONE):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,80);
				break;
			case (DRIVER):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,60);
				break;
			case (ROMOF):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,145);
				break;
			case (CATEGORY):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,100);
				break;
		}
	}
	visible_games = j;
	
	main_gui.displayed_list = displayed_list;
	/* 2 actif buttons */
	for (i = 0; i < 2; i++)
		gtk_clist_set_button_actions(GTK_CLIST(displayed_list), i, GTK_BUTTON_SELECTS);

	gtk_signal_connect (GTK_OBJECT (displayed_list), "select_row",
        		    GTK_SIGNAL_FUNC (on_row_selected),
        		    NULL);
	gtk_signal_connect (GTK_OBJECT(displayed_list), "key_press_event",
			    GTK_SIGNAL_FUNC (on_displayed_list_key_press_event),
			    NULL);
	gtk_signal_connect (GTK_OBJECT(displayed_list), "key_release_event",
			    GTK_SIGNAL_FUNC (on_displayed_list_key_release_event),
			    NULL);

	if (visible_games != 0)
		gtk_signal_connect (GTK_OBJECT (displayed_list), "button_press_event",
        		    	    GTK_SIGNAL_FUNC (on_row_right_clic),
        		   	    NULL);
	

	gtk_clist_thaw(GTK_CLIST(displayed_list));

	gtk_widget_show_all(main_gui.scrolled_window_games);

	message = g_strdup_printf("%d %s",visible_games,visible_games>1?_("games"):_("game"));
	gtk_statusbar_pop (main_gui.statusbar3, 1);
	gtk_statusbar_push (main_gui.statusbar3, 1, message);
	g_free(message);

	/* allow to correctly update the page_size and page_increment of the GtkAdjustment of the GtkClist*/
	/*FIXME this is just to display the game position properly
	  when I got the strange 0.0 position bug*/
	/* 0.28 doesn't occur anymore _O ?!?! */
/*	if(!first_run || !(gui_prefs.GUIPosX && gui_prefs.GUIPosY))*/
		while (gtk_events_pending())
			gtk_main_iteration();
	if (currentgame_row>0)
	{
	        gtk_clist_select_row(GTK_CLIST(displayed_list),currentgame_row,0);
		gtk_clist_moveto(GTK_CLIST(displayed_list),currentgame_row,0,0.5,0);
	}
	else
	{
	        gtk_clist_select_row(GTK_CLIST(displayed_list),0,0);
		gtk_clist_moveto(GTK_CLIST(displayed_list),0,0,0,0);
	}
	if (visible_games==0)
		select_game(NULL);
	first_run = FALSE;
	dirty_icon_cache = FALSE;
}

/*a test using list instead of tree */
void display_indented2_list (void)
{
	RomEntry *tmprom;
	GList *listpointer,*current_column;
  	gint i;
	GtkWidget *displayed_list;
	gchar **Textes = NULL;
	int currentgame_row=-1;
	/*FIXME this is just to display the game position properly
	  when I got the strange 0.0 position bug*/
	static gboolean first_run = TRUE;

	GdkPixmap *pixmap_unknown;
	GdkBitmap *mask_unknown;
	GdkPixmap *pixmap_rom;
	GdkBitmap *mask_rom;
	GdkPixmap *pixmap_norom;
	GdkBitmap *mask_norom;
	GdkPixmap *pixmap_redx;
	GdkBitmap *mask_redx;
	GdkPixmap *testpix = NULL;
	GdkPixmap *testmask = NULL;

	GtkStyle *liststyle;
	gchar *message;
	int j=0;

	Textes = g_new(gchar *,g_list_length(Columns));
	
	get_status_icons();
	gdk_pixbuf_render_pixmap_and_mask(Status_Icons[INCORRECT],
					&pixmap_norom,
					&mask_norom,
					127);
	gdk_pixbuf_render_pixmap_and_mask(Status_Icons[CORRECT],
					&pixmap_rom,
					&mask_rom,
					127);
	gdk_pixbuf_render_pixmap_and_mask(Status_Icons[UNKNOWN],
					&pixmap_unknown,
					&mask_unknown,
					127);
	gdk_pixbuf_render_pixmap_and_mask(Status_Icons[PROBLEMS],
					&pixmap_redx,
					&mask_redx,
					127);
	
	for(current_column = g_list_first(Columns),i=0;
			(current_column != NULL);
			current_column = g_list_next(current_column),i++)
	{
		switch (GPOINTER_TO_INT(current_column->data))
		{
			case (GAMENAME):
				Textes[i] = g_strdup(_("Game"));
				break;
			case (HAS_ROMS):
				Textes[i] = g_strdup(_("ROMs"));
				break;
			case (HAS_SAMPLES):
				Textes[i] = g_strdup(_("Samples"));
				break;
			case (ROMNAME):
				Textes[i] = g_strdup(_("Directory"));
				break;
			case (VECTOR):
				Textes[i] = g_strdup(_("Type"));
				break;
			case (CONTROL):
				Textes[i] = g_strdup(_("Trackball"));
				break;
			case (TIMESPLAYED):
				Textes[i] = g_strdup(_("Played"));
				break;
			case (MANU):
				Textes[i] = g_strdup(_("Manufacturer"));
				break;
			case (YEAR):
				Textes[i] = g_strdup(_("Year"));
				break;
			case (CLONE):
				Textes[i] = g_strdup(_("Clone of"));
				break;
			case (DRIVER):
				Textes[i] = g_strdup(_("Driver"));
				break;
			case (ROMOF):
				Textes[i] = g_strdup(_("Rom of"));
				break;
			case (CATEGORY):
				Textes[i] = g_strdup(_("Category"));
				break;
			case (MAMEVER):
				Textes[i] = g_strdup(_("Version"));
				break;
		}
	}
	
	displayed_list = gtk_clist_new_with_titles(g_list_length(Columns), Textes);
	for (i=0 ; i<g_list_length(Columns) ; i++)
		g_free(Textes[i]);
	gtk_clist_set_selection_mode(GTK_CLIST(displayed_list), GTK_SELECTION_BROWSE);
	gtk_container_add(GTK_CONTAINER(main_gui.scrolled_window_games), displayed_list);

	/* if a font as already be designated, use it */
	if (gui_prefs.ListFontStruct ) {
		liststyle = gtk_style_copy(gtk_widget_get_style(GTK_WIDGET(displayed_list)));
		if (liststyle == NULL) {
			gtk_style_unref(liststyle);
			liststyle = gtk_style_new();
			gxmame_debug("creating new style");
		}
		gxmame_debug("use style.");
		liststyle->font = gui_prefs.ListFontStruct;
		gtk_widget_set_style(GTK_WIDGET(displayed_list), liststyle);
	}
	gtk_clist_set_row_height(GTK_CLIST(displayed_list),gui_prefs.ListFontHeight);
	gtk_clist_freeze(GTK_CLIST(displayed_list));

	for(listpointer = g_list_first(roms);
			(listpointer != NULL);
			 listpointer= g_list_next(listpointer))
	{
		tmprom = (RomEntry *) listpointer->data;
		/* the icon cache is dirty (change fontsize or display) */
		if (dirty_icon_cache && (tmprom->icon_pixbuf != NULL))
		{
			gdk_pixbuf_unref(tmprom->icon_pixbuf);
			tmprom->icon_pixbuf = NULL;
		}
		if (game_filtered(tmprom)){
			for(current_column = g_list_first(Columns),i=0;
					(current_column != NULL);
					current_column = g_list_next(current_column),i++)
			{
				switch (GPOINTER_TO_INT(current_column->data))
				{
					case (GAMENAME):
						if (tmprom->the_trailer && gui_prefs.ModifyThe)
							Textes[i] = g_strstrip(g_strdup_printf("%s, The %s",tmprom->gamename,tmprom->gamenameext));
						else if (tmprom->the_trailer && !gui_prefs.ModifyThe)
							Textes[i] = g_strstrip(g_strdup_printf("The %s %s",tmprom->gamename,tmprom->gamenameext));
						else
							Textes[i] = g_strstrip(g_strdup_printf("%s %s",tmprom->gamename,tmprom->gamenameext));
						break;
					case (HAS_ROMS):
						Textes[i] = g_strndup(tmprom->has_roms?_("Yes"):_("No"), MAX_GAMENAME);
						break;
					case (HAS_SAMPLES):
						if (tmprom->nb_samples==0)
							Textes[i] = g_strndup(" ", MAX_GAMENAME);
						else 
						{
							Textes[i] = g_strndup(tmprom->has_samples?_("Yes"):_("No"), MAX_GAMENAME);
						}
						break;
					case (ROMNAME):
						Textes[i] = g_strndup(tmprom->romname, MAX_ROMNAME);
						break;
					case (VECTOR):
						Textes[i] = g_strndup(tmprom->vector?_("Vector"):_("Raster"), MAX_ROMNAME);
						break;
					case (CONTROL):
						if (!strcmp(tmprom->control,"trackball"))
							Textes[i] = g_strndup(_("Yes"), MAX_ROMNAME);
						else
							Textes[i] = g_strndup(_("No"), MAX_ROMNAME);
						break;
					case (TIMESPLAYED):
						Textes[i] = g_strdup_printf("%d", tmprom->timesplayed);
						break;
					case (MANU):
						Textes[i] = g_strndup(tmprom->manu, MAX_MANU);
						break;
					case (YEAR):
						Textes[i] = g_strndup(tmprom->year, MAX_YEAR);
						break;
					case (CLONE):
						Textes[i] = g_strndup(tmprom->cloneof, MAX_ROMNAME);
						break;
					case (DRIVER):
						Textes[i] = g_strndup(tmprom->driver, MAX_ROMNAME);
						break;
					case (ROMOF):
						Textes[i] = g_strndup(tmprom->romof, MAX_ROMNAME);
						break;
					case (CATEGORY):
						Textes[i] = g_strndup(tmprom->category, MAX_CATEGORY);
						break;
					case (MAMEVER):
						Textes[i] = g_strndup(tmprom->mame_ver_added, MAX_MAMEVER);
						break;
				}
			}
	
			
			tmprom->position = j = gtk_clist_append(GTK_CLIST(displayed_list), Textes);
	/* Should be the first column and not move (perhaps remove this first column from the columns list?)*/
			if (tmprom->has_roms == 2)
				gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
	                                      j,
	                                      0,
	                                      Textes[0],
	                                      5,
	                                      pixmap_unknown,
	                                      mask_unknown);
			else if (tmprom->has_roms == 0)
				gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
	                                      j,
	                                      0,
	                                      Textes[0],
	                                      5,
	                                      pixmap_norom,
	                                      mask_norom);
			else if (!tmprom->status)
				gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
	                                      j,
	                                      0,
	                                      Textes[0],
	                                      5,
	                                      pixmap_redx,
	                                      mask_redx);
			else
			{
				if (tmprom->icon_pixbuf != NULL)
				{
					gdk_pixbuf_render_pixmap_and_mask(tmprom->icon_pixbuf,
								&testpix,
								&testmask,
								127);
					gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
					              j,
					              0,
					              Textes[0],
					              5,
					              testpix,
					              testmask);
				}else
				{
					gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
					              j,
					              0,
					              Textes[0],
					              5,
					              pixmap_rom,
					              mask_rom);
				}
			}
	
			gtk_clist_set_row_data(GTK_CLIST(displayed_list),j,tmprom);
			if (strcmp(tmprom->cloneof,"-"))
			{
				gtk_clist_set_foreground (GTK_CLIST(displayed_list),
                                             		j,
							&(gui_prefs.clone_color));
				gtk_clist_set_shift(GTK_CLIST(displayed_list),
                                             	j,
                                             	0,
                                             	0,
                                             	16+5);
			}
			
			if (!strncmp(Textes[0],gui_prefs.DefaultGame,MAX_GAMENAME))
				currentgame_row=j; 
			
			for (i=0 ; i<g_list_length(Columns) ; i++)
				g_free(Textes[i]);
			/*allows to display the correct number of games*/
			j++;
		}	
		else
			tmprom->position = -1;
	}
	
	g_free(Textes);
	gdk_pixmap_unref(pixmap_unknown);
	gdk_pixmap_unref(mask_unknown);
	gdk_pixmap_unref(pixmap_rom);
	gdk_pixmap_unref(mask_rom);
	gdk_pixmap_unref(pixmap_norom);
	gdk_pixmap_unref(mask_norom);
	gdk_pixmap_unref(pixmap_redx);
	gdk_pixmap_unref(mask_redx);
	
	for(current_column = g_list_first(Columns),i=0;
			(current_column != NULL);
			current_column = g_list_next(current_column),i++)
	{
		switch (GPOINTER_TO_INT(current_column->data))
		{
			case (GAMENAME):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,200);
				break;
			case (HAS_ROMS):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,50);
				break;
			case (HAS_SAMPLES):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,50);
				break;
			case (ROMNAME):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,80);
				break;
			case (VECTOR):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,50);
				break;
			case (CONTROL):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,60);
				break;
			case (TIMESPLAYED):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,50);
				break;
			case (MANU):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,110);
				break;
			case (YEAR):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,50);
				break;
			case (CLONE):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,80);
				break;
			case (DRIVER):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,60);
				break;
			case (ROMOF):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,145);
				break;
			case (CATEGORY):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,100);
				break;
		}
	}
	visible_games = j;
	
	main_gui.displayed_list = displayed_list;
	/* 2 actif buttons */
	for (i = 0; i < 2; i++)
		gtk_clist_set_button_actions(GTK_CLIST(displayed_list), i, GTK_BUTTON_SELECTS);

	gtk_signal_connect (GTK_OBJECT (displayed_list), "select_row",
        		    GTK_SIGNAL_FUNC (on_row_selected),
        		    NULL);
	gtk_signal_connect (GTK_OBJECT(displayed_list), "key_press_event",
			    GTK_SIGNAL_FUNC (on_displayed_list_key_press_event),
			    NULL);
	gtk_signal_connect (GTK_OBJECT(displayed_list), "key_release_event",
			    GTK_SIGNAL_FUNC (on_displayed_list_key_release_event),
			    NULL);

	if (visible_games != 0)
		gtk_signal_connect (GTK_OBJECT (displayed_list), "button_press_event",
        		    	    GTK_SIGNAL_FUNC (on_row_right_clic),
        		   	    NULL);
	

	gtk_clist_thaw(GTK_CLIST(displayed_list));

	gtk_widget_show_all(main_gui.scrolled_window_games);

	message = g_strdup_printf("%d %s",visible_games,visible_games>1?_("games"):_("game"));
	gtk_statusbar_pop (main_gui.statusbar3, 1);
	gtk_statusbar_push (main_gui.statusbar3, 1, message);
	g_free(message);

	/* allow to correctly update the page_size and page_increment of the GtkAdjustment of the GtkClist*/
	/*FIXME this is just to display the game position properly
	  when I got the strange 0.0 position bug*/
	/* 0.28 doesn't occur anymore _O ?!?! */
/*	if(!first_run || !(gui_prefs.GUIPosX && gui_prefs.GUIPosY))*/
		while (gtk_events_pending())
			gtk_main_iteration();
	if (currentgame_row>0)
	{
	        gtk_clist_select_row(GTK_CLIST(displayed_list),currentgame_row,0);
		gtk_clist_moveto(GTK_CLIST(displayed_list),currentgame_row,0,0.5,0);
	}
	else
	{
	        gtk_clist_select_row(GTK_CLIST(displayed_list),0,0);
		gtk_clist_moveto(GTK_CLIST(displayed_list),0,0,0,0);
	}
	if (visible_games==0)
		select_game(NULL);
	first_run = FALSE;
	dirty_icon_cache = FALSE;
}

void create_gamelist_content(void)
{
	GtkWidget *displayed_list;
	
	displayed_list = GTK_BIN(main_gui.scrolled_window_games)->child;
	
	/* Free the previous structures */
	/* hide, empty, remove then destroy the content of scrolled window games */
	switch (gui_prefs.previous_mode)
	{
		case (LARGE_ICONS):
			gxmame_debug("free Large icons(not implemented yet)");
			break;
		case (SMALL_ICONS):
			gxmame_debug("free Small icons");
			gtk_widget_hide(displayed_list);
			gtk_clist_clear(GTK_CLIST(displayed_list));
			gtk_container_remove(GTK_CONTAINER(main_gui.scrolled_window_games),displayed_list);
/*			not needed because gtk_container_remove the widget that only had this reference
			gtk_widget_destroy(displayed_list);*/
			break;
		case (LIST):
			gxmame_debug("free gtk_clist (not implemented yet)");
			gxmame_debug("test of indented list using clist not ctree");
			gtk_widget_hide(displayed_list);
			gxmame_debug("Free gtk_clist (Indented(clist))");
			gtk_clist_clear(GTK_CLIST(displayed_list));
			gtk_container_remove(GTK_CONTAINER(main_gui.scrolled_window_games),displayed_list);
			break;
		case (DETAILS):
			gtk_widget_hide(displayed_list);
			gxmame_debug("Free gtk_clist (Details)");
			gtk_clist_clear(GTK_CLIST(displayed_list));
			gtk_container_remove(GTK_CONTAINER(main_gui.scrolled_window_games),displayed_list);
/*			not needed because gtk_container_remove the widget that only had this reference
			gtk_widget_destroy(displayed_list);*/
			break;
		case (INDENTED):
			gtk_widget_hide(displayed_list);
			gxmame_debug("Free gtk_clist (Indented)");
			gtk_clist_clear(GTK_CLIST(displayed_list));
			gtk_container_remove(GTK_CONTAINER(main_gui.scrolled_window_games),displayed_list);
			break;
	}	
	
	
	/* Test if the glist need to be sorted*/
	/*if (!glist_games_sorted())
		g_list_sort (roms, compare_func);*/
	/* create the new structures */
	switch (gui_prefs.current_mode)
	{
		case (LARGE_ICONS):
			gxmame_debug("large icons mode not implemented yet");
			break;
		case (SMALL_ICONS):
			gxmame_debug("Display small icons mode");
			display_small_icon_list();
			break;
		case (LIST):
			gxmame_debug("list mode not implemented yet");
			gxmame_debug("test of indented list using clist not ctree");
			display_indented2_list();
			break;
		case (DETAILS):
			gxmame_debug("Display details mode");
			display_details_list();
			break;
		case (INDENTED):
			gxmame_debug("Display indented mode");
			display_indented_list();
			break;
	}			
}




void set_icon(GtkWidget * widget, const gchar * name)
{
	GtkStyle *style;
	GdkBitmap *icon_mask = NULL;
	GdkPixmap *icon_pm = NULL;

	style = gtk_widget_get_style(widget);

	icon_pm = gdk_pixmap_create_from_xpm_d(widget->window,
					       &icon_mask,
					       &style->bg
					       [GTK_STATE_NORMAL],
					       (gchar **)
					       mamel_xpm);

	gdk_window_set_icon(widget->window, NULL, icon_pm, icon_mask);
	gdk_window_set_icon_name(widget->window, name);

	g_free(icon_pm);
	g_free(icon_mask);
}

void select_game(RomEntry *rom)
{
	gchar *play_menu_name;

	if (rom!=NULL)
	{
		/* Maybe I should remove the string default gamename and only keep a pointer to the rom struct ? */
		gui_prefs.DefaultGame_p = rom;
		if (rom->the_trailer && gui_prefs.ModifyThe)
			g_snprintf(gui_prefs.DefaultGame,MAX_GAMENAME,"%s, The %s",rom->gamename,rom->gamenameext);
		else if (rom->the_trailer && !gui_prefs.ModifyThe)
			g_snprintf(gui_prefs.DefaultGame,MAX_GAMENAME,"The %s %s",rom->gamename,rom->gamenameext);
		else
			g_snprintf(gui_prefs.DefaultGame,MAX_GAMENAME,"%s %s",rom->gamename,rom->gamenameext);

		g_strstrip(gui_prefs.DefaultGame);
	
		/* update menus */
		gtk_widget_set_sensitive(GTK_WIDGET(main_gui.play_menu),TRUE);
		gtk_widget_set_sensitive(GTK_WIDGET(main_gui.properties_menu),TRUE);
		gtk_widget_set_sensitive(GTK_WIDGET(main_gui.add_to_favorites),!rom->favourite);
		gtk_widget_set_sensitive(GTK_WIDGET(main_gui.remove_from_favorites),rom->favourite);
		gtk_widget_set_sensitive(GTK_WIDGET(main_gui.popup_add_to_favorites),!rom->favourite);
		gtk_widget_set_sensitive(GTK_WIDGET(main_gui.popup_remove_from_favorites),rom->favourite);
	
		play_menu_name = g_strdup_printf(_("_Play %s"),gui_prefs.DefaultGame);
	  	gtk_label_parse_uline (GTK_LABEL (GTK_BIN (main_gui.play_menu)->child),play_menu_name);
	  	gtk_label_parse_uline (GTK_LABEL (GTK_BIN (main_gui.popup_play_menu)->child),play_menu_name);
		g_free(play_menu_name);
	
		/* update statusbar */
		gtk_statusbar_pop (main_gui.statusbar1, 1);
		gtk_statusbar_push (main_gui.statusbar1, 1, gui_prefs.DefaultGame);
		gtk_statusbar_pop (main_gui.statusbar2, 1);
		if (rom->has_roms == 2)
			gtk_statusbar_push (main_gui.statusbar2, 1, _("UNKNOWN"));
		else if (rom->has_roms == 0)
			gtk_statusbar_push (main_gui.statusbar2, 1, _("ROMs missing"));
		else if (!rom->status)
			gtk_statusbar_push (main_gui.statusbar2, 1, _("Not working"));
		else
			gtk_statusbar_push (main_gui.statusbar2, 1, _("Working"));

		/* update screenshot panel */
		update_screenshot_panel(rom);
	}
	else /* no roms selected display the default picture */
	{
		gxmame_debug("no games selected");
		/* update menus */
		gtk_widget_set_sensitive(GTK_WIDGET(main_gui.play_menu),FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(main_gui.properties_menu),FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(main_gui.add_to_favorites),FALSE);
		gtk_widget_set_sensitive(GTK_WIDGET(main_gui.remove_from_favorites),FALSE);
	
		play_menu_name = g_strdup_printf(_("_Play"));
	  	gtk_label_parse_uline (GTK_LABEL (GTK_BIN (main_gui.play_menu)->child),play_menu_name);
	  	gtk_label_parse_uline (GTK_LABEL (GTK_BIN (main_gui.popup_play_menu)->child),play_menu_name);
		g_free(play_menu_name);
	
		/* update statusbar */
		gtk_statusbar_pop (main_gui.statusbar1, 1);
		gtk_statusbar_push (main_gui.statusbar1, 1, _("No game selected"));
		gtk_statusbar_pop (main_gui.statusbar2, 1);
		gtk_statusbar_push (main_gui.statusbar2, 1, " ");

		/* update screenshot panel */
		update_screenshot_panel(NULL);

	}
	
}


void show_progress_bar(void)
{
	if(gui_prefs.ShowStatusBar)
	{
		gtk_widget_hide (GTK_WIDGET(main_gui.tri_status_bar));
		gtk_statusbar_push(main_gui.status_progress_bar, 1, _("Game search 0%% complete"));
		gtk_widget_show (GTK_WIDGET(main_gui.combo_progress_bar));
	}	
}


void hide_progress_bar(void)
{
	if(gui_prefs.ShowStatusBar)
	{
		gtk_widget_hide (GTK_WIDGET(main_gui.combo_progress_bar));
		gtk_statusbar_pop(main_gui.status_progress_bar, 1);
		gtk_widget_show (GTK_WIDGET(main_gui.tri_status_bar));
	}	
}

void update_progress_bar(gfloat current_value)
{
	static gint current_displayed_value;
	gchar *displayed_message;
	if (current_displayed_value!=(gint)(current_value*100))
	{
		current_displayed_value=(gint)(current_value*100);
		displayed_message = g_strdup_printf(_("Game search %i%% complete"),current_displayed_value);
		gtk_statusbar_pop(main_gui.status_progress_bar, 1);
		gtk_statusbar_push(main_gui.status_progress_bar, 1, displayed_message);
	}
	gtk_progress_bar_update(main_gui.progress_progress_bar,current_value);
}

void update_game_in_list (guint position)
{
	gchar **Texte;
	gchar *Textes;
	GList *current_column;
	GdkPixmap *current_pixmap;
	GdkBitmap *current_mask;
	RomEntry *tmprom;
	
	tmprom = (RomEntry *)gtk_clist_get_row_data(GTK_CLIST(main_gui.displayed_list),position);		
	/* is the *[] really needed? a simple * could be enough? */
	Texte = g_malloc(sizeof(gchar *));
	
	if (tmprom->the_trailer && gui_prefs.ModifyThe)
		Texte[0] = g_strstrip(g_strdup_printf("%s, The %s",tmprom->gamename,tmprom->gamenameext));
	else if (tmprom->the_trailer && !gui_prefs.ModifyThe)
		Texte[0] = g_strstrip(g_strdup_printf("The %s %s",tmprom->gamename,tmprom->gamenameext));
	else
		Texte[0] = g_strstrip(g_strdup_printf("%s %s",tmprom->gamename,tmprom->gamenameext));

	get_status_icons();
	if (tmprom->has_roms == 2)
	{
		gdk_pixbuf_render_pixmap_and_mask(Status_Icons[UNKNOWN],
						&current_pixmap,
						&current_mask,
						127);
	}
	else if (tmprom->has_roms == 0)
	{
		gdk_pixbuf_render_pixmap_and_mask(Status_Icons[INCORRECT],
						&current_pixmap,
						&current_mask,
						127);
	}
	else if (!tmprom->status)
	{
		gdk_pixbuf_render_pixmap_and_mask(Status_Icons[PROBLEMS],
						&current_pixmap,
						&current_mask,
						127);
	}
	else
	{
 		if (tmprom->icon_pixbuf != NULL)
		{
			gdk_pixbuf_render_pixmap_and_mask(tmprom->icon_pixbuf,
						&current_pixmap,
						&current_mask,
						127);
		}
		else
		{
		gdk_pixbuf_render_pixmap_and_mask(Status_Icons[CORRECT],
						&current_pixmap,
						&current_mask,
						127);
		}
	}
	
	if (gui_prefs.current_mode == SMALL_ICONS)
	{
		gxmame_debug("update game display small icons mode");
			gtk_clist_set_pixtext(GTK_CLIST(main_gui.displayed_list),
		                      position,
		                      0,
		                      Texte[0],
		                      5,
		                      current_pixmap,
		                      current_mask);
	}
	if (gui_prefs.current_mode == DETAILS || gui_prefs.current_mode == INDENTED)
	{
		int i;
		gxmame_debug("update game display details mode");
		for(current_column = g_list_first(Columns),i=0;
				(current_column != NULL);
				current_column = g_list_next(current_column),i++)
		{
			switch (GPOINTER_TO_INT(current_column->data))
			{
				case (HAS_ROMS):
					Textes = g_strndup(tmprom->has_roms?_("Yes"):_("No"), MAX_GAMENAME);
					gtk_clist_set_text(GTK_CLIST(main_gui.displayed_list),
								position,i,Textes);
					break;
				case (HAS_SAMPLES):
					if (tmprom->nb_samples==0)
						Textes = g_strndup(" ", MAX_GAMENAME);
					else 
					{
						Textes = g_strndup(tmprom->has_samples?_("Yes"):_("No"), MAX_GAMENAME);
					}
					gtk_clist_set_text(GTK_CLIST(main_gui.displayed_list),
								position,i,Textes);
					break;
				case (TIMESPLAYED):
					Textes = g_strdup_printf("%d", tmprom->timesplayed);
					gtk_clist_set_text(GTK_CLIST(main_gui.displayed_list),
								position,i,Textes);
					break;
				/* in theory should never be something else than the first column */
				case(GAMENAME):
						gtk_clist_set_pixtext(GTK_CLIST(main_gui.displayed_list),
					                      position,
					                      0,
					                      Texte[0],
					                      5,
					                      current_pixmap,
					                      current_mask);
					break;
			}
		}
	}
	/*If I try to update the status bar here, I'll updtate the status bar for every games when I run a quicklist
	  I have to select the game when I return from this function */
	/*select_game(tmprom);*/
	/* if we are in the favorite folder, and favorite flag of the rom is not set
	  we need to remove it from the CList (that imply that the game is in the CList )*/
	/* problems because the row values are completly changed, I redisplay the complete game list
	if ((current_filter->type == FAVORITE) && !tmprom->favourite)
	{
		gxmame_debug("removing %s from favorite list from position %i",tmprom->romname,position);
		gtk_clist_remove(GTK_CLIST(main_gui.displayed_list),position);
	}
	*/
	gdk_pixmap_unref(current_pixmap);
	gdk_pixmap_unref(current_mask);
}


gboolean set_game_history(const RomEntry *rom)
{
	FILE *history_file;
	GtkText *history_txt;
	gchar line[2000];
	gint i, n;
	gchar *tmp, *p;
	gchar **games;
	gboolean found_game = FALSE;
	gboolean pointer_in_info = FALSE;
	gboolean extra_newline = FALSE;

	history_txt = GTK_TEXT(main_gui.history_box);
	history_file = fopen(gui_prefs.HistoryFile, "r");

	if (!history_file)
	{
		gxmame_debug("History.dat file not found");
		return (FALSE);
	}

	/* Scan through the file. First look for '$',then $info=game, $bio and $end */
	while (fgets(line, 500, history_file)) {
		p = line;
		tmp = p;
		if (*tmp == '$' && !found_game) {
			/* Found a line */
			if (!strncmp(p, "$info", 5)) {
				p= tmp=p+6;
				/* It is an info line */
				i = 0;
				while (*tmp && (*tmp++ != '\n'))
					i++;
				/* Sometimes the list is continued on the next line */
				i--;
				do {
					p[i] = fgetc(history_file);
					if (p[i] == '\n')
						i--;
					if (p[i] == '\r')
						i--;
					i++;
	
                                } while ((p[i - 1] != '$') && i < 2000 /* buffer size */);
                                if (p[i - 1] == '$')
                                      ungetc('$', history_file);
                               
                                p[i - 1] = 0;
                                games = g_strsplit(p, ",", 20);
                                n = 0;

				while ((games[n] != NULL)) {
					games[n] = g_strchomp(games[n]);

					if (!strcmp(games[n], rom->romname)) {
						/* It is the info for the wanted game */
						found_game = TRUE;
					}
					n++;
				}
				g_strfreev(games);
			}
		} else if (found_game && (*tmp == '$')) {
			if (!strncmp(p, "$bio", 4))
				pointer_in_info = TRUE;
			if (!strncmp(p, "$end", 4)) {
				pointer_in_info = FALSE;
				break;
			}
		} else if (found_game && pointer_in_info) {
			i = 0;
			while (*tmp && (*tmp != '\r') &&(*tmp++ != '\n'))
				i++;
			if (i == 0)
			{/* and a new line but not severals*/
				if (!extra_newline)
				{
					extra_newline = TRUE;
				}
				else
					extra_newline = FALSE;
			}
			else
			{
				extra_newline = FALSE;
			}
			if (!extra_newline)
			{
				p[i] = 0;
/*				gxmame_debug("Added history txt: %s", p);*/
				p[i] = '\n';
				p[i + 1] = 0;
				gtk_text_insert(history_txt,NULL, NULL, NULL, p, -1);
			}
		}
	}

	if (!found_game)
	{
		gxmame_debug("no history info found for %s",rom->romname);
	}

	fclose(history_file);

	return (found_game);
}


gboolean set_game_info(const RomEntry * rom)
{
	FILE *mameinfo_dat;
	GtkText *mameinfo_txt;
	gchar line[2000];
	gint i;
	gchar *tmp, *p;
	gboolean found_game = FALSE;
	gboolean pointer_in_info = FALSE;
	gboolean extra_newline = FALSE;
	
	mameinfo_txt = GTK_TEXT(main_gui.history_box);
	mameinfo_dat = fopen(gui_prefs.MameInfoFile, "r");

	if (!mameinfo_dat)
	{
		gxmame_debug("mameinfo_dat file not found");
		return (FALSE);
	}

	/* Scan through the file. First look for '$',then $info=game, $mame and $end */
	while (fgets(line, 500, mameinfo_dat)) {
		p = line;
		tmp = p;
		/* Found a line */
		if (*tmp == '$' && !found_game) {
			/* It is an info line */
			if (!strncmp(p, "$info", 5)) {
				p= tmp=p+6;
				while (*tmp && (*tmp != '\n')&& (*tmp++ != '\r'));
				tmp--;*tmp='\0';
				/* it's the god game */
				if (!strcmp(p, rom->romname) || !strcmp(p, rom->cloneof)) {
					found_game = TRUE;
				}
			}
		} else if (found_game && (*tmp == '$')) {
			if (!strncmp(p, "$mame", 5))
				pointer_in_info = TRUE;
			else if (!strncmp(p, "$end", 4)) {
				pointer_in_info = FALSE;
				break;
			}
		} else if (found_game && pointer_in_info) {
			i = 0;
			while (*tmp && (*tmp != '\r') &&(*tmp++ != '\n'))
				i++;
			if (i == 0)
			{/* and a new line but not severals*/
				if (!extra_newline)
				{
					extra_newline = TRUE;
				}
				else
					extra_newline = FALSE;
			}
			else
			{
				extra_newline = FALSE;
			}
			if (!extra_newline)
			{
				p[i] = 0;
/*				gxmame_debug("Added info txt: %s", p);*/
				p[i] = '\n';
				p[i + 1] = 0;
				gtk_text_insert(mameinfo_txt,NULL, NULL, NULL, p, -1);
			}
		}
	}

	if (!found_game)
	{
		gxmame_debug("no MameInfo info found for %s",rom->romname);
	}

	fclose(mameinfo_dat);

	return (found_game);
}


void change_screenshot(GtkWidget       *widget,
		       GdkEventButton  *event,
		       gpointer         user_data)
{	/* prevent the mouse wheel (button 4 & 5) to change the screenshot*/
	if (event && event->button <= 3)
	{
		gui_prefs.ShowFlyer = (++gui_prefs.ShowFlyer)%5;
		update_screenshot_panel(gui_prefs.DefaultGame_p);
	}
}


void update_screenshot_panel(RomEntry *rom)
{
	GdkPixbuf *pixbuf, *scaled_pixbuf;
	GdkPixmap *testpix = NULL;
	GdkPixmap *testmask = NULL;
	GdkColor transparent;
	gchar *filename;
	gboolean had_info=FALSE,had_history=FALSE;
	
	if (rom!=NULL)
	{
		
		switch (gui_prefs.ShowFlyer)
		{
			case(SNAPSHOTS):
				filename = g_strdup_printf("%s%s.png",gui_prefs.SnapshotDirectory,rom->romname);
				break;
			case(FLYERS):
				filename = g_strdup_printf("%s%s.png",gui_prefs.FlyerDirectroy,rom->romname);
				break;
			case(CABINETS):
				filename = g_strdup_printf("%s%s.png",gui_prefs.CabinetDirectory,rom->romname);
				break;
			case(MARQUEES):
				filename = g_strdup_printf("%s%s.png",gui_prefs.MarqueeDirectory,rom->romname);
				break;
			case(TITLES):
				filename = g_strdup_printf("%s%s.png",gui_prefs.TitleDirectory,rom->romname);
				break;
			default:
				filename = g_strdup_printf("%s%s.png",gui_prefs.SnapshotDirectory,rom->romname);
		}
			
		pixbuf = gdk_pixbuf_new_from_file (filename);
		g_free(filename);
		/* no picture found try parent game if any*/
		if ((pixbuf == NULL) && strcmp(rom->cloneof,"-"))
		{
			switch (gui_prefs.ShowFlyer)
			{
				case(SNAPSHOTS):
					filename = g_strdup_printf("%s%s.png",gui_prefs.SnapshotDirectory,rom->cloneof);
					break;
				case(FLYERS):
					filename = g_strdup_printf("%s%s.png",gui_prefs.FlyerDirectroy,rom->cloneof);
					break;
				case(CABINETS):
					filename = g_strdup_printf("%s%s.png",gui_prefs.CabinetDirectory,rom->cloneof);
					break;
				case(MARQUEES):
					filename = g_strdup_printf("%s%s.png",gui_prefs.MarqueeDirectory,rom->cloneof);
					break;
				case(TITLES):
					filename = g_strdup_printf("%s%s.png",gui_prefs.TitleDirectory,rom->cloneof);
					break;
				default:
					filename = g_strdup_printf("%s%s.png",gui_prefs.SnapshotDirectory,rom->cloneof);
			}
			pixbuf = gdk_pixbuf_new_from_file (filename);
			g_free(filename);
		}
		/* we havent found the picture in the directory, maybe we could try in a zipfile */
		if (pixbuf == NULL)
		{
			ZIP *zip;
			struct zipent* zipent;
			gchar *zipfile;
			char *tmp_buffer;
			switch (gui_prefs.ShowFlyer)
			{
				case(SNAPSHOTS):
					zipfile = g_strdup_printf("%ssnap.zip",gui_prefs.SnapshotDirectory);
					break;
				case(FLYERS):
					zipfile = g_strdup_printf("%sflyers.zip",gui_prefs.FlyerDirectroy);
					break;
				case(CABINETS):
					zipfile = g_strdup_printf("%scabinets.zip",gui_prefs.CabinetDirectory);
					break;
				case(MARQUEES):
					zipfile = g_strdup_printf("%smarquees.zip",gui_prefs.MarqueeDirectory);
					break;
				case(TITLES):
					zipfile = g_strdup_printf("%stitles.zip",gui_prefs.TitleDirectory);
					break;
				default:
					zipfile = g_strdup_printf("%ssnap.zip",gui_prefs.SnapshotDirectory);
			}
			if(!access(zipfile,R_OK))
	        	{
				zip = openzip(zipfile);
				if ((zip = openzip(zipfile)) != 0)
				{
					GdkPixbufLoader *loader;
					gxmame_debug("Succesfully open zip file '%s' !", zipfile);
					filename = g_strdup_printf("%s.",rom->romname);
					while ((zipent = readzip(zip)) != 0)
					{
						/* this should allows to find any format of picture in the zip, not only bmp */
						if (!strncmp(filename,zipent->name,strlen(rom->romname)+1))
						{
							gxmame_debug("found file name %s\twith CRC:%i\tsize%i",
								zipent->name,
								zipent->crc32,
								zipent->uncompressed_size);
							if ((tmp_buffer = g_malloc(zipent->uncompressed_size)) == 0)
								gxmame_debug("%s: out of memory!",zipent->name);
							else
							{	/* if the file successfully uncompress, try to load it in a pixbuf loader */
								if (readuncompresszip(zip, zipent, (char *)tmp_buffer) == 0)
								{
									loader = gdk_pixbuf_loader_new ();
									if (!gdk_pixbuf_loader_write (loader, tmp_buffer, zipent->uncompressed_size)) {
										gxmame_debug("Error while uncompressing %s from %s",zipent->name,zipfile);
									}
									else
									{
										gdk_pixbuf_loader_close (loader);
										pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
									}
								}
								g_free(tmp_buffer);
								tmp_buffer = 0;
							}
						/* prevent to read all zip file if we have found the picture's game (uncompressed successfuly or not) */
						break;
						}
					}
					g_free(filename);
					/* no picture found try parent game if any*/
					if ((pixbuf == NULL) && strcmp(rom->cloneof,"-"))
					{
						rewindzip(zip);
						gxmame_debug("picture not found, trying clone: %s",rom->cloneof);
						filename = g_strdup_printf("%s.",rom->cloneof);
						while ((zipent = readzip(zip)) != 0)
						{
							/* this should allows to find any format of picture in the zip, not only bmp */
							if (!strncmp(filename,zipent->name,strlen(rom->cloneof)+1))
							{
								gxmame_debug("found file name %s\twith CRC:%i\tsize%i",
									zipent->name,
									zipent->crc32,
									zipent->uncompressed_size);
								if ((tmp_buffer = g_malloc(zipent->uncompressed_size)) == 0)
									gxmame_debug("%s: out of memory!",zipent->name);
								else
								{	/* if the file successfully uncompress, try to load it in a pixbuf loader */
									if (readuncompresszip(zip, zipent, (char *)tmp_buffer) == 0)
									{
										loader = gdk_pixbuf_loader_new ();
										if (!gdk_pixbuf_loader_write (loader, tmp_buffer, zipent->uncompressed_size)) {
											gxmame_debug("Error while uncompressing %s from %s",zipent->name,zipfile);
										}
										else
										{
											gdk_pixbuf_loader_close (loader);
											pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
										}
									}
									g_free(tmp_buffer);
									tmp_buffer = 0;
								}
							/* prevent to read all zip file if we have found the picture's game (uncompressed successfuly or not) */
							break;
							}
						}
						g_free(filename);
					}
					closezip(zip);
				} /* if ((zip = openzip(zipfile)) != 0) */
				else
				{
					gxmame_debug("Error, cannot open zip file '%s' !\n", zipfile);
				}
			} /* if(!access(zipfile,R_OK)) */
		g_free(zipfile);
			
		} /* if (pixbuf == NULL) */
		
		/* We got a picture */
		if (pixbuf != NULL)
		{
			int width,height,wwidth,wheight;
			width = gdk_pixbuf_get_width(pixbuf);
			height = gdk_pixbuf_get_height(pixbuf);
			if (gui_prefs.ShowScreenShot == 1)
			{
				gdk_window_get_size ((main_gui.screenshoot_event_box)->window,&wwidth,&wheight); 
				/* the picture is wider than the window, resize it to the window size */
				if (width>wwidth)
				{
					height = wwidth*((gdouble)height/(gdouble)width);
					width = wwidth;
				}
			}
			scaled_pixbuf = gdk_pixbuf_scale_simple(pixbuf,
								width,
                                     				height,
                                     				GDK_INTERP_BILINEAR);
			gdk_pixbuf_unref (pixbuf);
			gdk_pixbuf_render_pixmap_and_mask(scaled_pixbuf,
						&testpix,
						&testmask,
						127);
	
			gdk_pixbuf_unref (scaled_pixbuf);
	
		}
		else
		{
			gxmame_debug("no picture, fall back to the default one");
			testpix = gdk_pixmap_create_from_xpm_d(MainWindow->window,
						&testmask,
						&transparent,
						gxmame_xpm);
		}
		gtk_container_remove(GTK_CONTAINER(main_gui.screenshoot_event_box),main_gui.main_screenshot);
		gtk_container_remove(GTK_CONTAINER(main_gui.screenshot_hist_vbox),main_gui.screenshoot_event_box);
		
		main_gui.main_screenshot = gtk_pixmap_new (testpix,testmask);

		main_gui.screenshoot_event_box = gtk_event_box_new();

		/* erase, fill and show the history box */
		/* should freeze the history_box here rather than each function otherwise, the position of the cursor will
		    appear between mameinfo and history */
		gtk_text_freeze(GTK_TEXT(main_gui.history_box));
		gtk_text_set_point(GTK_TEXT(main_gui.history_box), gtk_text_get_length(GTK_TEXT(main_gui.history_box)));
		gtk_text_backward_delete(GTK_TEXT(main_gui.history_box),
					 gtk_text_get_length(GTK_TEXT(main_gui.history_box)));
		had_history = set_game_history(rom);
		if (had_history)
			gtk_text_insert(GTK_TEXT(main_gui.history_box),NULL,NULL,NULL,"\n",-1);
		had_info = set_game_info(rom);
		gtk_text_thaw(GTK_TEXT(main_gui.history_box));
		if (had_history || had_info)
		{
			gtk_widget_show(GTK_WIDGET(main_gui.history_scrollwin));
			gtk_box_pack_end(main_gui.screenshot_hist_vbox,main_gui.screenshoot_event_box,FALSE,TRUE,5);
		}
		else
		{
			gtk_widget_hide(GTK_WIDGET(main_gui.history_scrollwin));
			gtk_box_pack_end(main_gui.screenshot_hist_vbox,main_gui.screenshoot_event_box,TRUE,TRUE,5);
		}
		gtk_container_add(GTK_CONTAINER(main_gui.screenshoot_event_box),GTK_WIDGET(main_gui.main_screenshot));
		gtk_widget_show (main_gui.screenshoot_event_box);
		gtk_widget_show (main_gui.main_screenshot);
		gtk_signal_connect (GTK_OBJECT (main_gui.screenshoot_event_box), "button-release-event",
				GTK_SIGNAL_FUNC (change_screenshot),
        		   	NULL);
	}
	else /* no roms selected display the default picture */
	{
		gtk_container_remove(GTK_CONTAINER(main_gui.screenshoot_event_box),main_gui.main_screenshot);
		gtk_container_remove(GTK_CONTAINER(main_gui.screenshot_hist_vbox),main_gui.screenshoot_event_box);
			
		testpix = gdk_pixmap_create_from_xpm_d(MainWindow->window,
						&testmask,
						&transparent,
						gxmame_xpm);
		main_gui.main_screenshot = gtk_pixmap_new (testpix,testmask);

		main_gui.screenshoot_event_box = gtk_event_box_new();
		gtk_box_pack_end(main_gui.screenshot_hist_vbox,main_gui.screenshoot_event_box,TRUE,TRUE,5);
		gtk_container_add(GTK_CONTAINER(main_gui.screenshoot_event_box),GTK_WIDGET(main_gui.main_screenshot));
		gtk_widget_show (main_gui.screenshoot_event_box);
		gtk_widget_show (main_gui.main_screenshot);

		/* erase and hide the history box */
		gtk_text_freeze(GTK_TEXT(main_gui.history_box));
		gtk_text_set_point(GTK_TEXT(main_gui.history_box), gtk_text_get_length(GTK_TEXT(main_gui.history_box)));
		gtk_text_backward_delete(GTK_TEXT(main_gui.history_box),
				 	gtk_text_get_length(GTK_TEXT(main_gui.history_box)));
		gtk_text_thaw(GTK_TEXT(main_gui.history_box));
		gtk_widget_hide(GTK_WIDGET(main_gui.history_scrollwin));
	}
}


void precheck_for_record       (GtkButton       *button,
                                gpointer         inp_selection)
{
	gchar *error_msg;
	gchar *inp_file;
	inp_file = gtk_file_selection_get_filename (GTK_FILE_SELECTION(inp_selection));
	/* test if the inp file exist */
	gxmame_debug("check play file selected: {%s}",inp_file);
	if (!access(inp_file,F_OK))
	{	/* if yes print a message and return to the selection screen */
		#include "../pixmaps-builtin/warning.xpm"
		GtkWidget *Loading_error;
		GtkWidget *packer1;
		GtkWidget *yes_no_box;
		GtkWidget *button_yes;
		GtkWidget *button_no;
		GtkWidget *pixmap1;
		GtkWidget *label1;
		GdkPixmap *gdk_pixmap1;
		GdkBitmap *mask1;
		GdkColor  transparent;
	
		
		Loading_error = gtk_window_new (GTK_WINDOW_DIALOG);
		gtk_widget_realize(Loading_error);
		gtk_window_set_title (GTK_WINDOW (Loading_error), _("GXMame Warning"));
		gtk_window_set_position (GTK_WINDOW (Loading_error), GTK_WIN_POS_CENTER);
		gtk_window_set_modal (GTK_WINDOW (Loading_error), TRUE);
		gtk_window_set_policy (GTK_WINDOW (Loading_error), FALSE, FALSE, FALSE);
		gtk_window_set_transient_for(GTK_WINDOW(Loading_error),GTK_WINDOW(inp_selection));
		gtk_window_set_modal(GTK_WINDOW(Loading_error),TRUE);
	
		packer1 = gtk_packer_new ();
		gtk_packer_set_default_border_width (GTK_PACKER (packer1), 5);
		gtk_widget_ref (packer1);
		gtk_object_set_data_full (GTK_OBJECT (Loading_error), "packer1", packer1,
					  (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (packer1);
		gtk_container_add (GTK_CONTAINER (Loading_error), packer1);
		gtk_container_set_border_width (GTK_CONTAINER (packer1), 10);
		
		yes_no_box = gtk_hbox_new (FALSE, 2);
		gtk_widget_ref (yes_no_box);
		gtk_object_set_data_full (GTK_OBJECT (MainWindow), "yes_no_box", yes_no_box,
					  (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (yes_no_box);
		
		gtk_packer_add_defaults (GTK_PACKER (packer1), yes_no_box, GTK_SIDE_BOTTOM,
					 GTK_ANCHOR_CENTER, (GtkPackerOptions) (0));
		


		button_yes = gtk_button_new_with_label (_("Yes"));
		gtk_widget_ref (button_yes);
		gtk_object_set_data_full (GTK_OBJECT (Loading_error), "button_yes", button_yes,
					  (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (button_yes);
		gtk_widget_set_usize(button_yes,50,-2);
		gtk_box_pack_start (GTK_BOX (yes_no_box), button_yes, FALSE, FALSE, 2);
		button_no = gtk_button_new_with_label (_("No"));
		gtk_widget_ref (button_no);
		gtk_object_set_data_full (GTK_OBJECT (Loading_error), "button_no", button_no,
					  (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (button_no);
		gtk_widget_set_usize(button_no,50,-2);
		gtk_box_pack_start (GTK_BOX (yes_no_box), button_no, FALSE, FALSE, 2);
		
	
		gdk_pixmap1 = gdk_pixmap_create_from_xpm_d(
	 				Loading_error->window,
	 				&mask1,
	 				&transparent,
	 				warning_xpm);
		pixmap1 = gtk_pixmap_new (gdk_pixmap1,mask1);
		gtk_widget_ref (pixmap1);
		gdk_pixmap_unref(gdk_pixmap1);
		gdk_pixmap_unref(mask1);
		gtk_object_set_data_full (GTK_OBJECT (Loading_error), "pixmap1", pixmap1,
					  (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (pixmap1);
		gtk_packer_add_defaults (GTK_PACKER (packer1), pixmap1, GTK_SIDE_LEFT,
					 GTK_ANCHOR_NORTH, (GtkPackerOptions) (0));

	
		error_msg = g_strdup_printf(_("A file named '%s' already exist\nDo you want to overwrite it ?"),inp_file);
		label1 = gtk_label_new (error_msg);
		g_free(error_msg);
		gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);
		gtk_widget_ref (label1);
		gtk_object_set_data_full (GTK_OBJECT (Loading_error), "label1", label1,
					  (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (label1);
		gtk_packer_add_defaults (GTK_PACKER (packer1), label1, GTK_SIDE_TOP,
					 GTK_ANCHOR_CENTER, (GtkPackerOptions) (GTK_PACK_EXPAND | GTK_FILL_X | GTK_FILL_Y));

		/* Need to hide it otherwise, the box stay until game is finished */
		gtk_signal_connect_object (GTK_OBJECT (button_yes), "clicked",
				    GTK_SIGNAL_FUNC (gtk_widget_hide),
				    GTK_OBJECT (Loading_error));
		gtk_signal_connect (GTK_OBJECT (button_yes), "clicked",
				    GTK_SIGNAL_FUNC (record_game),
				    (gpointer) inp_selection);
		gtk_signal_connect_object_after (GTK_OBJECT (button_yes), "clicked",
				    GTK_SIGNAL_FUNC (gtk_widget_destroy),
				    GTK_OBJECT (Loading_error));
		gtk_signal_connect_object (GTK_OBJECT (button_no), "clicked",
				    GTK_SIGNAL_FUNC (gtk_widget_destroy),
				    GTK_OBJECT (Loading_error));
	
		gtk_widget_grab_focus (button_yes);
	
		gtk_widget_show(Loading_error);
		gdk_window_raise(Loading_error->window);
		set_icon(Loading_error,_("Error"));
	}
	else
	{
		record_game(NULL,inp_selection);
	}
}


GtkWidget*
select_inp (RomEntry *Rom,gboolean play_record)
{
	GtkWidget *inp_selection;
	GtkWidget *ok_button_inp_file;
	GtkWidget *cancel_button_inp_file;
	gchar *temp_text;

	if(play_record)
		inp_selection = gtk_file_selection_new (_("Choose inp file to play"));
	else
		inp_selection = gtk_file_selection_new (_("Choose inp file to record"));
	gtk_object_set_data (GTK_OBJECT (inp_selection), "inp_selection", inp_selection);
	gtk_container_set_border_width (GTK_CONTAINER (inp_selection), 10);
	GTK_WINDOW (inp_selection)->type = GTK_WINDOW_DIALOG;
	gtk_window_set_position (GTK_WINDOW (inp_selection), GTK_WIN_POS_MOUSE);
	gtk_window_set_modal (GTK_WINDOW (inp_selection), TRUE);
	gtk_window_set_policy (GTK_WINDOW (inp_selection), FALSE, FALSE, FALSE);
	gtk_window_set_transient_for(GTK_WINDOW(inp_selection),GTK_WINDOW(MainWindow));
	
	temp_text = g_strdup_printf("%s/%s.inp",gui_prefs.InputDirectory, Rom->romname);
	gtk_file_selection_set_filename(GTK_FILE_SELECTION(inp_selection),temp_text);
	g_free(temp_text);
	ok_button_inp_file = GTK_FILE_SELECTION (inp_selection)->ok_button;
	gtk_object_set_data (GTK_OBJECT (inp_selection), "ok_button_inp_file", ok_button_inp_file);
	gtk_widget_show (ok_button_inp_file);
	GTK_WIDGET_SET_FLAGS (ok_button_inp_file, GTK_CAN_DEFAULT);

	cancel_button_inp_file = GTK_FILE_SELECTION (inp_selection)->cancel_button;
	gtk_object_set_data (GTK_OBJECT (inp_selection), "cancel_button_inp_file", cancel_button_inp_file);
	gtk_widget_show (cancel_button_inp_file);
	GTK_WIDGET_SET_FLAGS (cancel_button_inp_file, GTK_CAN_DEFAULT);

	if(play_record)
		gtk_signal_connect (GTK_OBJECT (ok_button_inp_file), "clicked",
				    GTK_SIGNAL_FUNC (playback_game),
				    (gpointer) inp_selection);
	else
		gtk_signal_connect (GTK_OBJECT (ok_button_inp_file), "clicked",
				    GTK_SIGNAL_FUNC (precheck_for_record),
				    (gpointer) inp_selection);
	/* reenable joystick, was disabled in callback.c (on_playback_input_activate/on_play_and_record_input_activate)*/
	gtk_signal_connect (GTK_OBJECT (cancel_button_inp_file), "clicked",
			    GTK_SIGNAL_FUNC (joy_focus_on),
			    NULL);
	gtk_signal_connect_object_after (GTK_OBJECT (cancel_button_inp_file), "clicked",
			    GTK_SIGNAL_FUNC (gtk_widget_destroy),
			    (gpointer) inp_selection);
	
	gtk_widget_show(inp_selection);
	set_icon(inp_selection,_("File Selection"));
	return inp_selection;
}
