/*
 * GXMame
 *
 * 2002, 2003, Stphane Pontier <shadow_walker@users.sourceforge.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include <X11/Xlib.h>
#include <gdk/gdkx.h>
#include <sys/stat.h>

#include "gxmame.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "gui.h"
#include "io.h"
#include "mameio.h"
#include "options.h"
#include "progression_window.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "gtkjoy.h"
#include <locale.h>
#include <unistd.h>

#define BUFFER_SIZE 1000

void gxmame_init(void);

/*from the gui.c*/
extern void select_game(RomEntry *rom);

int main (int argc, char *argv[])
{
#ifdef ENABLE_NLS
	bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
	textdomain (PACKAGE);
#endif

	gtk_set_locale ();
	setlocale(LC_NUMERIC, "C");
	gtk_init (&argc, &argv);

	add_pixmap_directory ("/usr/share/pixmaps");
	add_pixmap_directory ("/usr/local/share/pixmaps");

	gxmame_init();

	init_gui();
	
	/* only need to do a quick check and redisplay games if the not_checked_list is not empty */
	if(not_checked_list)
	{
		quick_check();
		create_gamelist_content();
	}
	/* got a segfault here if I'm trying to flush gtk event
	   and GUIPosX and GUIPosY are not Null */
	/*if(!(gui_prefs.GUIPosX && gui_prefs.GUIPosY))
		while(gtk_events_pending())
			gtk_main_iteration();*/
	/*FIXME this is just to display the game position properly
	  when I got the strange 0.0 position bug*/

	gxmame_debug("init done, starting main loop");
	gtk_main ();
  return 0;
}

void gxmame_debug(const gchar * format, ...)
{
#ifdef DEBUG
	va_list args;
	va_start(args, format);
	g_logv(NULL, G_LOG_LEVEL_DEBUG, format, args);
	va_end(args);
#endif
}


void gxmame_init()
{
	GTimer *mytimer;
	gchar *filename, *dirname;
	struct stat filestatbuf;
/* should load pref here*/
	filename = g_malloc(50);
	dirname = g_malloc(50);
	
	mytimer=g_timer_new();
	g_timer_start(mytimer);
	g_snprintf(filename, 50, "%s/.gxmame",getenv("HOME"));
	if (stat(filename,&filestatbuf)<0) 
	{
		gxmame_debug ("no initial directory creating one");
		g_snprintf(dirname,50, "mkdir %s",filename);
		system(dirname);
		
	}
	g_snprintf(filename, 50, "%s/.gxmame/options",getenv("HOME"));
	if (stat(filename,&filestatbuf)<0) 
	{
		gxmame_debug ("no options directory creating one");
		g_snprintf(dirname,50, "mkdir %s",filename);
		system(dirname);
		
	}
	g_free(filename);
	g_free(dirname);
	
	/* Load master config file*/
	if (!load_gxmamerc())
		g_message(_("gxmamerc not loaded, using default values"));
	
	executable_version.name=g_strdup("none");
	executable_version.version=g_strdup("none");
	executable_version.target=g_strdup("none");
	/* this function should always be followed by get_mame_version(mame_executable,EXECUTABLE)
	   since it use executable_version.* variable and we want these variable according to the mame_executable */
	check_executable_list();
	if (get_mame_version(mame_executable,EXECUTABLE))
		get_mame_options();
	
	if (!load_dirs_ini())
		g_message(_("dirs.ini not loaded, using default values"));
	if (!load_gxmame_ini())
		g_message(_("unable to load gxmame.ini, using default values"));

	/* to be sure to have these string initialized */
	list_version.name=g_strdup("none");
	list_version.version=g_strdup("none");
	list_version.target=g_strdup("none");
	if (!load_games_list())
	{
		g_message(_("gamelist not found, need to rebuild one"));
	}
	else
	{
		if (!load_games_ini())
			g_message(_("games.ini not loaded, using default values"));
		if (!load_catver_ini())
			g_message(_("catver not loaded, using default values"));
	}

	if (!load_options(NULL))
		g_message(_("default options not loaded, using default values"));
	
	g_timer_stop(mytimer);
	g_message(_("time elapsed:%e"),g_timer_elapsed(mytimer,NULL));
	srand(g_timer_elapsed(mytimer,NULL));
	g_timer_destroy(mytimer);
	
#ifndef NO_JOYSTICK
	if (gui_prefs.gui_joy)
	{
		if ((joydata=joystick_new(gui_prefs.Joystick_in_GUI, joystick_events, NULL)))
		{
			g_message(_("Joystick %s found"),joydata->device_name);
		} else
		{
			g_message(_("No Joystick found"));
		}
	}
#endif	
	/* doesn't matter if joystick is enabled or not but easier to handle after */
	joy_focus_on();
	
	/*TEMP creation of a fixed columns list*/
	Columns = NULL;
	Columns = g_list_append(Columns, GINT_TO_POINTER(GAMENAME));
/*	Columns = g_list_append(Columns, GINT_TO_POINTER(HAS_ROMS));*/
	Columns = g_list_append(Columns, GINT_TO_POINTER(HAS_SAMPLES));
	Columns = g_list_append(Columns, GINT_TO_POINTER(ROMNAME));
	Columns = g_list_append(Columns, GINT_TO_POINTER(VECTOR));
	Columns = g_list_append(Columns, GINT_TO_POINTER(CONTROL));
	Columns = g_list_append(Columns, GINT_TO_POINTER(TIMESPLAYED));
	Columns = g_list_append(Columns, GINT_TO_POINTER(MANU));
	Columns = g_list_append(Columns, GINT_TO_POINTER(YEAR));
	Columns = g_list_append(Columns, GINT_TO_POINTER(CLONE));
	Columns = g_list_append(Columns, GINT_TO_POINTER(DRIVER));
/*	Columns = g_list_append(Columns, GINT_TO_POINTER(ROMOF));*/
	Columns = g_list_append(Columns, GINT_TO_POINTER(MAMEVER));
	Columns = g_list_append(Columns, GINT_TO_POINTER(CATEGORY));
	
	
}


gboolean game_filtered (RomEntry * rom)
{
	gchar *manufacturer1 = NULL,*manufacturer2 = NULL ,**manufacturer_fields = NULL;
/* comparing taxt and text */
	if (current_filter->type == DRIVER)
	{
		if ((current_filter->is && !g_strcasecmp(rom->driver,(gchar *)current_filter->value)) ||
		    (!current_filter->is && g_strcasecmp(rom->driver,(gchar *)current_filter->value))
		    ) {
			return TRUE;
		}
	} else if (current_filter->type == CLONE)
	{
		if ((current_filter->is && !g_strcasecmp(rom->cloneof,(gchar *)current_filter->value)) ||
		    (!current_filter->is && g_strcasecmp(rom->cloneof,(gchar *)current_filter->value))
		    ) {
			return TRUE;
		}
	} else if (current_filter->type == CONTROL)
	{
		if ((current_filter->is && !g_strcasecmp(rom->control,(gchar *)current_filter->value)) ||
		    (!current_filter->is && g_strcasecmp(rom->control,(gchar *)current_filter->value))
		    ) {
			return TRUE;
		}
	} else if (current_filter->type == MAMEVER)
	{
		if ((current_filter->is && !g_strcasecmp(rom->mame_ver_added,(gchar *)current_filter->value)) ||
		    (!current_filter->is && g_strcasecmp(rom->mame_ver_added,(gchar *)current_filter->value))
		    ) {
			return TRUE;
		}
	} else if (current_filter->type == CATEGORY)
	{
		if ((current_filter->is && !g_strcasecmp(rom->category,(gchar *)current_filter->value)) ||
		    (!current_filter->is && g_strcasecmp(rom->category,(gchar *)current_filter->value))
		    ) {
			return TRUE;
		}
/* comparing boolean and boolean */
	} else if (current_filter->type == FAVORITE)
	{
		if ((current_filter->is && rom->favourite)||
		    (!current_filter->is && !rom->favourite)
		    ) {
			return TRUE;
		}
	} else if (current_filter->type == VECTOR)
	{
		if ((current_filter->is && rom->vector)||
		    (!current_filter->is && !rom->vector)
		    ) {
			return TRUE;
		}
	} else if (current_filter->type == STATUS)
	{
		if ((current_filter->is && (rom->status))||
		    (!current_filter->is && !(rom->status))
		    ) {
			return TRUE;
		}
/* Comparing int and int */
	} else if (current_filter->type == HAS_ROMS)
	{
		if ((current_filter->is && (rom->has_roms == GPOINTER_TO_INT(current_filter->value)))  ||
		    (!current_filter->is && !(rom->has_roms == GPOINTER_TO_INT(current_filter->value)))
		    ) {
			return TRUE;
		}
	} else if (current_filter->type == TIMESPLAYED)
	{
		if ((current_filter->is && (rom->timesplayed == GPOINTER_TO_INT(current_filter->value))) ||
		    (!current_filter->is && !(rom->timesplayed == GPOINTER_TO_INT(current_filter->value)))
		    ) {
			return TRUE;
		}
	} else if (current_filter->type == CHANNELS)
	{
		if ((current_filter->is && (rom->channels == GPOINTER_TO_INT(current_filter->value))) ||
		    (!current_filter->is && !(rom->channels == GPOINTER_TO_INT(current_filter->value)))
		    ) {
			return TRUE;
		}
/* Comparing text and int */
	} else if (current_filter->type == YEAR)
	{
		if ((current_filter->is && (atoi(rom->year) == GPOINTER_TO_INT(current_filter->value))) ||
		    (!current_filter->is && !(atoi(rom->year) == GPOINTER_TO_INT(current_filter->value)))
		    ) {
			return TRUE;
		}
/* comparing parsed text and text */
	} else if (current_filter->type == MANU)
	{		
		manufacturer_fields = get_manufacturers(rom);
		/* we have now one or two clean manufacturer(s) we still need to differentiates sub companies*/
		if (manufacturer_fields[1] != NULL)
		{
			manufacturer1 = g_strdup(manufacturer_fields[0]);
			manufacturer2 = g_strdup(manufacturer_fields[1]);
			if ((current_filter->is && !g_strncasecmp(manufacturer1,(gchar *)current_filter->value,5)) ||
		    	    (!current_filter->is && g_strncasecmp(manufacturer1,(gchar *)current_filter->value,5)) ||
			    (current_filter->is && !g_strncasecmp(manufacturer2,(gchar *)current_filter->value,5)) ||
		    	    (!current_filter->is && g_strncasecmp(manufacturer2,(gchar *)current_filter->value,5))
		    		)
			{
				g_strfreev(manufacturer_fields);
				g_free(manufacturer1);g_free(manufacturer2);
				return TRUE;
			}
			g_free(manufacturer1);g_free(manufacturer2);
		}
		else
		{
			if ((current_filter->is && !g_strncasecmp(manufacturer_fields[0],(gchar *)current_filter->value,5)) ||
		    	    (!current_filter->is && g_strncasecmp(manufacturer_fields[0],(gchar *)current_filter->value,5))
		    		)
			{
				g_strfreev(manufacturer_fields);
				return TRUE;
			}
		}
		g_strfreev(manufacturer_fields);
	}
	return FALSE;
}

gchar **get_manufacturers (RomEntry * rom)
{
	gchar /**manufacturer1 = NULL,*manufacturer2 = NULL ,*/**manufacturer_fields = NULL;
	manufacturer_fields = g_strsplit(rom->manu,"]",0);
	if (manufacturer_fields[1] != NULL)
	{
		/* we have two winners [Company 1] (company 2)*/
		g_strstrip(g_strdelimit(manufacturer_fields[0],"[",' '));
		g_strstrip(g_strdelimit(manufacturer_fields[1],"()",' '));
	} else
	{
		g_strfreev(manufacturer_fields);
		manufacturer_fields = g_strsplit(rom->manu,"+",0);
		if (manufacturer_fields[1] != NULL)
		{
			/* we have two winners  Company1+Company2*/
			g_strstrip(g_strdelimit(manufacturer_fields[0],"[",' '));
			g_strstrip(g_strdelimit(manufacturer_fields[1],"()",' '));
		} else
		{
			g_strfreev(manufacturer_fields);
			manufacturer_fields = g_strsplit(rom->manu,"/",0);
			if (manufacturer_fields[1] != NULL)
			{
				/* we have two winners  Company1/Company2*/
				g_strstrip(g_strdelimit(manufacturer_fields[0],"[",' '));
				g_strstrip(g_strdelimit(manufacturer_fields[1],"()",' '));
			}
			else
			{
				g_strfreev(manufacturer_fields);
				manufacturer_fields = g_strsplit(rom->manu,"(",0);
				if (manufacturer_fields[1] != NULL)
				{
					/* we have two winners  Company1/Company2*/
					g_strstrip(g_strdelimit(manufacturer_fields[0],"[",' '));
					g_strstrip(g_strdelimit(manufacturer_fields[1],"()",' '));
				}
			}
		}
	}
	return manufacturer_fields;
}


/* launch following the commandline prepared by play_game, playback_game and record_game 
   then test if the game is launched, detect error and update game status */
void launch_emulation(RomEntry *rom,gchar *options)
{
	FILE *xmame_pipe=NULL;
	gchar line[BUFFER_SIZE];
	gchar *p,*p2;
	gfloat done;
	gint nb_loaded = 0;
	GList *extra_output = NULL, *extra_output2 = NULL;
	gboolean error_during_load,other_error;
	GtkWidget *progress_window;
	
#ifndef NO_JOYSTICK
	joystick_close(joydata);
	joydata=NULL;
#endif	
	
	
	progress_window = create_loading_progress_window(TRUE);
	g_snprintf(line,BUFFER_SIZE,_("Loading %s:"),gui_prefs.DefaultGame);
	gtk_window_set_title(GTK_WINDOW(progress_window),line);
	gtk_widget_show_all(progress_window);
	set_icon(progress_window,_("progression"));
	gtk_widget_hide(MainWindow);
	
	/* need to use printf otherwise, with gxmame_debug, we dont see the complete command line*/
	printf("Message: running command %s\n",options);
        xmame_pipe = popen(options, "r");
	g_message(_("Loading %s:"),rom->gamename);
	/* Loading */
	error_during_load = other_error = FALSE;
	while (fgets(line, BUFFER_SIZE, xmame_pipe))
	{
			/* remove the last \n */
			for(p=line;(*p && (*p != '\n'));p++);
			*p='\0';p++;
		g_message("xmame: %s",line);
		if (!strncmp(line,"loading",7))
		{
			nb_loaded++;
			/*search for the : */
			for(p=line;(*p && (*p != ':'));p++);
			p=p+2;
			for(p2=p;(*p2 && (*p2 != '\n'));p2++);
			p2='\0';
			done = (gfloat) ((gfloat) (nb_loaded) /
					(gfloat) (rom->nb_roms));
			update_loading_progress_window (done,p);
			while (gtk_events_pending()) gtk_main_iteration();
		}
		else if(!strncmp(line,"error",5)||
			!strncmp(line,"Error: Input file",17))	/* Input file is for a different game as specified */
		{	/* the game didn't even began to load*/
			other_error = TRUE;
			extra_output = g_list_append(extra_output,g_strdup(line));
		}
		if (!strncmp(line,"done",4))
			break;
	}
	gtk_widget_destroy(progress_window);
	while (gtk_events_pending()) gtk_main_iteration();
	/*check if errors */
	while (fgets(line, BUFFER_SIZE, xmame_pipe))
	{
		for(p=line;(*p && (*p != '\n'));p++);
		*p='\0';p++;
		g_message("xmame: %s",line);
		if (!strncmp(line,"ERROR",5))		/* bad rom or no rom found */
		{
			error_during_load = TRUE;
			extra_output = g_list_append(extra_output,g_strdup(line));
			break;
		}					/* another error occurred after game loaded */
		else if (!strncmp(line,"X Error",7) ||		/* X11 mode not found*/
			 !strncmp(line,"SDL: Unsupported",16) ||
			 !strncmp(line,"Unable to start",15) ||
			 !strncmp(line,"Unspected X Error",17))
		{
			other_error = TRUE;
			extra_output = g_list_append(extra_output,g_strdup(line));
		} else
		{
			extra_output = g_list_append(extra_output,g_strdup(line));
		}	
	}
	
	pclose(xmame_pipe);
	if (error_during_load || other_error)
	{
		int size;
		char **message = NULL;
		gxmame_debug("error during load");
		size = g_list_length(extra_output)+1;
		message = g_new(gchar *,size);
		for(g_list_first(extra_output),size=0;extra_output != NULL;)
		{
			message[size++] = extra_output->data;
			extra_output = g_list_next(extra_output);
		}
		message[size++] = NULL;
		GXMame_error_message(ERROR, g_strjoinv("\n",message));
		g_strfreev(message);
		
		/* free the g_list but it seems that since extra_output point to NULL. the list is not here anymore
		  and since we freed the array, textes are not here anymore but the next and previous pointer?*/
		g_list_foreach(extra_output,(GFunc)g_free,NULL);
		g_list_free (extra_output);
		
		/* update game informations if it was an rom problem */
		if (error_during_load)
			rom->has_roms=0;
		
	}
	else
	{
		gxmame_debug("game over");
		for(g_list_first(extra_output);extra_output != NULL;)
		{
			extra_output2 = extra_output;
			extra_output = g_list_next(extra_output);
			g_free(extra_output2->data);
			g_list_remove(extra_output2,extra_output2->data);
		}
		
		/* update game informations */
		rom->timesplayed++;
		rom->has_roms=1;
	}
	gtk_widget_show(MainWindow);
        g_free(options);
	/* update the gui for the times played and romstatus if there was any error */
	update_game_in_list(rom->position);
	select_game(rom);
#ifndef NO_JOYSTICK
	if(gui_prefs.gui_joy)
		joydata=joystick_new(gui_prefs.Joystick_in_GUI, joystick_events, NULL);
#endif	
}

/* Prepare the commandline to use to play a game */
void play_game(RomEntry *rom)
{
	gchar	*opt;
	gchar	*rompath,*samplepath,*iomisc;
	gchar	*Video_Related_options,
		*Sound_Related_options,
		*Renderer_Related_options,
		*Input_Related_options,
		*Misc_Related_options,
		*Vector_Related_options;
	
	if (!strcmp(executable_version.name,"Not executable"))
	{
		opt = g_strdup_printf(_("%s is not a valid executable"),mame_executable);
		GXMame_error_message(ERROR,opt);
		g_free(opt);
		return;
	}
	
	/* prepares options*/
	rompath = create_RomPath_Related_options_string();
	samplepath = create_SamplePath_Related_options_string();
	iomisc = create_io_Related_options_string();
	Video_Related_options = create_Video_Related_options_string();
	Sound_Related_options = create_Sound_Related_options_string();
	Renderer_Related_options = create_Renderer_Related_options_string();
	Input_Related_options = create_Input_Related_options_string();
	Misc_Related_options = create_Misc_Related_options_string();
	if (rom->vector)
		Vector_Related_options = create_Vector_Related_options_string();
	else
		Vector_Related_options = g_strdup("");				
	/* create the command */
        opt=g_strdup_printf("%s %s %s %s %s %s %s %s %s %s %s -noloadconfig 2>&1",
			real_mame_executable,
			rompath,
			samplepath,
			iomisc,
			Video_Related_options,
			Sound_Related_options,
			Renderer_Related_options,
			Input_Related_options,
			Misc_Related_options,
			Vector_Related_options,
			rom->romname);
	/*free options*/
	g_free(rompath);
	g_free(samplepath);
	g_free(iomisc);
	g_free(Video_Related_options);
	g_free(Sound_Related_options);
	g_free(Renderer_Related_options);
	g_free(Input_Related_options);
	g_free(Misc_Related_options);
	g_free(Vector_Related_options);
	
	launch_emulation(rom,opt);
}


/* Prepare the commandline to use to playback a game */
void
playback_game        (GtkButton       *button,
                      gpointer         user_data)
{
	gchar *error_msg;
	gchar *inp_file, *brut_inp_file;
	brut_inp_file = gtk_file_selection_get_filename (GTK_FILE_SELECTION(user_data));
	inp_file = escape_space(brut_inp_file);
	/* test if the inp file is readable */
	gxmame_debug("play selected: {%s}",inp_file);
	/* nedd to do a test on the unescaped string here otherwise doesn't even find the file  */
	if (access(brut_inp_file,R_OK))
	{	/* if not print a message and return to the selection screen */
		error_msg = g_strdup_printf(_("Could not open '%s' as valid input file"),brut_inp_file);
		GXMame_error_message(ERROR,error_msg);
		g_free(error_msg);
	}
	else
	{
		gchar	*opt;
		gchar	*rompath,*samplepath,*iomisc;
		gchar	*Video_Related_options,
			*Sound_Related_options,
			*Renderer_Related_options,
			*Input_Related_options,
			*Misc_Related_options,
			*Vector_Related_options;
		if (!strcmp(executable_version.name,"Not executable"))
		{
			opt = g_strdup_printf(_("%s is not a valid executable"),mame_executable);
			GXMame_error_message(ERROR,opt);
			g_free(opt);
			gtk_widget_destroy(user_data);
			/* reenable joystick, was disabled in callback.c (on_playback_input_activate/on_play_and_record_input_activate)*/
			joy_focus_on();
			return;
		}
		gxmame_debug("Playback game %s",inp_file);
		/* prepares options*/
		rompath = create_RomPath_Related_options_string();
		samplepath = create_SamplePath_Related_options_string();
		iomisc = create_io_Related_options_string();
		Video_Related_options = create_Video_Related_options_string();
		Sound_Related_options = create_Sound_Related_options_string();
		Renderer_Related_options = create_Renderer_Related_options_string();
		Input_Related_options = create_Input_Related_options_string();
		Misc_Related_options = create_Misc_Related_options_string();
		if (gui_prefs.DefaultGame_p->vector)
			Vector_Related_options = create_Vector_Related_options_string();
		else
			Vector_Related_options = g_strdup("");
		/* create the command */
 		opt=g_strdup_printf("%s %s %s %s %s %s %s %s %s %s %s -playback %s -noloadconfig 2>&1",
				real_mame_executable,
				rompath,
				samplepath,
				iomisc,
				Video_Related_options,
				Sound_Related_options,
				Renderer_Related_options,
				Input_Related_options,
				Misc_Related_options,
				Vector_Related_options,
				gui_prefs.DefaultGame_p->romname,
				inp_file);
		/*free options*/
		g_free(rompath);
		g_free(samplepath);
		g_free(iomisc);
		g_free(Video_Related_options);
		g_free(Sound_Related_options);
		g_free(Renderer_Related_options);
		g_free(Input_Related_options);
		g_free(Misc_Related_options);
		g_free(Vector_Related_options);
	

		gtk_widget_destroy(user_data);
		launch_emulation(gui_prefs.DefaultGame_p,opt);
		
		/* reenable joystick, was disabled in callback.c (on_playback_input_activate/on_play_and_record_input_activate)*/
		joy_focus_on();
	}
	g_free(inp_file);
}


/* Prepare the commandline to use to record a game */
void
record_game        (GtkButton       *button,
                    gpointer         user_data)
{
	gchar *inp_file, *brut_inp_file;
	gchar	*opt;
	gchar	*rompath,*samplepath,*iomisc;
	gchar	*Video_Related_options,
		*Sound_Related_options,
		*Renderer_Related_options,
		*Input_Related_options,
		*Misc_Related_options,
		*Vector_Related_options;
	brut_inp_file = gtk_file_selection_get_filename (GTK_FILE_SELECTION(user_data));
	inp_file = escape_space(brut_inp_file);
	
 	if (!strcmp(executable_version.name,"Not executable"))
 	{
 		opt = g_strdup_printf(_("%s is not a valid executable"),mame_executable);
 		GXMame_error_message(ERROR,opt);
 		g_free(opt);
		gtk_widget_destroy(user_data);
		/* reenable joystick, was disabled in callback.c (on_playback_input_activate/on_play_and_record_input_activate)*/
		joy_focus_on();
 	}
	else
	{
	 	gxmame_debug("Record game %s",inp_file);
	 	/* prepares options*/
	 	rompath = create_RomPath_Related_options_string();
	 	samplepath = create_SamplePath_Related_options_string();
	 	iomisc = create_io_Related_options_string();
	 	Video_Related_options = create_Video_Related_options_string();
	 	Sound_Related_options = create_Sound_Related_options_string();
	 	Renderer_Related_options = create_Renderer_Related_options_string();
	 	Input_Related_options = create_Input_Related_options_string();
	 	Misc_Related_options = create_Misc_Related_options_string();
	 	if (gui_prefs.DefaultGame_p->vector)
	 		Vector_Related_options = create_Vector_Related_options_string();
	 	else
	 		Vector_Related_options = g_strdup("");
	 	/* create the command */
	 	opt=g_strdup_printf("%s %s %s %s %s %s %s %s %s %s %s -record %s -noloadconfig 2>&1",
	 			real_mame_executable,
	 			rompath,
	 			samplepath,
	 			iomisc,
	 			Video_Related_options,
	 			Sound_Related_options,
	 			Renderer_Related_options,
	 			Input_Related_options,
	 			Misc_Related_options,
	 			Vector_Related_options,
	 			gui_prefs.DefaultGame_p->romname,
	 			inp_file);
	 	/*free options*/
	 	g_free(rompath);
	 	g_free(samplepath);
	 	g_free(iomisc);
	 	g_free(Video_Related_options);
	 	g_free(Sound_Related_options);
	 	g_free(Renderer_Related_options);
	 	g_free(Input_Related_options);
	 	g_free(Misc_Related_options);
	 	g_free(Vector_Related_options);

		gtk_widget_destroy(user_data);
		launch_emulation(gui_prefs.DefaultGame_p,opt);
		/* reenable joystick, was disabled in callback.c (on_playback_input_activate/on_play_and_record_input_activate)*/
		joy_focus_on();
	}
	g_free(inp_file);
}


void exit_gxmame(void)
{
	g_message(_("Exiting GXMame..."));
	/* prevent to erase games preference if the game list was not loaded
	  (gamelist file corruption or rebuild of a list with a bogus executable)*/
	if (roms)
		save_games_ini();
	save_gxmame_ini();
	save_dirs_ini();
	save_gxmamerc();
	save_options(NULL);
#ifndef NO_JOYSTICK
	joystick_close(joydata);
	joydata=NULL;
#endif	
	gtk_main_quit();
	gtk_exit (0);
}


gint compare_game_name(RomEntry *Rom1,RomEntry *Rom2)
{
	gint result=0;
	if ((!Rom1->the_trailer && !Rom2->the_trailer) ||
	    (Rom1->the_trailer && Rom2->the_trailer)||
			gui_prefs.ModifyThe)
	{
		if (!(result=strcmp(Rom1->gamename,Rom2->gamename)))
			result=strcmp(Rom1->gamenameext,Rom2->gamenameext);
	}
	else if (Rom1->the_trailer && !Rom2->the_trailer)
		result=strncmp("The ",Rom2->gamename,4);
	else if (!Rom1->the_trailer && Rom2->the_trailer)
		result=strncmp(Rom1->gamename,"The ",4);
	else
		result=0;
	return(result);
}

/*test of a comparison function (indented list) to be removed*/
gint compare_game_name2(RomEntry *Rom1,RomEntry *Rom2)
{
	gint result=0;
	if (!strcmp(Rom1->cloneof,"-"))
	{
	/* none of the games are clones -> compare and sort original games */
		if (!strcmp(Rom2->cloneof,"-"))
		{
			/* both or no games have a trailing "the" */
			if ((!Rom1->the_trailer && !Rom2->the_trailer) ||
			    (Rom1->the_trailer && Rom2->the_trailer)||
					gui_prefs.ModifyThe)
			{
				if (!(result=strcmp(Rom1->gamename,Rom2->gamename)))
					result=strcmp(Rom1->gamenameext,Rom2->gamenameext);
			}
			/* game 1 has "the" but not game 2  */
			else if (Rom1->the_trailer && !Rom2->the_trailer)
				result=strncmp("The ",Rom2->gamename,4);
			/* game 2 has "the" but not game 1 */
			else if (!Rom1->the_trailer && Rom2->the_trailer)
				result=strncmp(Rom1->gamename,"The ",4);
			else
				result=0;
		}
	/* game 2 is a clone but not game 1	-> compare and sort clone game2 romname 1 with pref to 1 if same */
		else
		{
			if (!(result=strcmp(Rom1->romname,Rom2->cloneof)))
				result=-1;
		}
	}
	else
	{
	/* game 1 is a clone but not game 2	-> compare and sort clone game1 romname 2 with pref to 2 if same */
		if (!strcmp(Rom2->cloneof,"-"))
		{
			if (!(result=strcmp(Rom1->cloneof,Rom2->romname)))
				result=1;
		}
	/* both games are clones	-> compare and sort clones games */
		else
		{	
			if (!(result=strcmp(Rom1->cloneof,Rom2->cloneof)))
			{
				/* both or no games have a trailing "the" */
				if ((!Rom1->the_trailer && !Rom2->the_trailer) ||
				    (Rom1->the_trailer && Rom2->the_trailer)||
						gui_prefs.ModifyThe)
				{
					if (!(result=strcmp(Rom1->gamename,Rom2->gamename)))
						result=strcmp(Rom1->gamenameext,Rom2->gamenameext);
				}
				/* game 1 has "the" but not game 2  */
				else if (Rom1->the_trailer && !Rom2->the_trailer)
					result=strncmp("The ",Rom2->gamename,4);
				/* game 2 has "the" but not game 1 */
				else if (!Rom1->the_trailer && Rom2->the_trailer)
					result=strncmp(Rom1->gamename,"The ",4);
				else
					result=0;
			}
		}
	}
	return(result);
}

void destroy_simple_filter (GtkCTree *ctree,
                            GtkCTreeNode *node,
                            gpointer data)
{
	simple_filter *node_data;
	
	/* free a simple filter */
	/* note: value for manufacturer, (year?), categories, and version 
	   will be freed when I free glist 
	   (at the condition that GTK accept to give my memory back for glists)*/
	node_data = gtk_ctree_node_get_row_data (ctree,node);
	if((node_data->type)==CLONE ||		
	   (node_data->type)==CLONE ||		
	   (node_data->type)==DRIVER ||		
	   (node_data->type)==CONTROL)
	{
		g_free(node_data->value);		
	}
		
	g_free(node_data);
	gtk_ctree_remove_node(ctree,node);
}


/*void free_game_glist_element  (gpointer data,
                               gpointer user_data)
{
	RomEntry *tmprom;
	tmprom = (RomEntry *)data;
	g_free(tmprom);
}
*/
		
void lookup_game(gchar *search_string)
{
	gchar *string2;
	guint row,len;
	RomEntry *tmprom;
	gboolean found = FALSE;
	row = gui_prefs.DefaultGame_p->position;
	if (visible_games>1)
	{	
		tmprom = (RomEntry *)gtk_clist_get_row_data(GTK_CLIST(main_gui.displayed_list),row);
		
		len = strlen(search_string);
		/* put this value to 0 will give a strange behaviour when not ordered alphabetically */
		/* if (g_strncasecmp(search_string, tmprom->gamename, 1)<0)
			row = 0;*/
		/* try to find the a match bettwen current position and end of list */
		do {
			tmprom = (RomEntry *)gtk_clist_get_row_data(GTK_CLIST(main_gui.displayed_list),row);
			if (tmprom->the_trailer && !gui_prefs.ModifyThe)
			{
				if (len <=4)
				{
					if (!g_strncasecmp(search_string, "The ", len))
						found =TRUE;
				}
				else
				{
					string2 = search_string +4;
					if (!g_strncasecmp(search_string, "The ", 4) && 
					    !g_strncasecmp(string2, tmprom->gamename, len-4))
						found =TRUE;
				}
			}
			else
			{
				if (!g_strncasecmp(search_string, tmprom->gamename, len))
					found =TRUE;
			}
			row++;
		} while (!found && row < visible_games);
		/* haven't found anything, will try now between the beginning of the list and current position */
		if (!found)
		{
			row = 0;
			do {
				tmprom = (RomEntry *)gtk_clist_get_row_data(GTK_CLIST(main_gui.displayed_list),row);
				if (tmprom->the_trailer && !gui_prefs.ModifyThe)
				{
					if (len <=4)
					{
						if (!g_strncasecmp(search_string, "The ", len))
							found =TRUE;
					}
					else
					{
						string2 = search_string +4;
						if (!g_strncasecmp(search_string, "The ", 4) &&
						    !g_strncasecmp(string2, tmprom->gamename, len-4))
							found =TRUE;
					}
				}
				else
				{
					if (!g_strncasecmp(search_string, tmprom->gamename, len))
						found =TRUE;
				}
				row++;
			} while (!found && row < gui_prefs.DefaultGame_p->position);
		}
		
		if (found)
		{
			gxmame_debug("found: %s",tmprom->gamename);
			row--;
		}
		if (!found && (search_string[0] == search_string[1]) && (row < visible_games-1))
		{
			/* since we haven't found anything but user keep pressing the same button, we're going to the next row*/
			found = TRUE;
			row++;
			gxmame_debug("not found, go to the next row");
			/*test the first letter of the next game*/
			tmprom = (RomEntry *)gtk_clist_get_row_data(GTK_CLIST(main_gui.displayed_list),row);
			if ((!(g_strncasecmp(search_string, "The ", 1)) && tmprom->the_trailer && !gui_prefs.ModifyThe) ||
			    (!(g_strncasecmp(search_string, tmprom->gamename, 1)) && !tmprom->the_trailer && !gui_prefs.ModifyThe) ||	
			    (!(g_strncasecmp(search_string, tmprom->gamename, 1)) && gui_prefs.ModifyThe) )
			{
				/* prevent to have a search string too big*/
				search_string[1]='\0';
			} else
			{
				/* prevent to continue to go to the next line if user continue with the same letter */
				search_string[0]='\0';
			}
		}
		

		if (found)
		{
			gxmame_debug("found row:%i",row);
	        	gtk_clist_select_row(GTK_CLIST(main_gui.displayed_list),row,0);
			gtk_clist_moveto(GTK_CLIST(main_gui.displayed_list),row,0,0.5,0);
		}
	}
}


