/*
 * GXMame
 *
 * 2002, 2003, Stphane Pontier <shadow_walker@users.sourceforge.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "gxmame.h"
#include "io.h"
#include "gui.h"
#include "progression_window.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>

int compare_year(gint *Y1,gint *Y2)
{
	int temp;
	temp = (Y1>Y2)?1:0;
	return temp;
}

gboolean load_games_list()
{
	FILE *gamelist;
	gint romindex, i, count;
	GList *listpointer;
	
	gint year;

	gchar *filename;
	gchar line[2000];
	gchar **tmp_array = NULL;
	gchar *tmp, *p;
	RomEntry *rom;
	gboolean exe_version_checked = FALSE, manufacturer_in_list;
	gchar *manufacturer1,*manufacturer2,**manufacturer_fields = NULL;

	supported_samples_games = 0;
	romindex = 0;
	g_message(_("Loading gamelist"));

	filename = g_malloc(200);

	/* clean the existing list if any */
	g_list_foreach(roms, (GFunc) g_free, NULL);
	g_list_free(roms);
	roms = NULL;

	g_snprintf(filename, 200, "%s/.gxmame/gamelist",
		   getenv("HOME"));
	gamelist = fopen(filename, "r");
	if (gamelist == NULL) {
		g_free(filename);
		return (FALSE);
	}

	while (fgets(line, 500, gamelist)) {
		count = 0;
		i = 0;

		p = line;
		tmp = p;

		/* Skip comments */
		if (*tmp != '#') {
			while (*tmp && (*tmp++ != '\n')) {
				i++;
				count++;
			}
			tmp_array = g_strsplit(p, "", 38);
			rom = g_malloc(sizeof(RomEntry));
			if (rom == NULL)
				g_warning
				    ("Out of memory while loading gamelist");

			g_snprintf(rom->romname, MAX_ROMNAME,
				   "%s", tmp_array[0]);
			g_snprintf(rom->gamename, MAX_GAMENAME,
				   "%s", tmp_array[1]);
			g_snprintf(rom->gamenameext, MAX_GAMENAME,
				   "%s", tmp_array[2]);
			if (!strcmp(tmp_array[3], "true"))
				rom->the_trailer = TRUE;
			else 
				rom->the_trailer = FALSE;

			g_snprintf(rom->year, MAX_YEAR, "%s",
				   tmp_array[4]);
			g_snprintf(rom->manu, MAX_MANU, "%s",
				   tmp_array[5]);
			g_snprintf(rom->cloneof, MAX_ROMNAME,
				   "%s", tmp_array[6]);
			g_snprintf(rom->romof, MAX_ROMNAME,
				   "%s", tmp_array[7]);
			g_snprintf(rom->driver, MAX_ROMNAME,
				   "%s", tmp_array[8]);

			if (!strcmp(tmp_array[9], "true"))
				rom->status = TRUE;
			else 
				rom->status = FALSE;
			rom->colors = atoi(tmp_array[10]);
			
			/* CPU */
			rom->cpu1_sound_flag = rom->cpu2_sound_flag = rom->cpu3_sound_flag = rom->cpu4_sound_flag = FALSE;
			if (!strncmp(tmp_array[11], "(sound)",7)) {
				p = tmp_array[11]; p=p+7;
				g_snprintf(rom->cpu1, MAX_CPU,
					   "%s", p);
				rom->cpu1_sound_flag = TRUE;
			}
			else
				g_snprintf(rom->cpu1, MAX_CPU,
					   "%s", tmp_array[11]);
			rom->cpu1_clock = atoi(tmp_array[12]);
			
			if (!strncmp(tmp_array[13], "(sound)",7)) {
				p = tmp_array[13]; p=p+7;
				g_snprintf(rom->cpu2, MAX_CPU,
					   "%s", p);
				rom->cpu2_sound_flag = TRUE;
			}
			else
				g_snprintf(rom->cpu2, MAX_CPU,
					   "%s", tmp_array[13]);
			rom->cpu2_clock = atoi(tmp_array[14]);
			
			if (!strncmp(tmp_array[15], "(sound)",7)) {
				p = tmp_array[15]; p=p+7;
				g_snprintf(rom->cpu3, MAX_CPU,
					   "%s", p);
				rom->cpu3_sound_flag = TRUE;
			}
			else
				g_snprintf(rom->cpu3, MAX_CPU,
					   "%s", tmp_array[15]);
			rom->cpu3_clock = atoi(tmp_array[16]);
			
			if (!strncmp(tmp_array[17], "(sound)",7)) {
				p = tmp_array[17]; p=p+7;
				g_snprintf(rom->cpu4, MAX_CPU,
					   "%s", p);
				rom->cpu4_sound_flag = TRUE;
			}
			else
				g_snprintf(rom->cpu4, MAX_CPU,
					   "%s", tmp_array[17]);
			rom->cpu4_clock = atoi(tmp_array[18]);
			
			
			
			
			if (strcmp(tmp_array[19], "")) {
				g_snprintf(rom->sound1, MAX_CPU,
					   "%s", tmp_array[19]);
			}
			rom->sound1_clock = atoi(tmp_array[20]);
			
			if (strcmp(tmp_array[21], "")) {
				g_snprintf(rom->sound2, MAX_CPU,
					   "%s", tmp_array[21]);
			}
			rom->sound2_clock = atoi(tmp_array[22]);
			
			if (strcmp(tmp_array[23], "")) {
				g_snprintf(rom->sound3, MAX_CPU,
					   "%s", tmp_array[23]);
			}
			rom->sound3_clock = atoi(tmp_array[24]);
			
			if (strcmp(tmp_array[25], "")) {
				g_snprintf(rom->sound4, MAX_CPU,
					   "%s", tmp_array[25]);
			}
			rom->sound4_clock = atoi(tmp_array[26]);
			
			rom->num_players = atoi(tmp_array[27]);
			rom->num_buttons = atoi(tmp_array[28]);
			if (strcmp(tmp_array[29], "")) {
				g_snprintf(rom->control,
					   MAX_CONTROL, "%s",
					   tmp_array[29]);
			}
			if (!strcmp(tmp_array[30], "true"))
				rom->vector = TRUE;
			else
				rom->vector = FALSE;

			rom->screen_x = atoi(tmp_array[31]);
			rom->screen_y = atoi(tmp_array[32]);
			rom->screen_freq = atoi(tmp_array[33]);
			if (!strcmp(tmp_array[34], "horizontal"))
				rom->horizontal = TRUE;
			else
				rom->horizontal = FALSE;
			
			rom->channels = atoi(tmp_array[35]);
			rom->nb_roms = atoi(tmp_array[36]);
			rom->nb_samples = atoi(tmp_array[37]);
			if  (rom->nb_samples>0)
				supported_samples_games++;
			g_strfreev(tmp_array);
			
			/*initialize timesplayed and favorite fields*/
			rom->timesplayed = 0;
			rom->favourite = 0;

			/*generate Glist for years*/
			year = atoi(rom->year);
			if  (!g_list_find(years, GINT_TO_POINTER(year)))
			{
				years = g_list_prepend(years, GINT_TO_POINTER(year));
			}

			/*generate glist for manufacturers*/
			manufacturer_fields = get_manufacturers (rom);
			manufacturer1 = g_strdup(manufacturer_fields[0]);
			manufacturer_in_list = FALSE;
			listpointer = g_list_first(manufacturers);
			while ((listpointer != NULL))
			{
				if (!strncmp(listpointer->data,manufacturer1,5))
				{
					manufacturer_in_list = TRUE;
					break;
				}
				listpointer = g_list_next(listpointer);
			}
			if (!manufacturer_in_list)
			{
			 	manufacturers = g_list_prepend(manufacturers, manufacturer1);
			}
			if (manufacturer_fields[1] != NULL)
			{
				manufacturer2 = g_strdup(manufacturer_fields[1]);
				manufacturer_in_list = FALSE;
				listpointer = g_list_first(manufacturers);
				while ((listpointer != NULL))
				{
					if (!strncmp(listpointer->data,manufacturer2,5))
					{
						manufacturer_in_list = TRUE;
						break;
					}
					listpointer = g_list_next(listpointer);
				}
				if (!manufacturer_in_list)
				{
				 	manufacturers = g_list_prepend(manufacturers, manufacturer2);
				}
			}
			g_strfreev(manufacturer_fields);
			
			/* fill the some fields with default value if we have problems
			   with game.ini after */
			rom->timesplayed = 0;
			rom->has_roms = 2;
			rom->has_samples = 2;
			rom->favourite = FALSE;
			rom->icon_pixbuf = NULL;

			/* fill the some fields with default value if we have problems
			   with catver after */
			g_snprintf(rom->category,MAX_CATEGORY,"Unknown");
			g_snprintf(rom->mame_ver_added,MAX_MAMEVER,"Unknown");

			roms = g_list_insert_sorted(roms, (gpointer) rom, (GCompareFunc )compare_game_name);
		} else if (!exe_version_checked) {
			/* Check for the GXMame version in the top comment line */
			while (*tmp && (*tmp++ != '\n')) {
				i++;
				count++;
			}
			p[i]='\0';
			tmp_array = g_strsplit(p, " ", 3);

			gxmame_debug("Checking version of gamelist file: %s - %s",tmp_array[1], tmp_array[2]);

			if (strcmp(tmp_array[1], "GXMame")) {
				g_message(_("Could not recognize format of gamelist.\nYou need to build a new gamelist."));
				g_free(list_version.version);
				list_version.version = g_strdup("unknown");
				g_strfreev(tmp_array);
				return (FALSE);
			} else if (g_strtod(tmp_array[2], NULL) < 0.7) {
				g_message(_("Gamelist from too old GXMame\nThe format of the gamelist has changed in this version.\nYou need to build a new gamelist."));
				g_free(list_version.version);
				list_version.version = g_strdup("too old");
				g_strfreev(tmp_array);
				return (FALSE);
			}
			g_strfreev(tmp_array);
			exe_version_checked = TRUE;
		} else {
			/* Check for the GXMame version in the top comment line */
			while (*tmp && (*tmp++ != '\n')) {
				i++;
				count++;
			}
			p[i]='\0';
			gxmame_debug("reading comment: %s",p);

			if (!strncmp(p, "# Name",6))
			{
				p = p+7;
				g_free(list_version.name);
				list_version.name = g_strdup(p);
			} else if (!strncmp(p, "# Target",8))
			{
				p = p+9;
				g_free(list_version.target);
				list_version.target = g_strdup(p);
			} else if (!strncmp(p, "# Version",9))
			{ 
				p = p+10;
				g_free(list_version.version);
				list_version.version = g_strdup(p);
			}
			
/*			if (strncmp(p, "# Version",9)) {
				g_message(_("Could not recognize format of gamelist.\nYou need to build a new gamelist."));
				list_exe_mame_version = g_strdup("unknown");
				return (FALSE);
			} else  {
				p = p+10;
				list_exe_mame_version = g_strdup(p);
			}*/
		}
	}

	fclose(gamelist);
	g_free(filename);

	supported_games = g_list_length(roms);
	g_message(_("Loaded %d roms by %d manufacturers covering %d years."),g_list_length(roms),g_list_length(manufacturers),g_list_length(years));
	g_message(_("with %d games supporting samples."),supported_samples_games);

	return (TRUE);
}


gboolean load_games_ini(void)
{
	FILE *gameini_list;
	gchar *filename;
	gchar line[2000];
	gchar **option_value = NULL;
	GList *listpointer;
	RomEntry *tmprom = NULL;
	gboolean game_found=FALSE;
	
	filename = g_strdup_printf("%s/.gxmame/games.ini",getenv("HOME"));	
	gameini_list = fopen(filename, "r");
	if (gameini_list == NULL)
	{
		gxmame_debug("games.ini not found");
		g_free(filename);
		not_checked_list = g_list_copy (roms);
		return (FALSE);
	}
	while (fgets(line, 500, gameini_list))
	{
		if(*line =='[')
		{	
			g_strstrip(g_strdelimit(line,"[]\r\n",' '));
			/* ok only if mame32k games,ini doesn't change */
			/* find the the rom in the glist */
			for(listpointer = g_list_first(roms), game_found = FALSE;
					(listpointer != NULL);
					listpointer = g_list_next(listpointer))
			{
				tmprom = (RomEntry *) listpointer->data;
				if (!strcmp(tmprom->romname,line))
				{
					game_found = TRUE;
					break;
				}
			}
			if (game_found)
			{
				while (fgets(line, 500, gameini_list))
				{/* need to test the carriage return if the file come from Mame32K under windows*/
					if(*line =='\r' ||*line =='\n')
						break;
					option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
					if (option_value[1] != NULL)
					{	if (!strcmp(option_value[0],"PlayCount"))
						{	tmprom->timesplayed = atoi(option_value[1]);
						} else if (!strcmp(option_value[0],"HasRoms"))
						{	tmprom->has_roms = atoi(option_value[1]);
						} else if (!strcmp(option_value[0],"HasSamples"))
						{	tmprom->has_samples = atoi(option_value[1]);
						} else if (!strcmp(option_value[0],"Favorite"))
						{	tmprom->favourite = atoi(option_value[1])?TRUE:FALSE;
						}else if (!strcmp(option_value[0],"AutofireDelay"))
						{	/*store this somewhere to be able to put it again
							  when we will save the game.ini*/
						}
					g_strfreev(option_value);
					}
				}
				if((tmprom->has_roms == 2) || ((tmprom->has_roms==0) && gui_prefs.GameCheck))
				{
					not_checked_list = g_list_append(not_checked_list, tmprom);
				}
			}
		}
		
	}
	fclose(gameini_list);
	g_free(filename);
	return TRUE;
}


gboolean save_games_ini(void)
{
	FILE *game_ini_file=NULL;
	gchar *filename;
	gchar *romname = NULL;
	gchar *timesplayed;
	gchar *has_roms;
	gchar *has_samples;
	gchar *favourite;
	RomEntry *rom;
	GList *list_pointer;
	
	
	filename = g_strdup_printf("%s/.gxmame/games.ini",getenv("HOME"));	
	
	if ((game_ini_file = fopen(filename, "w")) == NULL)
	{
		gxmame_debug("unable to write games.ini");
		g_free(filename);
		return (FALSE);
	}
	
	romname = g_malloc0(MAX_ROMNAME);
	timesplayed = g_malloc0(5);
	has_roms = g_malloc0(5);
	has_samples = g_malloc0(5);
	favourite = g_malloc0(5);
	
	for (list_pointer = g_list_first(roms);
			(list_pointer != NULL);
			list_pointer=g_list_next(list_pointer))
	{
		rom = (RomEntry *)list_pointer->data;
		g_snprintf(timesplayed,5,"%i",rom->timesplayed);
		g_snprintf(has_roms,5,"%i",rom->has_roms);
		g_snprintf(has_samples,5,"%i",rom->has_samples);
		g_snprintf(favourite,5,"%s",rom->favourite?"1":"0");
		fputc ('[',game_ini_file);
		fputs (rom->romname,game_ini_file);
		fputs ("]",game_ini_file);
		fputs ("\r\nPlayCount=",game_ini_file);
		fputs (timesplayed,game_ini_file);
		fputs ("\r\nHasRoms=",game_ini_file);
		fputs (has_roms,game_ini_file);
		fputs ("\r\nHasSamples=",game_ini_file);
		fputs (has_samples,game_ini_file);
		fputs ("\r\nFavorite=",game_ini_file);
		fputs (favourite,game_ini_file);
		fputs ("\r\n\r\n",game_ini_file);
		
	}
	fclose(game_ini_file);
	g_free(filename);
	g_free(romname);
	g_free(timesplayed);
	g_free(has_roms);
	g_free(has_samples);
	g_free(favourite);
	return TRUE;
}


gboolean load_catver_ini(void)
{
	FILE *catver_file;
	gchar *filename;
	gchar line[2000];
	gchar **game_value = NULL;
	GList *listpointer;
	RomEntry *tmprom = NULL;
	gboolean game_found=FALSE;
	gchar *category;
	gchar *version;
	gboolean category_in_list, version_in_list;
	
	filename = g_strdup_printf("%scatver.ini",gui_prefs.catverDirectory);	
	catver_file = fopen(filename, "r");
	if (catver_file == NULL)
	{
		gxmame_debug("catver.ini not found in %s",filename);
		g_free(filename);
		filename = g_strdup_printf("%scatver",gui_prefs.catverDirectory);	
		catver_file = fopen(filename, "r");
		if (catver_file == NULL)
		{
			gxmame_debug("catver not found in %s",filename);
			g_free(filename);
			return (FALSE);
		}
	}
	fgets(line, 500, catver_file);
	if (*line!=';')
	{
		g_free(filename);
		return FALSE;
	}
	
	/* Jump initial comments and empty lines */
	while(*line=='\r' || *line=='\n' ||*line==';')
	{	g_strstrip(g_strdelimit(line,"\r\n",' '));
		gxmame_debug("Loading catver: %s",line);
		fgets(line, 500, catver_file);
	}
	
	if(strncmp(line,"[Category]", 10))
	{
		g_free(filename);
		return FALSE;
	}
		
	while (fgets(line, 500, catver_file))
	{
		/*end of categories */
		if (*line=='[')
			break;
		if (*line=='\r' ||*line=='\n' || *line==';')
			continue;
		
		g_strstrip(g_strdelimit(line,"\r\n",' '));
		game_value = g_strsplit(line,"=",2);
		for(listpointer = g_list_first(roms), game_found = FALSE;
				(listpointer != NULL);
				listpointer = g_list_next(listpointer))
		{
			tmprom = (RomEntry *) listpointer->data;
			if (!strcmp(tmprom->romname,game_value[0]))
			{
				game_found = TRUE;
				break;
			}
		}
		if (game_found)
		{
			g_snprintf(tmprom->category,MAX_CATEGORY, game_value[1]);
		}
		/* create the glist for categories */
		category_in_list = FALSE;
		listpointer = g_list_first(categories);
		while ((listpointer != NULL))
		{
			if (!strcmp(listpointer->data,game_value[1]))
			{
				category_in_list = TRUE;
				break;
			}
			listpointer = g_list_next(listpointer);
		}
		if (!category_in_list)
		{
			category = g_strdup(game_value[1]);
		 	categories = g_list_prepend(categories, category);
		}
		g_strfreev(game_value);
	}
	
	/* and now come the version*/
	if(strncmp(line,"[VerAdded]", 10))
	{
		gxmame_debug("[VerAdded] expected but found:%s",line);
		g_free(filename);
		return FALSE;
	}
		
	while (fgets(line, 500, catver_file))
	{
		/* Jump initials comment and empty line */
		if (*line=='\r' ||*line=='\n' || *line==';')
			continue;

		g_strstrip(g_strdelimit(line,"\r\n",' '));
		game_value = g_strsplit(line,"=",2);
		for(listpointer = g_list_first(roms), game_found = FALSE;
				(listpointer != NULL);
				listpointer = g_list_next(listpointer))
		{
			tmprom = (RomEntry *) listpointer->data;
			if (!strcmp(tmprom->romname,game_value[0]))
			{
				game_found = TRUE;
				break;
			}
		}
		if (game_found)
		{
			g_snprintf(tmprom->mame_ver_added,MAX_MAMEVER, game_value[1]);
		}
		/* create the glist for versions */
		version_in_list = FALSE;
		listpointer = g_list_first(versions);
		while ((listpointer != NULL))
		{
			if (!strcmp(listpointer->data,game_value[1]))
			{
				version_in_list = TRUE;
				break;
			}
			listpointer = g_list_next(listpointer);
		}
		if (!version_in_list)
		{
			version = g_strdup(game_value[1]);
		 	versions = g_list_prepend(versions, version);
		}
		g_strfreev(game_value);
	}
	fclose(catver_file);
	g_free(filename);
	return TRUE;
}


/* preferences for the gui */
gboolean load_gxmame_ini(void)
{
	FILE *gameini_list;
	gchar *filename;
	gchar line[2000];
	gchar **option_value = NULL;
	gchar **option_value2 = NULL;
	
	g_snprintf(gui_prefs.DefaultGame,MAX_GAMENAME,"%s","Pac-Man (Midway)");
	gui_prefs.FolderID = 1;
	gui_prefs.ShowScreenShot = 1;
	gui_prefs.ShowFlyer = 0;
	gui_prefs.ShowToolBar = 1;
	gui_prefs.ShowStatusBar = 1;
	gui_prefs.ShowFolderList = 1;
	gui_prefs.SortColumn = 0;
	gui_prefs.SortReverse = 0;
	gui_prefs.GUIPosX = 0;
	gui_prefs.GUIPosY = 0;
	gui_prefs.GUIWidth = 640;
	gui_prefs.GUIHeight = 400;
	gui_prefs.previous_mode = 3;
	gui_prefs.current_mode = 3;
	gui_prefs.Splitters[0] = 150;
	gui_prefs.Splitters[1] = 300;
	gui_prefs.ModifyThe = TRUE;
	gui_prefs.GameCheck = FALSE;
	gui_prefs.VersionCheck = TRUE;
	gui_prefs.gui_joy = TRUE;
	
	filename = g_strdup_printf("%s/.gxmame/gxmame.ini",getenv("HOME"));	
	gameini_list = fopen(filename, "r");
	if (gameini_list == NULL)
	{
		gxmame_debug("gxmame.ini not found");
		g_free(filename);
		return (FALSE);
	}
	/* Be careful of the \r\n terminal due to windows */
	while (fgets(line, 500, gameini_list))
	{
		if (!strncmp(line,"[default]",9))
		{	
			while (fgets(line, 500, gameini_list))
			{/* need to test the carriage return if the file come from Mame32K under windows*/
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"DefaultGame"))
					{	g_snprintf(gui_prefs.DefaultGame,MAX_GAMENAME,"%s",option_value[1]);
					} else if (!strcmp(option_value[0],"FolderID"))
					{	gui_prefs.FolderID = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"ShowScreenShot"))
					{	gui_prefs.ShowScreenShot = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"ShowFlyer"))
					{	gui_prefs.ShowFlyer = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"ShowToolBar"))
					{	gui_prefs.ShowToolBar = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"ShowStatusBar"))
					{	gui_prefs.ShowStatusBar = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"ModifyThe"))
					{	gui_prefs.ModifyThe = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"ShowFolderList"))
					{	gui_prefs.ShowFolderList = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"GameCheck"))
					{	gui_prefs.GameCheck = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"VersionCheck"))
					{	gui_prefs.VersionCheck = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"gui_joy"))
					{	gui_prefs.gui_joy = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"SortColumn"))
					{	gui_prefs.SortColumn = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"SortReverse"))
					{	gui_prefs.SortReverse = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"GUIPosX"))
					{	gui_prefs.GUIPosX = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"GUIPosY"))
					{	gui_prefs.GUIPosY = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"GUIWidth"))
					{	gui_prefs.GUIWidth = atoi(option_value[1]);
					}else if (!strcmp(option_value[0],"GUIHeight"))
					{	gui_prefs.GUIHeight = atoi(option_value[1]);
					}else if (!strcmp(option_value[0],"ListMode"))
					{	gui_prefs.current_mode = gui_prefs.previous_mode = atoi(option_value[1]);
					}else if (!strcmp(option_value[0],"Splitters"))
					{	
						option_value2 = g_strsplit(option_value[1],",",2);
						gui_prefs.Splitters[0] = atoi(option_value2[0]);
						gui_prefs.Splitters[1] = atoi(option_value2[1]);
						g_strfreev(option_value2);
					} else if (!strcmp(option_value[0],"ListFont"))
					{	gui_prefs.ListFont = g_strdup(option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}
	}
	fclose(gameini_list);
	g_free(filename);
	return TRUE;
}


gboolean save_gxmame_ini(void)
{
	FILE *gxmame_ini_file = NULL;
	gchar *filename;
	gchar *option_string = NULL;
	
	
	filename = g_strdup_printf("%s/.gxmame/gxmame.ini",getenv("HOME"));	
	
	if ((gxmame_ini_file = fopen(filename, "w")) == NULL)
	{
		gxmame_debug("unable to write gxmame.ini");
		g_free(filename);
		return (FALSE);
	}
	
	option_string = g_malloc0(MAX_PATHNAME);
 
	fputs ("[default]",gxmame_ini_file);
	fputs ("\r\nDefaultGame=",gxmame_ini_file);
	fputs (gui_prefs.DefaultGame,gxmame_ini_file);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.FolderID);
	fputs ("\r\nFolderID=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.ShowScreenShot);
	fputs ("\r\nShowScreenShot=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.ShowFlyer);
	fputs ("\r\nShowFlyer=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.ShowToolBar);
	fputs ("\r\nShowToolBar=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.ModifyThe);
	fputs ("\r\nModifyThe=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);

	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.ShowStatusBar);
	fputs ("\r\nShowStatusBar=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.ShowFolderList);
	fputs ("\r\nShowFolderList=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.GameCheck);
	fputs ("\r\nGameCheck=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);

	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.VersionCheck);
	fputs ("\r\nVersionCheck=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);

	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.gui_joy);
	fputs ("\r\ngui_joy=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);

	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.SortColumn);
	fputs ("\r\nSortColumn=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.SortReverse);
	fputs ("\r\nSortReverse=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	gdk_window_get_position(MainWindow->window, &gui_prefs.GUIPosX,&gui_prefs.GUIPosY);
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.GUIPosX);
	fputs ("\r\nGUIPosX=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.GUIPosY);
	fputs ("\r\nGUIPosY=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	gdk_window_get_size(MainWindow->window, &gui_prefs.GUIWidth,&gui_prefs.GUIHeight);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.GUIWidth);
	fputs ("\r\nGUIWidth=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);

	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.GUIHeight);
	fputs ("\r\nGUIHeight=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.current_mode);
	fputs ("\r\nListMode=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i,%i",
			gui_prefs.ShowFolderList?main_gui.scrolled_window_filters->allocation.width:gui_prefs.Splitters[0],
			gui_prefs.ShowScreenShot?main_gui.scrolled_window_games->allocation.width:gui_prefs.Splitters[1]);
	fputs ("\r\nSplitters=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	fputs ("\r\nListFont=",gxmame_ini_file);
	if (gui_prefs.ListFont)
		fputs (gui_prefs.ListFont,gxmame_ini_file);
	
	fputs ("\r\n\r\n",gxmame_ini_file);

	fclose(gxmame_ini_file);
	g_free(filename);
	g_free(option_string);
	return TRUE;
}


/* this the master config file, it will contain the path to every other config file
   when we will share these config files with another mame32 application */
gboolean load_gxmamerc(void)
{
	FILE *gamerc_file;
	gchar *filename;
	gchar line[2000];
	gchar **option_value = NULL;
	
	/* default values */
	mame_executable = g_strdup("xmame");
	/* for the xmame_executables_array, we first try to load it, if not found,
	   we take the mame_executable value
	   Only after we will check the validity of executable and put them in a hash table 
	   to use them in the GUI */
	xmame_executables_array = NULL;
	gui_prefs.catverDirectory = g_strdup_printf("%s/.gxmame/",getenv("HOME"));
	gui_prefs.CheatFile = g_strdup("/usr/lib/games/xmame/cheat.dat");
	gui_prefs.HiscoreFile = g_strdup("/usr/lib/games/xmame/hiscore.dat");
	gui_prefs.HistoryFile = g_strdup("/usr/lib/games/xmame/history.dat");
	gui_prefs.MameInfoFile = g_strdup("/usr/lib/games/xmame/mameinfo.dat");   
	
	gui_prefs.clone_color.red=47031;
	gui_prefs.clone_color.green=47031;
	gui_prefs.clone_color.blue=47031;
	gui_prefs.Joystick_in_GUI = g_strdup("/dev/js0");   
	
	filename = g_strdup_printf("%s/.gxmame/gxmamerc",getenv("HOME"));	
	gamerc_file = fopen(filename, "r");
	if (gamerc_file == NULL)
	{
		/* set a default value for the xmame_executables_array */
		if (xmame_executables_array==NULL)
			xmame_executables_array = g_strsplit(mame_executable,";",2);
		gxmame_debug("gxmamerc not found");
		g_free(filename);
		return (FALSE);
	}

	while (fgets(line, 500, gamerc_file))
	{
		/* skip comments */
		if (*line!='#')
		{	
			if(*line =='\n')
				break;
			option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\n",' ')),"=",0);
			if (option_value[1] != NULL)
			{	if (!strcmp(option_value[0],"mame_executable"))
				{
					g_free(mame_executable);
					mame_executable = g_strdup(option_value[1]);
				} else if (!strcmp(option_value[0],"xmame_executables_array"))
				{	if (xmame_executables_array!=NULL)
						g_strfreev(xmame_executables_array);
					xmame_executables_array = g_strsplit(option_value[1],";",0);
				} else if (!strcmp(option_value[0],"catverDirectory"))
				{
					g_free(gui_prefs.catverDirectory);
					gui_prefs.catverDirectory = g_strdup(option_value[1]);
				} else if (!strcmp(option_value[0],"CheatFile"))
				{	
					g_free(gui_prefs.CheatFile);
					gui_prefs.CheatFile = g_strdup(option_value[1]);
				} else if (!strcmp(option_value[0],"HiscoreFile"))
				{	
					g_free(gui_prefs.HiscoreFile);
					gui_prefs.HiscoreFile = g_strdup(option_value[1]);
				} else if (!strcmp(option_value[0],"HistoryFile"))
				{	
					g_free(gui_prefs.HistoryFile);
					gui_prefs.HistoryFile = g_strdup(option_value[1]);
				} else if (!strcmp(option_value[0],"MameInfoFile"))
				{	
					g_free(gui_prefs.MameInfoFile);
					gui_prefs.MameInfoFile = g_strdup(option_value[1]);
				} else if (!strcmp(option_value[0],"Joystick_in_GUI"))
				{	
					g_free(gui_prefs.Joystick_in_GUI);
					gui_prefs.Joystick_in_GUI = g_strdup(option_value[1]);
				} else if (!strcmp(option_value[0],"clone_color.red"))
				{	
					gui_prefs.clone_color.red = atoi(option_value[1]);
				} else if (!strcmp(option_value[0],"clone_color.green"))
				{	
					gui_prefs.clone_color.green = atoi(option_value[1]);
				} else if (!strcmp(option_value[0],"clone_color.blue"))
				{	
					gui_prefs.clone_color.blue = atoi(option_value[1]);
				}
				g_strfreev(option_value);
			}
		}
	}
	/* if we haven't found xmame_executables_array in config file, used the mame_executable */
	if (xmame_executables_array==NULL)
		xmame_executables_array = g_strsplit(mame_executable,";",2);
	fclose(gamerc_file);
	g_free(filename);
	return TRUE;
}


gboolean save_gxmamerc(void)
{
	FILE *gamerc_file;
	gchar *filename, *option_string;
	guint i;
	
	filename = g_strdup_printf("%s/.gxmame/gxmamerc",getenv("HOME"));	
	gamerc_file = fopen(filename, "w");
	if (gamerc_file == NULL)
	{
		gxmame_debug("unable to write gxmamerc");
		g_free(filename);
		return (FALSE);
	}
	fputs ("# master config file of GXMame",gamerc_file);
	fputs ("\nmame_executable=",gamerc_file);
	fputs (mame_executable,gamerc_file);
	fputs ("\nxmame_executables_array=",gamerc_file);
	for (i=0;xmame_executables_array[i]!=NULL;i++)
	{	
		fputs (xmame_executables_array[i],gamerc_file);
		if (xmame_executables_array[i+1]!=NULL)
			fputs (";",gamerc_file);
	}
	
	option_string = g_malloc0(MAX_OPTION_STRING);

	fputs ("\ncatverDirectory=",gamerc_file);
	fputs (gui_prefs.catverDirectory,gamerc_file);
	fputs ("\nCheatFile=",gamerc_file);
	fputs (gui_prefs.CheatFile,gamerc_file);
	fputs ("\nHiscoreFile=",gamerc_file);
	fputs (gui_prefs.HiscoreFile,gamerc_file);
	fputs ("\nHistoryFile=",gamerc_file);
	fputs (gui_prefs.HistoryFile,gamerc_file);
	fputs ("\nMameInfoFile=",gamerc_file);
	fputs (gui_prefs.MameInfoFile,gamerc_file);
	fputs ("\nJoystick_in_GUI=",gamerc_file);
	fputs (gui_prefs.Joystick_in_GUI,gamerc_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"\nclone_color.red=%i",gui_prefs.clone_color.red);
	fputs (option_string,gamerc_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"\nclone_color.green=%i",gui_prefs.clone_color.green);
	fputs (option_string,gamerc_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"\nclone_color.blue=%i",gui_prefs.clone_color.blue);
	fputs (option_string,gamerc_file);
	fputs ("\n",gamerc_file);
	
	fclose(gamerc_file);
	g_free(filename);
	g_free(option_string);
	
	return TRUE;
}


/* this is where directory paths are set (common with mame32k) */
gboolean load_dirs_ini(void)
{
	FILE *dirs_ini_file;
	gchar *filename;
	gchar line[2000];
	gchar **option_value = NULL;
	
	gui_prefs.RomPath = g_strsplit("/usr/lib/games/xmame/roms/",";",2);
	gui_prefs.SamplePath = g_strsplit("/usr/lib/games/xmame/samples/",";",2);
	gui_prefs.SnapshotDirectory = g_strdup("/usr/lib/games/xmame/snap/");
	gui_prefs.ArtworkDirectory = g_strdup("/usr/lib/games/xmame/artwork/");
	gui_prefs.HiscoreDirectory = g_strdup("~/.xmame/hi/");
	gui_prefs.FlyerDirectroy = g_strdup("/usr/lib/games/xmame/flyers/");
	gui_prefs.CabinetDirectory = g_strdup("/usr/lib/games/xmame/cabinets/");
	gui_prefs.MarqueeDirectory = g_strdup("/usr/lib/games/xmame/marquees/");
	gui_prefs.TitleDirectory = g_strdup("/usr/lib/games/xmame/titles/");
	gui_prefs.IconDirectory = g_strdup("/usr/lib/games/xmame/icons/");
	
	gui_prefs.DiffDirectory = g_strdup("~/.xmame/diff/");
	gui_prefs.CtrlrDirectory = g_strdup("/usr/lib/games/xmame/ctrlr/");
	
	gui_prefs.InputDirectory = g_strdup("~/.xmame/inp/");
	gui_prefs.NVRamDirectory = g_strdup("~/.xmame/nvram/");
	gui_prefs.MemCardDirectory = g_strdup("~/.xmame/memcard/");
	gui_prefs.ConfigDirectory = g_strdup("~/.xmame/cfg/");
	gui_prefs.StateDirectory = g_strdup("~/.xmame/sta/");
	gui_prefs.inipath = g_strdup("~/.xmame/ini/");
	
	filename = g_strdup_printf("%s/.gxmame/dirs.ini",getenv("HOME"));	
	dirs_ini_file = fopen(filename, "r");
	if (dirs_ini_file == NULL)
	{
		gxmame_debug("dirs.ini not found");
		g_free(filename);
		return (FALSE);
	}
	/* Be careful of the \r\n terminal due to windows */
	while (fgets(line, 500, dirs_ini_file))
	{
		if (!strncmp(line,"[directories]",13))
		{	
			while (fgets(line, 500, dirs_ini_file))
			{/* need to test the carriage return if the file come from Mame32K under windows*/
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"RomPath"))
					{	
						g_strfreev(gui_prefs.RomPath);
						gui_prefs.RomPath = g_strsplit(option_value[1],";",0);
					} else if (!strcmp(option_value[0],"SamplePath"))
					{	
						g_strfreev(gui_prefs.SamplePath);
						gui_prefs.SamplePath = g_strsplit(option_value[1],";",0);
					} else if (!strcmp(option_value[0],"ConfigDirectory"))
					{	
						g_free(gui_prefs.ConfigDirectory);
						gui_prefs.ConfigDirectory = g_strdup(option_value[1]);
/*					} else if (!strcmp(option_value[0],"FolderDirectory"))
					{	
						g_free(gui_prefs.FolderDirectory);
						gui_prefs.FolderDirectory = g_strdup(option_value[1]);
*/					} else if (!strcmp(option_value[0],"NVRamDirectory"))
					{	
						g_free(gui_prefs.NVRamDirectory);
						gui_prefs.NVRamDirectory = g_strdup(option_value[1]);
					} else if (!strcmp(option_value[0],"CtrlrDirectory"))
					{	
						g_free(gui_prefs.CtrlrDirectory);
						gui_prefs.CtrlrDirectory = g_strdup(option_value[1]);
					} else if (!strcmp(option_value[0],"MemCardDirectory"))
					{	
						g_free(gui_prefs.MemCardDirectory);
						gui_prefs.MemCardDirectory = g_strdup(option_value[1]);
					} else if (!strcmp(option_value[0],"InputDirectory"))
					{	
						g_free(gui_prefs.InputDirectory);
						gui_prefs.InputDirectory = g_strdup(option_value[1]);
					} else if (!strcmp(option_value[0],"HiscoreDirectory"))
					{	
						g_free(gui_prefs.HiscoreDirectory);
						gui_prefs.HiscoreDirectory = g_strdup(option_value[1]);
					} else if (!strcmp(option_value[0],"StateDirectory"))
					{	
						g_free(gui_prefs.StateDirectory);
						gui_prefs.StateDirectory = g_strdup(option_value[1]);
					} else if (!strcmp(option_value[0],"ArtworkDirectory"))
					{
						g_free(gui_prefs.ArtworkDirectory);
						gui_prefs.ArtworkDirectory = g_strdup(option_value[1]);
					} else if (!strcmp(option_value[0],"SnapshotDirectory"))
					{	
						g_free(gui_prefs.SnapshotDirectory);
						gui_prefs.SnapshotDirectory = g_strdup(option_value[1]);
/*					} else if (!strcmp(option_value[0],"WavDirectory"))
					{	g_snprintf(gui_prefs.WavDirectory,MAX_PATHNAME,"%s",option_value[1]);
*/					} else if (!strcmp(option_value[0],"FlyerDirectroy"))
					{
						g_free(gui_prefs.FlyerDirectroy);
						gui_prefs.FlyerDirectroy = g_strdup(option_value[1]);
					} else if (!strcmp(option_value[0],"CabinetDirectory"))
					{
						g_free(gui_prefs.CabinetDirectory);
						gui_prefs.CabinetDirectory = g_strdup(option_value[1]);
					} else if (!strcmp(option_value[0],"MarqueeDirectory"))
					{	
						g_free(gui_prefs.MarqueeDirectory);
						gui_prefs.MarqueeDirectory = g_strdup(option_value[1]);
					} else if (!strcmp(option_value[0],"TitleDirectory"))
					{	
						g_free(gui_prefs.TitleDirectory);
						gui_prefs.TitleDirectory = g_strdup(option_value[1]);
					} else if (!strcmp(option_value[0],"DiffDirectory"))
					{	
						g_free(gui_prefs.DiffDirectory);
						gui_prefs.DiffDirectory = g_strdup(option_value[1]);
					} else if (!strcmp(option_value[0],"IconDirectory"))
					{	
						g_free(gui_prefs.IconDirectory);
						gui_prefs.IconDirectory = g_strdup(option_value[1]);
					} else if (!strcmp(option_value[0],"inipath"))
					{	
						g_free(gui_prefs.inipath);
						gui_prefs.inipath = g_strdup(option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}
	}
	fclose(dirs_ini_file);
	g_free(filename);
	return TRUE;
}


gboolean save_dirs_ini(void)
{
	FILE *dir_ini_file = NULL;
	gchar *filename;
	int i;
	
	filename = g_strdup_printf("%s/.gxmame/dirs.ini",getenv("HOME"));	
	
	if ((dir_ini_file = fopen(filename, "w")) == NULL)
	{
		gxmame_debug("unable to write dirs.ini");
		g_free(filename);
		return (FALSE);
	}
	
      
	fputs ("[directories]",dir_ini_file);
	fputs ("\r\nRomPath=",dir_ini_file);
	for (i=0;gui_prefs.RomPath[i]!=NULL;i++)
	{	
		fputs (gui_prefs.RomPath[i],dir_ini_file);
		if (gui_prefs.RomPath[i+1]!=NULL)
			fputs (";",dir_ini_file);
	}
	fputs ("\r\nSamplePath=",dir_ini_file);
	for (i=0;gui_prefs.SamplePath[i]!=NULL;i++)
	{
		fputs (gui_prefs.SamplePath[i],dir_ini_file);
		if (gui_prefs.SamplePath[i+1]!=NULL)
			fputs (";",dir_ini_file);
	}
	fputs ("\r\nConfigDirectory=",dir_ini_file);
	fputs (gui_prefs.ConfigDirectory,dir_ini_file);
	fputs ("\r\nNVRamDirectory=",dir_ini_file);
	fputs (gui_prefs.NVRamDirectory,dir_ini_file);
	fputs ("\r\nCtrlrDirectory=",dir_ini_file);
	fputs (gui_prefs.CtrlrDirectory,dir_ini_file);
	fputs ("\r\nMemCardDirectory=",dir_ini_file);
	fputs (gui_prefs.MemCardDirectory,dir_ini_file);
	fputs ("\r\nInputDirectory=",dir_ini_file);
	fputs (gui_prefs.InputDirectory,dir_ini_file);
	fputs ("\r\nHiscoreDirectory=",dir_ini_file);
	fputs (gui_prefs.HiscoreDirectory,dir_ini_file);
	fputs ("\r\nStateDirectory=",dir_ini_file);
	fputs (gui_prefs.StateDirectory,dir_ini_file);
	fputs ("\r\nArtworkDirectory=",dir_ini_file);
	fputs (gui_prefs.ArtworkDirectory,dir_ini_file);
	fputs ("\r\nSnapshotDirectory=",dir_ini_file);
	fputs (gui_prefs.SnapshotDirectory,dir_ini_file);
	fputs ("\r\nFlyerDirectroy=",dir_ini_file);
	fputs (gui_prefs.FlyerDirectroy,dir_ini_file);
	fputs ("\r\nCabinetDirectory=",dir_ini_file);
	fputs (gui_prefs.CabinetDirectory,dir_ini_file);
	fputs ("\r\nMarqueeDirectory=",dir_ini_file);
	fputs (gui_prefs.MarqueeDirectory,dir_ini_file);
	fputs ("\r\nTitleDirectory=",dir_ini_file);
	fputs (gui_prefs.TitleDirectory,dir_ini_file);
	fputs ("\r\nDiffDirectory=",dir_ini_file);
	fputs (gui_prefs.DiffDirectory,dir_ini_file);
	fputs ("\r\nIconDirectory=",dir_ini_file);
	fputs (gui_prefs.IconDirectory,dir_ini_file);
	fputs ("\r\ninipath=",dir_ini_file);
	fputs (gui_prefs.inipath,dir_ini_file);
	
	fputs ("\r\n\r\n",dir_ini_file);
	fclose(dir_ini_file);
	g_free(filename);
	return TRUE;
}


/* here are the options passed to X-mame*/
gboolean load_options(gpointer rom)
{
	FILE *options_file;
	gchar *filename;
	gchar line[2000];
	gchar **option_value = NULL;
	xmame_game_options *target;
	
	if (rom==NULL)
	{
		target = &default_options;
		filename = g_strdup_printf("%s/.gxmame/options/default",getenv("HOME"));
	}
	else
	{
		/* free a previous game_option structure (first free additional options of the game) */
		if (game_options)
		{
			g_free(((xmame_game_options *)game_options)->additional_options);
			g_free(game_options);
		}
		filename = g_strdup_printf("%s/.gxmame/options/%s",getenv("HOME"),((RomEntry *)rom)->romname);
		/* if there is no file with that name, exit, will use default options */
		if(access(filename,R_OK))
		{
			game_options = NULL;
			gxmame_debug("option file for %s not found",((RomEntry *)rom)->romname);
			g_free(filename);
			return (FALSE);
		}
		/* otherwise, create a xmame_game_options struct for this game */
		game_options = g_malloc (sizeof(xmame_game_options));
		target = game_options;
	}
	
	/* default options */
/*** Video Related ***/
	target->bpp = 0;
	target->arbheight = 0;
	target->heightscale = 1;
	target->widthscale = 1;
/*	target->scale = 0.00000;*/
	target->effect = 0;
	target->autodouble = TRUE;		
	target->dirty = TRUE;
	target->scanlines = FALSE;
	target->artwork = TRUE;
	target->use_backdrops = TRUE;
	target->use_overlays = TRUE;
	target->use_bezels = TRUE;
	target->artwork_crop = FALSE;
	target->artwork_resolution = 0;
	target->frameskipper = 0;
	target->throttle = TRUE;
	target->sleepidle = FALSE;
	target->autoframeskip = TRUE;
	target->maxautoframeskip = 8;
	target->frameskip = 0;
	target->brightness = 1.00000;
	target->gamma_correction = 1.00000;
	target->norotate = FALSE;
	target->ror = FALSE;
	target->rol = FALSE;
	target->flipx = FALSE;
	target->flipy = FALSE;

/*** Vector Games Related ***/
	target->vectorres_flag = FALSE;
	strcpy (target->vectorres, "640x480");
	target->beam = 1.000000;
	target->flicker = 0.000000;
	target->antialias = TRUE;
	target->translucency = TRUE;
	target->intensity = 1.5;

/*display_opts (depend target)*/
/*** X11 Related ***/
	target->x11_mode = 0;
				
				
 /*** X11-window Related ***/
	target->cursor = TRUE;
	target->mitshm = TRUE;
	target->xvext = TRUE;
	target->vidix = FALSE;
	target->yuv = FALSE;
	target->yv12 = FALSE;
	target->xvfullscreen = FALSE;
	target->xsync = TRUE;
	target->privatecmap = FALSE;
	target->xil = TRUE;
	target->mtxil = FALSE;
	target->run_in_root_window = FALSE;
	target->root_window_id = 0;
	target->xvgeom_flag = FALSE;
	strcpy (target->geometry,"640x480");		

 /*** X11-input related ***/
 /* (X11 & GL Glide) */
	target->grabmouse = FALSE;
	target->grabkeyboard = FALSE;
	target->winkeys = FALSE;
	strcpy (target->mapkey, " ");
				

/*** OpenGL Related ***/
	target->glfullscreen = FALSE;
	target->gldblbuffer = TRUE;
	target->gltexture_size = FALSE;
	target->glforceblitmode = FALSE;
	target->glext78 = TRUE;
	target->glbilinear = TRUE;
	target->gldrawbitmap = TRUE;
	target->gldrawbitmapvec = TRUE;
	target->glcolormod = TRUE;
	target->glbeam = 1.000000;
	target->glalphablending = TRUE;
	target->glantialias = TRUE;
	target->glantialiasvec = TRUE;
	strcpy (target->gllibname, "libGL.so");
	strcpy (target->glulibname, "libGLU.so");
	target->cabview = FALSE;
	strcpy (target->cabinet, "glmamejau");
	target->glres_flag = FALSE;
	strcpy (target->glres, " ");
				

/*** SDL Related ***/
/* 	target->listmodes = 0;	*/	
	target->fullscreen = TRUE;
	target->sdl_auto_mode = TRUE;
	target->modenumber = 0;
	strcpy (target->sdlmapkey, " ");

/*** GGI Related ***/
	target->ggilinear = FALSE;
	target->ggi_force_resolution = FALSE;
	target->xres = 640;
	target->yres = 480;

/*** Svgalib Related ***/
	target->tweak = FALSE;
	target->planar = TRUE;
	target->linear = FALSE;
	target->centerx = 0;
	target->centery = 0;

/*** FX (Glide) Related ***/
	strcpy (target->resolution, "640x480");
	target->fxgkeepaspect = TRUE;

/*** Photon Related ***/
	target->render_mode = 0;

 /*** Photon-window Related ***/
	target->phcursor = TRUE;
                 
 /*** Photon-input Related ***/
/*	target->phgrabmouse = FALSE;
	target->phwinkeys = FALSE;
	strcpy (target->phmapkey, " ");
*/

 /*** Video Mode Selection Related ***/
 /*** (SVGA, xf86_dga_opts, GGI) ***/
	target->keepaspect = 1;
	target->displayaspectratio = 1.330000;
	strcpy (target->disablemode, " ");

/*** Sound Related ***/
	target->sound = TRUE;
	target->samples = TRUE;
	target->fakesound = FALSE;
	target->samplefre = 22050;
	target->bufsize = 3.000000;
	target->volume = -3;
	strcpy (target->audiodevice, " ");
	strcpy (target->mixerdevice, " ");
	strcpy (target->soundfile, "xmameout.wav");

/*** Input Related ***/
	target->joytype = 0;
	target->analogstick = FALSE;
	target->mouse = FALSE;
	target->hotrod = FALSE;
	target->hotrodse = FALSE;
	target->usbpspad = FALSE;
	target->rapidfire = FALSE;
	target->ugcicoin = FALSE;
	
	strcpy (target->keymap, " ");
  
 /*i386*/
/* find a way to put the right default device depending the arch
#if defined(__ARCH_netbsd) || defined(__ARCH_freebsd) || defined(__ARCH_openbsd)
	strcpy (target->joydevname, "/dev/joy");	
#elif defined __ARCH_linux*/
	strcpy (target->joydevname, "/dev/js");
/*#endif */
	
 /***joy_pad_opts***  *FM townpad ***/
	strcpy (target->paddevname, "/dev/pad00");
 /***joy_x11_opts***/
	strcpy (target->x11joyname, " ");
 /***Xinput***/
	strcpy (target->XInput_trackball1, " ");
	strcpy (target->XInput_trackball2, " ");
	strcpy (target->XInput_trackball3, " ");
	strcpy (target->XInput_trackball4, " ");
/* 	strcpy (target->XInput_joystick1, " "); */
/* 	strcpy (target->XInput_joystick2, " "); */
/* 	strcpy (target->XInput_joystick3, " "); */
/* 	strcpy (target->XInput_joystick4, " "); */

/*** Network Related ***/
	target->master = 2;
	strcpy (target->slave, "localhost");
	target->netmapkey = FALSE;
	target->parallelsync = FALSE;
	target->bind_flag = FALSE; 
	target->bind = 9000;
	target->statedebug = FALSE;
/*** Digital sound related ***/
	strcpy (target->dsp_plugin, " ");
/*list-dsp-plugins  */
	target->timer = FALSE;
				

/*** Sound mixer related ***/
	strcpy (target->sound_mixer_plugin, " ");
/* list-mixer-plugins */

 /***arts***/
	target->artsBufferTime = 10;
		
 /*** Alsa Sound System 0.5***/
/* list-alsa-cards */
	target->alsacard = 0;
	target->alsadevice = 0;
        	       
        /*** Alsa Sound System 0.9 ***/
/* list-alsa-cards
list-alsa-pcm	*/
	strcpy (target->alsa_pcm, "default");
	target->alsa_buffer = 250000;
        	       
        /*** QNX Audio related ***/
	target->audio_preferred = TRUE;
				

       /*** Display  Related ***/
/*	strcpy (target->display_plugin, " ");*/
/* list-display-plugins	*/
/*	target->dwidthscale = 1;
	target->dheightscale = 1;
	target->dscanlines = FALSE;
	target->display_aspect_ratio = 1.33;
	target->keep_aspect = TRUE;
*/
/* Misc */
	target->cheat = 1;
	target->keyboard_leds = 1;
	target->debug = 0;
	strcpy (target->debug_size, "640x480");
	target->log_flag = FALSE;
	strcpy (target->log, "xmame.log");
	target->cfgname_flag = FALSE;
	strcpy (target->cfgname, "");
	target->use_additional_options = FALSE;
	target->additional_options = g_strdup ("");
	
	target->skip_disclaimer = FALSE;
	target->skip_gameinfo = FALSE;
	target->bios = 0;
	
	
	options_file = fopen(filename, "r");
	if (options_file == NULL)
	{
		gxmame_debug("options file not found");
		g_free(filename);
		return (FALSE);
	}
	
	
	while (fgets(line, 500, options_file))
	{
	if (!strncmp(line,"[Video]",7))
		{
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"bpp"))
					{
						target->bpp = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"arbheight"))
					{
						target->arbheight = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"heightscale"))
					{
						target->heightscale = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"widthscale"))
					{
						target->widthscale = atoi( option_value[1]);
/*					} else if (!strcmp(option_value[0],"scale"))
					{
						target->scale = atof( option_value[1]);
*/					} else if (!strcmp(option_value[0],"effect"))
					{
						target->effect = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"autodouble"))
					{
						target->autodouble = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"dirty"))
					{
						target->dirty = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"scanlines"))
					{
						target->scanlines = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"artwork"))
					{
						target->artwork = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"use_backdrops"))
					{
						target->use_backdrops = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"use_overlays"))
					{
						target->use_overlays = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"use_bezels"))
					{
						target->use_bezels = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"artwork_crop"))
					{
						target->artwork_crop = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"artwork_resolution"))
					{
						target->artwork_resolution = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"frameskipper"))
					{
						target->frameskipper = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"throttle"))
					{
						target->throttle = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"sleepidle"))
					{
						target->sleepidle = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"autoframeskip"))
					{
						target->autoframeskip = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"maxautoframeskip"))
					{
						target->maxautoframeskip = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"frameskip"))
					{
						target->frameskip = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"brightness"))
					{
						target->brightness = atof( option_value[1]);
						/*  converted old percentage to dosmame brightness complient 0.5-2.0 range*/
						if ((target->brightness > 2) || (target->brightness < 0.5))
							target->brightness=(target->brightness)*(1.5/100)+0.5;
					} else if (!strcmp(option_value[0],"gamma_correction"))
					{
						target->gamma_correction = atof( option_value[1]);
					} else if (!strcmp(option_value[0],"norotate"))
					{
						target->norotate = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"ror"))
					{
						target->ror = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"rol"))
					{
						target->rol = atoi( option_value[1]);
					}  else if (!strcmp(option_value[0],"flipx"))
					{
						target->flipx = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"flipy"))
					{
						target->flipy = atoi( option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[Vector]",8))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"vectorres_flag"))
					{
						target->vectorres_flag = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"vectorres"))
					{
						g_snprintf(target->vectorres,10,option_value[1]);
					} else if (!strcmp(option_value[0],"beam"))
					{
						target->beam = atof( option_value[1]);
					} else if (!strcmp(option_value[0],"flicker"))
					{
						target->flicker = atof( option_value[1]);
					} else if (!strcmp(option_value[0],"antialias"))
					{
						target->antialias = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"translucency"))
					{
						target->translucency = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"intensity"))
					{
						target->intensity = atof( option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[X11]",5))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"x11_mode"))
					{
						target->x11_mode = atof( option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[X11-window]",12))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"cursor"))
					{
						target->cursor = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"mitshm"))
					{
						target->mitshm = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"xvext"))
					{
						target->xvext = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"vidix"))
					{
						target->vidix = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"yuv"))
					{
						target->yuv = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"yv12"))
					{
						target->yv12 = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"xvfullscreen"))
					{
						target->xvfullscreen = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"xsync"))
					{
						target->xsync = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"privatecmap"))
					{
						target->privatecmap = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"xil"))
					{
						target->xil = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"mtxil"))
					{
						target->mtxil = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"run_in_root_window"))
					{
						target->run_in_root_window = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"root_window_id"))
					{
						target->root_window_id = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"xvgeom_flag"))
					{
						target->xvgeom_flag = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"geometry"))
					{
						g_snprintf(target->geometry,10,option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[X11-input]",11))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"grabmouse"))
					{
						target->grabmouse = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"grabkeyboard"))
					{
						target->grabkeyboard = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"winkeys"))
					{
						target->winkeys = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"mapkey"))
					{
						g_snprintf(target->mapkey,14,option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[OpenGL]",8))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"glfullscreen"))
					{
						target->glfullscreen = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"gldblbuffer"))
					{
						target->gldblbuffer = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"gltexture_size"))
					{
						target->gltexture_size = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"glforceblitmode"))
					{
						target->glforceblitmode = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"glext78"))
					{
						target->glext78 = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"glbilinear"))
					{
						target->glbilinear = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"gldrawbitmap"))
					{
						target->gldrawbitmap = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"gldrawbitmapvec"))
					{
						target->gldrawbitmapvec = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"glcolormod"))
					{
						target->glcolormod = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"glbeam"))
					{
						target->glbeam = atof( option_value[1]);
					} else if (!strcmp(option_value[0],"glalphablending"))
					{
						target->glalphablending = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"glantialias"))
					{
						target->glantialias = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"glantialiasvec"))
					{
						target->glantialiasvec = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"gllibname"))
					{
						g_snprintf(target->gllibname,20,option_value[1]);
					} else if (!strcmp(option_value[0],"glulibname"))
					{
						g_snprintf(target->glulibname,20,option_value[1]);
					} else if (!strcmp(option_value[0],"cabview"))
					{
						target->cabview = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"cabinet"))
					{
						g_snprintf(target->cabinet,20,option_value[1]);
					} else if (!strcmp(option_value[0],"glres_flag"))
					{
						target->glres_flag = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"glres"))
					{
						g_snprintf(target->glres,10,option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[SDL]",5))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"fullscreen"))
					{
						target->fullscreen = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"sdl_auto_mode"))
					{
						target->sdl_auto_mode = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"modenumber"))
					{
						target->modenumber = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"sdlmapkey"))
					{
						g_snprintf(target->sdlmapkey,14,option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[GGI]",5))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"ggilinear"))
					{
						target->ggilinear = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"ggi_force_resolution"))
					{
						target->ggi_force_resolution = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"xres"))
					{
						target->xres = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"yres"))
					{
						target->yres = atoi( option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[Svgalib]",9))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"tweak"))
					{
						target->tweak = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"planar"))
					{
						target->planar = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"linear"))
					{
						target->linear = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"centerx"))
					{
						target->centerx = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"centery"))
					{
						target->centery = atoi( option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[FX (Glide)]",12))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"resolution"))
					{
						g_snprintf(target->resolution,10,option_value[1]);
					} else if (!strcmp(option_value[0],"fxgkeepaspect"))
					{
						target->fxgkeepaspect = atoi( option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[Photon]",8))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"render_mode"))
					{
						target->render_mode = atoi( option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[Photon-window]",15))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"phcursor"))
					{
						target->phcursor = atoi( option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}
/*		if (!strncmp(line,"[Photon-input]",14))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"phgrabmouse"))
					{
						target->phgrabmouse = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"phwinkeys"))
					{
						target->phwinkeys = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"phmapkey"))
					{
						g_snprintf(target->phmapkey,14,option_value[1]);
					} 
				g_strfreev(option_value);
				}
			}
		}*/
		if (!strncmp(line,"[Video Mode]",12))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"keepaspect"))
					{
						target->keepaspect = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"displayaspectratio"))
					{
						target->displayaspectratio = atof(option_value[1]);
					} else if (!strcmp(option_value[0],"disablemode"))
					{
						g_snprintf(target->disablemode,15,option_value[1]);
					} 
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[Sound]",7))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"sound"))
					{
						target->sound = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"samples"))
					{
						target->samples = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"fakesound"))
					{
						target->fakesound = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"samplefre"))
					{
						target->samplefre = atof( option_value[1]);
					} else if (!strcmp(option_value[0],"bufsize"))
					{
						target->bufsize = atof( option_value[1]);
					} else if (!strcmp(option_value[0],"volume"))
					{
						target->volume = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"audiodevice"))
					{
						g_snprintf(target->audiodevice,20,option_value[1]);
					} else if (!strcmp(option_value[0],"mixerdevice"))
					{
						g_snprintf(target->mixerdevice,20,option_value[1]);
					} else if (!strcmp(option_value[0],"soundfile"))
					{
						g_snprintf(target->soundfile,20,option_value[1]);
					} 
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[Input]",7))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"joytype"))
					{
						target->joytype = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"analogstick"))
					{
						target->analogstick = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"mouse"))
					{
						target->mouse = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"hotrod"))
					{
						target->hotrod = atof( option_value[1]);
					} else if (!strcmp(option_value[0],"hotrodse"))
					{
						target->hotrodse = atof( option_value[1]);
					} else if (!strcmp(option_value[0],"usbpspad"))
					{
						target->usbpspad = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"rapidfire"))
					{
						target->rapidfire = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"keymap"))
					{
						g_snprintf(target->keymap,4,option_value[1]);
					} else if (!strcmp(option_value[0],"ugcicoin"))
					{
						target->ugcicoin = atoi( option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[i386]",6))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"joydevname"))
					{
						g_snprintf(target->joydevname,20,option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[joy_pad_opts]",14))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"paddevname"))
					{
						g_snprintf(target->paddevname,20,option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[joy_x11_opts]",14))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"x11joyname"))
					{
						g_snprintf(target->x11joyname,20,option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[Xinput]",8))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"XInput_trackball1"))
					{
						g_snprintf(target->XInput_trackball1,20,option_value[1]);
					} else if (!strcmp(option_value[0],"XInput_trackball2"))
					{
						g_snprintf(target->XInput_trackball2,20,option_value[1]);
					} else if (!strcmp(option_value[0],"XInput_trackball3"))
					{
						g_snprintf(target->XInput_trackball3,20,option_value[1]);
					} else if (!strcmp(option_value[0],"XInput_trackball4"))
					{
						g_snprintf(target->XInput_trackball4,20,option_value[1]);
/*					} else if (!strcmp(option_value[0],"XInput_joystick1"))
					{
						g_snprintf(target->XInput_joystick1,20,option_value[1]);
					} else if (!strcmp(option_value[0],"XInput_joystick2"))
					{
						g_snprintf(target->XInput_joystick2,20,option_value[1]);
					} else if (!strcmp(option_value[0],"XInput_joystick3"))
					{
						g_snprintf(target->XInput_joystick3,20,option_value[1]);
					} else if (!strcmp(option_value[0],"XInput_joystick4"))
					{
						g_snprintf(target->XInput_joystick4,20,option_value[1]);
*/					}
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[Network]",9))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"network_flag"))
					{
						target->network_flag = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"master"))
					{
						target->master = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"slave"))
					{
						g_snprintf(target->slave,20,option_value[1]);
					} else if (!strcmp(option_value[0],"netmapkey"))
					{
						target->netmapkey = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"parallelsync"))
					{
						target->parallelsync = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"bind_flag"))
					{
						target->bind_flag = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"bind"))
					{
						target->bind = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"statedebug"))
					{
						target->statedebug = atoi( option_value[1]);
					} 
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[Digital sound]",15))
		{
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"dsp_plugin"))
					{
						g_snprintf(target->dsp_plugin,20,option_value[1]);
					} else if (!strcmp(option_value[0],"timer"))
					{
						target->timer = atoi( option_value[1]);
					} 
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[Sound mixer]",13))
		{
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{
					if (!strcmp(option_value[0],"sound_mixer_plugin"))
					{
						g_snprintf(target->sound_mixer_plugin,20,option_value[1]);
					} 
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[arts]",6))
		{
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"artsBufferTime"))
					{
						target->artsBufferTime = atoi( option_value[1]);
					} 
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[Alsa Sound System 0.5]",23))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"alsacard"))
					{
						target->alsacard = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"alsadevice"))
					{
						target->alsadevice = atoi( option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[Alsa Sound System 0.9]",23))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"alsa_pcm"))
					{
						g_snprintf(target->alsa_pcm,20,option_value[1]);
					} else if (!strcmp(option_value[0],"alsa_buffer"))
					{
						target->alsa_buffer = atoi( option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}
		if (!strncmp(line,"[QNX Audio]",11))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"audio_preferred"))
					{
						target->audio_preferred = atoi( option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}
/*		if (!strncmp(line,"[Display]",9))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"display_plugin"))
					{
						g_snprintf(target->display_plugin,20,option_value[1]);
					} else if (!strcmp(option_value[0],"dwidthscale"))
					{
						target->dwidthscale = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"dheightscale"))
					{
						target->dheightscale = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"dscanlines"))
					{
						target->dscanlines = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"display_aspect_ratio"))
					{
						target->display_aspect_ratio = atof( option_value[1]);
					} else if (!strcmp(option_value[0],"keep_aspect"))
					{
						target->keep_aspect = atoi( option_value[1]);
					}
				g_strfreev(option_value);
				}
			}
		}*/
		if (!strncmp(line,"[Misc]",6))
		{	
			while (fgets(line, 500, options_file))
			{
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"cheat"))
					{
						target->cheat = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"keyboard_leds"))
					{
						target->keyboard_leds = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"debug"))
					{
						target->debug = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"debug_size"))
					{
						g_snprintf(target->debug_size,10,option_value[1]);
					} else if (!strcmp(option_value[0],"log_flag"))
					{
						target->log_flag = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"log"))
					{
						g_snprintf(target->log,20,option_value[1]);
					} else if (!strcmp(option_value[0],"cfgname_flag"))
					{
						target->cfgname_flag = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"cfgname"))
					{
						g_snprintf(target->cfgname,20,option_value[1]);
					} else if (!strcmp(option_value[0],"use_additional_options"))
					{
						target->use_additional_options = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"additional_options"))
					{
						g_free(target->additional_options);
						target->additional_options = g_strdup (option_value[1]);
					} else if (!strcmp(option_value[0],"skip_disclaimer"))
					{
						target->skip_disclaimer = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"skip_gameinfo"))
					{
						target->skip_gameinfo = atoi( option_value[1]);
					} else if (!strcmp(option_value[0],"bios"))
					{
						target->bios = atoi( option_value[1]);
					}

				g_strfreev(option_value);
				}
			}
		}
	}
	
	fclose(options_file);
	g_free(filename);
	
	/* we should not allow xv vidix and DGA at the same time, priority to older (DGA) */
	if (target->x11_mode == 1)
	{
		target->xvext = FALSE;
		target->vidix = FALSE;
	} else if (target->xvext == 1)
		target->vidix = FALSE;
	/* select only one if YUV and YV12 made are both valid */
	if (target->yuv == TRUE)
		target->yv12 = FALSE;
	
	return TRUE;
}

gboolean save_options(gpointer rom)
{
	FILE *options_file=NULL;
	gchar *filename;
	gchar *option_string = NULL;
	xmame_game_options *target;
	
	if (rom==NULL)
	{
		target = &default_options;
		filename = g_strdup_printf("%s/.gxmame/options/default",getenv("HOME"));	
	}
	else
	{
		/* no game preferences to save */
		if (!game_options)
			return (TRUE);
		filename = g_strdup_printf("%s/.gxmame/options/%s",getenv("HOME"),((RomEntry *)rom)->romname);	
		target = game_options;
	}
	
	
	if ((options_file = fopen(filename, "w")) == NULL)
	{
		gxmame_debug("unable to write %s option", (rom)?(((RomEntry *)rom)->romname):"default");
		g_free(filename);
		return (FALSE);
	}
	
	option_string = g_malloc0(MAX_OPTION_STRING);
	
	fputs ("[Video]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"bpp=%i\n",target->bpp);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"arbheight=%i\n",target->arbheight);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"heightscale=%i\n",target->heightscale);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"widthscale=%i\n",target->widthscale);
	fputs (option_string,options_file);
/*	g_snprintf(option_string,MAX_OPTION_STRING,"scale=%.5f\n",target->scale);
	fputs (option_string,options_file);
*/	g_snprintf(option_string,MAX_OPTION_STRING,"effect=%i\n",target->effect);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"autodouble=%i\n",target->autodouble);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"dirty=%i\n",target->dirty);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"scanlines=%i\n",target->scanlines);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"artwork=%i\n",target->artwork);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"use_backdrops=%i\n",target->use_backdrops);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"use_overlays=%i\n",target->use_overlays);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"use_bezels=%i\n",target->use_bezels);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"artwork_crop=%i\n",target->artwork_crop);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"artwork_resolution=%i\n",target->artwork_resolution);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"frameskipper=%i\n",target->frameskipper);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"throttle=%i\n",target->throttle);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"sleepidle=%i\n",target->sleepidle);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"autoframeskip=%i\n",target->autoframeskip);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"maxautoframeskip=%i\n",target->maxautoframeskip);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"frameskip=%i\n",target->frameskip);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"brightness=%f\n",target->brightness);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"gamma_correction=%.5f\n",target->gamma_correction);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"norotate=%i\n",target->norotate);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"ror=%i\n",target->ror);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"rol=%i\n",target->rol);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"flipx=%i\n",target->flipx);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"flipy=%i\n",target->flipy);
	fputs (option_string,options_file);

	fputs ("\n[Vector]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"vectorres_flag=%i\n",target->vectorres_flag);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"vectorres=%s\n",target->vectorres);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"beam=%.5f\n",target->beam);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"flicker=%.5f\n",target->flicker);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"antialias=%i\n",target->antialias);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"translucency=%i\n",target->translucency);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"intensity=%.5f\n",target->intensity);
	fputs (option_string,options_file);
	
	fputs ("\n[X11]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"x11_mode=%i\n",target->x11_mode);
	fputs (option_string,options_file);
	
	fputs ("\n[X11-window]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"cursor=%i\n",target->cursor);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"mitshm=%i\n",target->mitshm);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"xvext=%i\n",target->xvext);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"vidix=%i\n",target->vidix);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"yuv=%i\n",target->yuv);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"yv12=%i\n",target->yv12);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"xvfullscreen=%i\n",target->xvfullscreen);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"xsync=%i\n",target->xsync);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"privatecmap=%i\n",target->privatecmap);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"xil=%i\n",target->xil);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"mtxil=%i\n",target->mtxil);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"run_in_root_window=%i\n",target->run_in_root_window);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"root_window_id=%i\n",target->root_window_id);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"xvgeom_flag=%i\n",target->xvgeom_flag);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"geometry=%s\n",target->geometry);
	fputs (option_string,options_file);
	
	fputs ("\n[X11-input]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"grabmouse=%i\n",target->grabmouse);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"grabkeyboard=%i\n",target->grabkeyboard);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"winkeys=%i\n",target->winkeys);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"mapkey=%s\n",target->mapkey);
	fputs (option_string,options_file);

	fputs ("\n[OpenGL]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"glfullscreen=%i\n",target->glfullscreen);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"gldblbuffer=%i\n",target->gldblbuffer);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"gltexture_size=%i\n",target->gltexture_size);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"glforceblitmode=%i\n",target->glforceblitmode);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"glext78=%i\n",target->glext78);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"glbilinear=%i\n",target->glbilinear);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"gldrawbitmap=%i\n",target->gldrawbitmap);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"gldrawbitmapvec=%i\n",target->gldrawbitmapvec);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"glcolormod=%i\n",target->glcolormod);
	fputs (option_string,options_file);
/*	g_snprintf(option_string,MAX_OPTION_STRING,"glbeam=%5f\n",target->glbeam);
	fputs (option_string,options_file);
*/	g_snprintf(option_string,MAX_OPTION_STRING,"glalphablending=%i\n",target->glalphablending);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"glantialias=%i\n",target->glantialias);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"glantialiasvec=%i\n",target->glantialiasvec);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"gllibname=%s\n",target->gllibname);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"glulibname=%s\n",target->glulibname);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"cabview=%i\n",target->cabview);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"cabinet=%s\n",target->cabinet);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"glres_flag=%i\n",target->glres_flag);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"glres=%s\n",target->glres);
	fputs (option_string,options_file);

	fputs ("\n[SDL]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"fullscreen=%i\n",target->fullscreen);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"sdl_auto_mode=%i\n",target->sdl_auto_mode);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"modenumber=%i\n",target->modenumber);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"sdlmapkey=%s\n",target->sdlmapkey);
	fputs (option_string,options_file);
	
	fputs ("\n[GGI]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"ggilinear=%i\n",target->ggilinear);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"ggi_force_resolution=%i\n",target->ggi_force_resolution);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"xres=%i\n",target->xres);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"yres=%i\n",target->yres);
	fputs (option_string,options_file);
	
	fputs ("\n[Svgalib]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"tweak=%i\n",target->tweak);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"planar=%i\n",target->planar);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"linear=%i\n",target->linear);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"centerx=%i\n",target->centerx);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"centery=%i\n",target->centery);
	fputs (option_string,options_file);

	fputs ("\n[FX (Glide)]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"resolution=%s\n",target->resolution);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"fxgkeepaspect=%i\n",target->fxgkeepaspect);
	fputs (option_string,options_file);
	
	fputs ("\n[Photon]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"render_mode=%i\n",target->render_mode);
	fputs (option_string,options_file);
	
	fputs ("\n[Photon-window]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"phcursor=%i\n",target->phcursor);
	fputs (option_string,options_file);
	
/*	fputs ("\n[Photon-input]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"phgrabmouse=%i\n",target->phgrabmouse);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"phwinkeys=%i\n",target->phwinkeys);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"phmapkey=%s\n",target->phmapkey);
	fputs (option_string,options_file);
*/	
	fputs ("\n[Video Mode]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"keepaspect=%i\n",target->keepaspect);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"displayaspectratio=%5f\n",target->displayaspectratio);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"disablemode=%s\n",target->disablemode);
	fputs (option_string,options_file);
	
	fputs ("\n[Sound]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"sound=%i\n",target->sound);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"samples=%i\n",target->samples);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"fakesound=%i\n",target->fakesound);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"samplefre=%i\n",target->samplefre);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"bufsize=%5f\n",target->bufsize);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"volume=%i\n",target->volume);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"audiodevice=%s\n",target->audiodevice);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"mixerdevice=%s\n",target->mixerdevice);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"soundfile=%s\n",target->soundfile);
	fputs (option_string,options_file);
	
	fputs ("\n[Input]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"joytype=%i\n",target->joytype);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"analogstick=%i\n",target->analogstick);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"mouse=%i\n",target->mouse);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"hotrod=%i\n",target->hotrod);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"hotrodse=%i\n",target->hotrodse);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"usbpspad=%i\n",target->usbpspad);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"rapidfire=%i\n",target->rapidfire);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"ugcicoin=%i\n",target->ugcicoin);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"keymap=%s\n",target->keymap);
	fputs (option_string,options_file);
		
	fputs ("\n[i386]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"joydevname=%s\n",target->joydevname);
	fputs (option_string,options_file);
	
	fputs ("\n[joy_pad_opts]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"paddevname=%s\n",target->paddevname);
	fputs (option_string,options_file);
	
	fputs ("\n[joy_x11_opts]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"x11joyname=%s\n",target->x11joyname);
	fputs (option_string,options_file);
	
	fputs ("\n[Xinput]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"XInput_trackball1=%s\n",target->XInput_trackball1);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"XInput_trackball2=%s\n",target->XInput_trackball2);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"XInput_trackball3=%s\n",target->XInput_trackball3);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"XInput_trackball4=%s\n",target->XInput_trackball4);
	fputs (option_string,options_file);
/*	g_snprintf(option_string,MAX_OPTION_STRING,"XInput_joystick1=%s\n",target->XInput_joystick1);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"XInput_joystick2=%s\n",target->XInput_joystick2);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"XInput_joystick3=%s\n",target->XInput_joystick3);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"XInput_joystick4=%s\n",target->XInput_joystick4);
	fputs (option_string,options_file);*/
	
	fputs ("\n[Network]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"network_flag=%i\n",target->network_flag);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"master=%i\n",target->master);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"slave=%s\n",target->slave);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"netmapkey=%i\n",target->netmapkey);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"parallelsync=%i\n",target->parallelsync);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"bind_flag=%i\n",target->bind_flag);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"bind=%i\n",target->bind);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"statedebug=%i\n",target->statedebug);
	fputs (option_string,options_file);
	
	fputs ("\n[Digital sound]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"dsp_plugin=%s\n",target->dsp_plugin);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"timer=%i\n",target->timer);
	fputs (option_string,options_file);

	fputs ("\n[Sound mixer]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"sound_mixer_plugin=%s\n",target->sound_mixer_plugin);
	fputs (option_string,options_file);

	fputs ("\n[arts]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"artsBufferTime=%i\n",target->artsBufferTime);
	fputs (option_string,options_file);
	
	fputs ("\n[Alsa Sound System 0.5]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"alsacard=%i\n",target->alsacard);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"alsadevice=%i\n",target->alsadevice);
	fputs (option_string,options_file);
	
	fputs ("\n[Alsa Sound System 0.9]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"alsa_pcm=%s\n",target->alsa_pcm);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"alsa_buffer=%i\n",target->alsa_buffer);
	fputs (option_string,options_file);
	
	fputs ("\n[QNX Audio]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"audio_preferred=%i\n",target->audio_preferred);
	fputs (option_string,options_file);
	
/*	fputs ("\n[Display]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"display_plugin=%s\n",target->display_plugin);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"dwidthscale=%i\n",target->dwidthscale);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"dheightscale=%i\n",target->dheightscale);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"dscanlines=%i\n",target->dscanlines);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"display_aspect_ratio=%5f\n",target->display_aspect_ratio);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"keep_aspect=%i\n",target->keep_aspect);
	fputs (option_string,options_file);
*/	
	
	fputs ("\n[Misc]\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"cheat=%i\n",target->cheat);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"keyboard_leds=%i\n",target->keyboard_leds);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"debug=%i\n",target->debug);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"debug_size=%s\n",target->debug_size);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"log_flag=%i\n",target->log_flag);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"log=%s\n",target->log);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"cfgname_flag=%i\n",target->cfgname_flag);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"cfgname=%s\n",target->cfgname);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"use_additional_options=%i\n",target->use_additional_options);
	fputs (option_string,options_file);
	/* here I'm using directly the additional option because I don't know its length */
	fputs ("additional_options=",options_file);
	fputs (target->additional_options,options_file);
	fputs ("\n",options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"skip_disclaimer=%i\n",target->skip_disclaimer);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"skip_gameinfo=%i\n",target->skip_gameinfo);
	fputs (option_string,options_file);
	g_snprintf(option_string,MAX_OPTION_STRING,"bios=%i\n",target->bios);
	fputs (option_string,options_file);
		
	fclose(options_file);
	g_free(filename);
	g_free(option_string);
	
	return TRUE;
}


void quick_check(void)
{
	static gboolean quick_check_running;
	GList *list_pointer;
	GList *list_pointer2;
	gchar *filename;
	gint nb_rom_not_checked;
	gfloat done;
	int i;
	struct stat filestatbuf;
	RomEntry *rom;
if(!quick_check_running)
{	/*prevent user to launch several quick check at the same time */
	show_progress_bar();
	quick_check_running=1;
	filename = g_malloc(200);
	nb_rom_not_checked = g_list_length(not_checked_list);
	g_message(_("Performing quick check, please wait:"));
	/* need to use the last or I can find the list anymore*/
	for (list_pointer = g_list_last(not_checked_list);(list_pointer != NULL);)
	{
		rom = (RomEntry *)list_pointer->data;
		/* Looking for roms */
		rom->has_roms = 2;
		for (i=0;gui_prefs.RomPath[i]!=NULL;i++)
		{	
			g_snprintf(filename, 200, "%s%s.zip",gui_prefs.RomPath[i],rom->romname);
		        if(!access(filename,R_OK))
		        {
/*		        	printf("rom %s found,%s\n",rom->romname,rom->gamename);*/
		        	rom->has_roms = 1;
		        }
		        else
		        {	/*test of directories*/
				g_snprintf(filename, 200,"%s%s",gui_prefs.RomPath[i],rom->romname);
				if (stat(filename,&filestatbuf)>=0) 
				{
					rom->has_roms = 1;
				}
				else
				{
		        		if(strcmp(rom->cloneof,"-"))
		        		{
/*		        			printf("rom %s not found, trying clone %s\n",rom->romname,rom->cloneof);*/
		        			g_snprintf(filename, 200, "%s%s.zip",gui_prefs.RomPath[i],rom->cloneof);
		        			if(!access(filename,R_OK))
		        			{
/*		        				printf("    clone %s found\n",rom->cloneof);*/
		        				rom->has_roms = 1;
		        			}
						else
						{
							g_snprintf(filename, 200,"%s%s",gui_prefs.RomPath[i],rom->cloneof);
							if (stat(filename,&filestatbuf)>=0)
							{
								rom->has_roms = 1;
							}
						}
		        		}
				}
		        }
			/* prevent to look into other folders if we already find the rom*/
			if(rom->has_roms == 1) break;
		}
		if(rom->has_roms == 2)
			rom->has_roms = 0;		
		/* Looking for samples */
		if (rom->nb_samples==0)
		{	/* no sample -> no need to check anything */
			rom->has_samples = 0;
		}
		else
		{
			rom->has_samples = 2;
			for (i=0;gui_prefs.SamplePath[i]!=NULL;i++)
			{
				g_snprintf(filename, 200, "%s%s.zip",gui_prefs.SamplePath[i],rom->romname);
			        if(!access(filename,R_OK))
			        {
/*			        	printf("samples %s found,%s\n",rom->romname,rom->gamename);*/
			        	rom->has_samples = 1;
			        }
			        else
			        {	/*test of directories*/
					g_snprintf(filename, 200,"%s%s",gui_prefs.SamplePath[i],rom->romname);
					if (stat(filename,&filestatbuf)>=0)
					{
						rom->has_samples = 1;
					}
					else
					{
			        		if(strcmp(rom->sampleof,"-"))
			        		{
/*			        			printf("samples %s not found, trying clone %s\n",rom->romname,rom->sampleof);*/
			        			g_snprintf(filename, 200, "%s%s.zip",gui_prefs.SamplePath[i],rom->sampleof);
			        			if(!access(filename,R_OK))
			        			{
/*			        				printf("    clone %s found\n",rom->sampleof);*/
			        				rom->has_samples = 1;
			        			}
							else
							{
								g_snprintf(filename, 200,"%s%s",gui_prefs.SamplePath[i],rom->sampleof);
								if (stat(filename,&filestatbuf)>=0)
								{
									rom->has_samples = 1;
								}
							}
			        		}
					}
			        }
				/* prevent to look into other folders if we already find the sample*/
				if(rom->has_samples == 1) break;
			}
			if(rom->has_samples == 2)
				rom->has_samples = 0;
		}
		list_pointer2=list_pointer;
		list_pointer = g_list_previous(list_pointer);
		/* is this remove the list of memory or keep a copy of pointer ?*/
		if (g_list_length(not_checked_list)>1)
			g_list_remove_link(not_checked_list,list_pointer2);
		else{/* I dont like this way to empty/free? the list*/
			list_pointer2->next=NULL;
			list_pointer2->prev=NULL;
			not_checked_list=NULL;
		}
		done = (gfloat)(supported_games - g_list_length(not_checked_list))/(gfloat)supported_games;
/*		g_message("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b%6.2f%%",done * 100 );*/
		update_progress_bar(done);		
		/* allow to refresh the display during the quick check */
		while(gtk_events_pending())
			gtk_main_iteration();
		/* if the game is in the list, update it */
		if (rom->position > -1)
			update_game_in_list(rom->position);
	}
	g_free(filename);
	quick_check_running=0;
	hide_progress_bar();
}
else gxmame_debug("Quick check already running");
}
